@@ -2,7 +2,7 @@
 #
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #  SLEPc - Scalable Library for Eigenvalue Problem Computations
-#  Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+#  Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
 #
 #  This file is part of SLEPc.
 #     
@@ -33,6 +33,7 @@ import blzpack
 import trlan  
 import lapack
 import primme
+import slepc4py
 
 if not hasattr(sys, 'version_info') or not sys.version_info[1] >= 2:
   print '**** You must have Python version 2.2 or higher to run config/configure.py ******'
@@ -71,6 +72,7 @@ trlanlibs = []
 haveprimme = 0
 primmedir = ''
 primmelibs = []
+getslepc4py = 0
 prefixdir = ''
 
 for i in sys.argv[1:]:
@@ -106,12 +108,14 @@ for i in sys.argv[1:]:
     haveprimme = 1
   elif i.startswith('--with-primme'):
     haveprimme = not i.endswith('=0')
+  elif i.startswith('--download-slepc4py'):
+    getslepc4py = not i.endswith('=0')
   elif i.startswith('--prefix='):
     prefixdir = i.split('=')[1]
   elif i.startswith('--h') or i.startswith('-h') or i.startswith('-?'):
     print 'SLEPc Configure Help'
     print '-'*80
-    print '  --prefix=<dir>                   : Specifiy location to install SLEPc (eg. /usr/local)'
+    print '  --prefix=<dir>                   : Specifiy location to install SLEPc (e.g., /usr/local)'
     print 'ARPACK:'
     print '  --with-arpack                    : Indicate if you wish to test for ARPACK (PARPACK)'
     print '  --with-arpack-dir=<dir>          : Indicate the directory for ARPACK libraries'
@@ -128,39 +132,53 @@ for i in sys.argv[1:]:
     print '  --with-primme                    : Indicate if you wish to test for PRIMME'
     print '  --with-primme-dir=<dir>          : Indicate the directory for PRIMME libraries'
     print '  --with-primme-flags=<flags>      : Indicate comma-separated flags for linking PRIMME'
+    print 'slepc4py:'
+    print '  --download-slepc4py              : Download and install slepc4py in SLEPc directory'
     sys.exit(0)
   else:
-    sys.exit('ERROR: Invalid argument ' + i +' use -h for help')
+    sys.exit('ERROR: Invalid argument ' + i +'. Use -h for help')
+
+prefixinstall = not prefixdir==''
 
 # Check if enviroment is ok
 print 'Checking environment...'
-if 'SLEPC_DIR' not in os.environ:
-  sys.exit('ERROR: SLEPC_DIR enviroment variable is not set')
-slepcdir = os.environ['SLEPC_DIR']
-if not os.path.exists(slepcdir) or not os.path.exists(os.sep.join([slepcdir,'config'])):
-  sys.exit('ERROR: SLEPC_DIR enviroment variable is not valid')
-if os.path.realpath(os.getcwd()) != os.path.realpath(slepcdir):
-  sys.exit('ERROR: SLEPC_DIR is not the current directory')
+if 'SLEPC_DIR' in os.environ:
+  slepcdir = os.environ['SLEPC_DIR']
+  if not os.path.exists(slepcdir) or not os.path.exists(os.sep.join([slepcdir,'config'])):
+    sys.exit('ERROR: SLEPC_DIR enviroment variable is not valid')
+  if os.path.realpath(os.getcwd()) != os.path.realpath(slepcdir):
+    sys.exit('ERROR: SLEPC_DIR is not the current directory')
+else:
+  slepcdir = os.getcwd();
+  if not os.path.exists(os.sep.join([slepcdir,'config'])):
+    sys.exit('ERROR: Current directory is not valid')
 
-if 'PETSC_DIR' not in os.environ:
-  sys.exit('ERROR: PETSC_DIR enviroment variable is not set')
-petscdir = os.environ['PETSC_DIR']
-if not os.path.exists(petscdir):
-  sys.exit('ERROR: PETSC_DIR enviroment variable is not valid')
+if 'PETSC_DIR' in os.environ:
+  petscdir = os.environ['PETSC_DIR']
+  if not os.path.exists(petscdir):
+    sys.exit('ERROR: PETSC_DIR enviroment variable is not valid')
+else:
+  if prefixdir:
+    petscdir = prefixdir
+    os.environ['PETSC_DIR'] = petscdir
+  else:
+    sys.exit('ERROR: PETSC_DIR enviroment variable is not set')
 
 # Check PETSc version
 petscversion.Load(petscdir)
-if petscversion.VERSION < '3.0.0':
+if petscversion.VERSION < '3.1':
   sys.exit('ERROR: This SLEPc version is not compatible with PETSc version '+petscversion.VERSION) 
+if petscversion.PATCHLEVEL < '4' and petscversion.RELEASE == '1':
+  sys.exit('ERROR: PETSc 3.1 patchlevel 4 required. Please upgrade to petsc-3.1-p4 or later') 
 
 # Check some information about PETSc configuration
 petscconf.Load(petscdir)
 if not petscconf.PRECISION in ['double','single','matsingle']:
   sys.exit('ERROR: This SLEPc version does not work with '+petscconf.PRECISION+' precision')
-if prefixdir and not petscconf.ISINSTALL:
+if prefixinstall and not petscconf.ISINSTALL:
   sys.exit('ERROR: SLEPc cannot be configured for non-source installation if PETSc is not configured in the same way.')
 
-# Create architecture directory and configuration file
+# Create architecture directory and configuration files
 archdir = os.sep.join([slepcdir,petscconf.ARCH])
 if not os.path.exists(archdir):
   try:
@@ -186,6 +204,12 @@ try:
   slepcconf.write('SLEPC_INSTALL_DIR =' + prefixdir +'\n')
 except:
   sys.exit('ERROR: cannot create configuration file in ' + confdir)
+try:
+  slepcrules = open(os.sep.join([confdir,'slepcrules']),'w')
+except:
+  sys.exit('ERROR: cannot create rules file in ' + confdir)
+if prefixinstall and os.path.isfile(os.sep.join([prefixdir,'include','slepc.h'])):
+  sys.exit('ERROR: prefix directory ' + prefixdir + ' contains files from a previous installation')
 
 # Open log file
 log.Open(os.sep.join([confdir,'configure.log']))
@@ -205,12 +229,12 @@ log.Write('='*80)
 
 # Check if PETSc is working
 log.Println('Checking PETSc installation...')
-if petscversion.VERSION > '3.0.0':
+if petscversion.VERSION > '3.1':
   log.Println('WARNING: PETSc version '+petscversion.VERSION+' is newer than SLEPc version')
 if petscversion.RELEASE != '1':
   log.Println('WARNING: using PETSc development version')
 if petscconf.ISINSTALL:
-  if petscconf.INSTALL_DIR != petscdir:
+  if os.path.realpath(petscconf.INSTALL_DIR) != os.path.realpath(petscdir):
     log.Println('WARNING: PETSC_DIR does not point to PETSc installation path')
 if not check.Link([],[],[]):
   log.Exit('ERROR: Unable to link with PETSc')
@@ -228,7 +252,13 @@ if haveprimme:
 # Check for missing LAPACK functions
 missing = lapack.Check(slepcconf)
 
+# Download and install slepc4py
+if getslepc4py:
+  slepc4py.Install()
+slepc4py.addMakeRule(slepcrules,prefixdir,prefixinstall,getslepc4py)
+
 slepcconf.close()
+slepcrules.close()
 
 log.Println('')
 log.Println('='*80)
