@@ -1,7 +1,7 @@
 #
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #  SLEPc - Scalable Library for Eigenvalue Problem Computations
-#  Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+#  Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
 #
 #  This file is part of SLEPc.
 #     
@@ -31,7 +31,7 @@ def Load(petscdir):
     PETSCVARIABLES = os.sep.join([petscdir,ARCH,'conf','petscvariables'])
   else:
     ISINSTALL = 1
-    ARCH = 'unknown'
+    ARCH = 'installed-petsc'
     PETSCVARIABLES = os.sep.join([petscdir,'conf','petscvariables'])
 
   try:
@@ -48,12 +48,6 @@ def Load(petscdir):
 	MAKE = v
       elif k == 'INSTALL_DIR':
         INSTALL_DIR = v
-      elif k == 'PETSC_ARCH_NAME':
-        ARCH = v
     f.close()
   except:
     sys.exit('ERROR: PETSc is not configured for architecture ' + ARCH)
-
-  if ISINSTALL and ARCH == 'unknown':
-    sys.exit('ERROR: PETSc architecture name is not defined')
-    
