@@ -3,7 +3,7 @@
 !
 !  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 !  SLEPc - Scalable Library for Eigenvalue Problem Computations
-!  Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+!  Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
 !
 !  This file is part of SLEPc.
 !     
@@ -34,8 +34,9 @@
 
 #define STSHELL     'shell'
 #define STSHIFT     'shift'
-#define STSINV      'sinvert'
+#define STSINVERT   'sinvert'
 #define STCAYLEY    'cayley'
 #define STFOLD      'fold'
+#define STPRECOND   'precond'
 
 #endif
