@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -23,7 +23,7 @@
 
 #if !defined(__SLEPCEPS_H)
 #define __SLEPCEPS_H
-#include "slepc.h"
+#include "slepcsys.h"
 #include "slepcst.h"
 #include "slepcip.h"
 PETSC_EXTERN_CXX_BEGIN
@@ -53,6 +53,8 @@ E*/
 #define EPSARNOLDI   "arnoldi"
 #define EPSLANCZOS   "lanczos"
 #define EPSKRYLOVSCHUR "krylovschur"
+#define EPSGD        "gd"
+#define EPSJD        "jd"
 #define EPSLAPACK    "lapack"
 /* the next ones are interfaces to external libraries */
 #define EPSARPACK    "arpack"
@@ -68,8 +70,11 @@ E*/
 
 .seealso: EPSSetProblemType(), EPSGetProblemType()
 E*/
-typedef enum { EPS_HEP=1,  EPS_GHEP,
-               EPS_NHEP,   EPS_GNHEP, EPS_PGNHEP } EPSProblemType;
+typedef enum { EPS_HEP=1,
+               EPS_GHEP,
+               EPS_NHEP,
+               EPS_GNHEP,
+               EPS_PGNHEP } EPSProblemType;
 
 /*E
     EPSExtraction - determines the type of extraction technique employed
@@ -79,17 +84,13 @@ typedef enum { EPS_HEP=1,  EPS_GHEP,
 
 .seealso: EPSSetExtraction(), EPSGetExtraction()
 E*/
-typedef enum { EPS_RITZ=1,   EPS_HARMONIC,
-               EPS_REFINED,  EPS_REFINED_HARMONIC } EPSExtraction;
-
-/*E
-    EPSClass - determines if the eigensolver is one- or two-sided
-
-    Level: intermediate
-
-.seealso: EPSSetClass(), EPSGetClass()
-E*/
-typedef enum { EPS_ONE_SIDE, EPS_TWO_SIDE } EPSClass;
+typedef enum { EPS_RITZ=1,
+               EPS_HARMONIC,
+               EPS_HARMONIC_RELATIVE,
+               EPS_HARMONIC_RIGHT,
+               EPS_HARMONIC_LARGEST,
+               EPS_REFINED,
+               EPS_REFINED_HARMONIC } EPSExtraction;
 
 /*E
     EPSWhich - determines which part of the spectrum is requested
@@ -98,9 +99,40 @@ typedef enum { EPS_ONE_SIDE, EPS_TWO_SIDE } EPSClass;
 
 .seealso: EPSSetWhichEigenpairs(), EPSGetWhichEigenpairs()
 E*/
-typedef enum { EPS_LARGEST_MAGNITUDE, EPS_SMALLEST_MAGNITUDE,
-               EPS_LARGEST_REAL,      EPS_SMALLEST_REAL,
-               EPS_LARGEST_IMAGINARY, EPS_SMALLEST_IMAGINARY } EPSWhich;
+typedef enum { EPS_LARGEST_MAGNITUDE=1,
+               EPS_SMALLEST_MAGNITUDE,
+               EPS_LARGEST_REAL,
+               EPS_SMALLEST_REAL,
+               EPS_LARGEST_IMAGINARY,
+               EPS_SMALLEST_IMAGINARY,
+               EPS_TARGET_MAGNITUDE,
+               EPS_TARGET_REAL, 
+               EPS_TARGET_IMAGINARY,
+               EPS_WHICH_USER } EPSWhich;
+
+/*E
+    EPSBalance - the type of balancing used for non-Hermitian problems
+
+    Level: intermediate
+
+.seealso: EPSSetBalance()
+E*/
+typedef enum { EPS_BALANCE_NONE=1,
+               EPS_BALANCE_ONESIDE,
+               EPS_BALANCE_TWOSIDE,
+               EPS_BALANCE_USER } EPSBalance;
+
+/*E
+    EPSConv - determines the convergence test
+
+    Level: intermediate
+
+.seealso: EPSSetConvergenceTest(), EPSSetConvergenceTestFunction()
+E*/
+typedef enum { EPS_CONV_ABS=1,
+               EPS_CONV_EIG,
+               EPS_CONV_NORM,
+               EPS_CONV_USER } EPSConv;
 
 EXTERN PetscErrorCode EPSCreate(MPI_Comm,EPS *);
 EXTERN PetscErrorCode EPSDestroy(EPS);
@@ -110,8 +142,8 @@ EXTERN PetscErrorCode EPSSetProblemType(EPS,EPSProblemType);
 EXTERN PetscErrorCode EPSGetProblemType(EPS,EPSProblemType*);
 EXTERN PetscErrorCode EPSSetExtraction(EPS,EPSExtraction);
 EXTERN PetscErrorCode EPSGetExtraction(EPS,EPSExtraction*);
-EXTERN PetscErrorCode EPSSetClass(EPS,EPSClass);
-EXTERN PetscErrorCode EPSGetClass(EPS,EPSClass*);
+EXTERN PetscErrorCode EPSSetBalance(EPS,EPSBalance,PetscInt,PetscReal);
+EXTERN PetscErrorCode EPSGetBalance(EPS,EPSBalance*,PetscInt*,PetscReal*);
 EXTERN PetscErrorCode EPSSetOperators(EPS,Mat,Mat);
 EXTERN PetscErrorCode EPSGetOperators(EPS,Mat*,Mat*);
 EXTERN PetscErrorCode EPSSetFromOptions(EPS);
@@ -119,8 +151,6 @@ EXTERN PetscErrorCode EPSSetUp(EPS);
 EXTERN PetscErrorCode EPSSolve(EPS);
 EXTERN PetscErrorCode EPSView(EPS,PetscViewer);
 
-EXTERN PetscErrorCode EPSInitializePackage(char *);
-
 EXTERN PetscErrorCode EPSSetTarget(EPS,PetscScalar);
 EXTERN PetscErrorCode EPSGetTarget(EPS,PetscScalar*);
 EXTERN PetscErrorCode EPSSetST(EPS,ST);
@@ -129,20 +159,26 @@ EXTERN PetscErrorCode EPSSetIP(EPS,IP);
 EXTERN PetscErrorCode EPSGetIP(EPS,IP*);
 EXTERN PetscErrorCode EPSSetTolerances(EPS,PetscReal,PetscInt);
 EXTERN PetscErrorCode EPSGetTolerances(EPS,PetscReal*,PetscInt*);
+EXTERN PetscErrorCode EPSSetConvergenceTestFunction(EPS,PetscErrorCode (*)(EPS,PetscScalar,PetscScalar,PetscReal,PetscReal*,void*),void*);
+EXTERN PetscErrorCode EPSSetConvergenceTest(EPS eps,EPSConv conv);
+EXTERN PetscErrorCode EPSGetConvergenceTest(EPS eps,EPSConv *conv);
+EXTERN PetscErrorCode EPSEigRelativeConverged(EPS,PetscScalar,PetscScalar,PetscReal,PetscReal*,void*);
+EXTERN PetscErrorCode EPSAbsoluteConverged(EPS,PetscScalar,PetscScalar,PetscReal,PetscReal*,void*);
+EXTERN PetscErrorCode EPSNormRelativeConverged(EPS,PetscScalar,PetscScalar,PetscReal,PetscReal*,void*);
 EXTERN PetscErrorCode EPSSetDimensions(EPS,PetscInt,PetscInt,PetscInt);
 EXTERN PetscErrorCode EPSGetDimensions(EPS,PetscInt*,PetscInt*,PetscInt*);
 
 EXTERN PetscErrorCode EPSGetConverged(EPS,PetscInt*);
 EXTERN PetscErrorCode EPSGetEigenpair(EPS,PetscInt,PetscScalar*,PetscScalar*,Vec,Vec);
-EXTERN PetscErrorCode EPSGetValue(EPS,PetscInt,PetscScalar*,PetscScalar*);
-EXTERN PetscErrorCode EPSGetRightVector(EPS,PetscInt,Vec,Vec);
-EXTERN PetscErrorCode EPSGetLeftVector(EPS,PetscInt,Vec,Vec);
+EXTERN PetscErrorCode EPSGetEigenvalue(EPS,PetscInt,PetscScalar*,PetscScalar*);
+EXTERN PetscErrorCode EPSGetEigenvector(EPS,PetscInt,Vec,Vec);
+EXTERN PetscErrorCode EPSGetEigenvectorLeft(EPS,PetscInt,Vec,Vec);
 EXTERN PetscErrorCode EPSComputeRelativeError(EPS,PetscInt,PetscReal*);
 EXTERN PetscErrorCode EPSComputeRelativeErrorLeft(EPS,PetscInt,PetscReal*);
 EXTERN PetscErrorCode EPSComputeResidualNorm(EPS,PetscInt,PetscReal*);
 EXTERN PetscErrorCode EPSComputeResidualNormLeft(EPS,PetscInt,PetscReal*);
 EXTERN PetscErrorCode EPSGetInvariantSubspace(EPS,Vec*);
-EXTERN PetscErrorCode EPSGetLeftInvariantSubspace(EPS,Vec*);
+EXTERN PetscErrorCode EPSGetInvariantSubspaceLeft(EPS,Vec*);
 EXTERN PetscErrorCode EPSGetErrorEstimate(EPS,PetscInt,PetscReal*);
 EXTERN PetscErrorCode EPSGetErrorEstimateLeft(EPS,PetscInt,PetscReal*);
 
@@ -153,20 +189,31 @@ EXTERN PetscErrorCode EPSGetMonitorContext(EPS,void **);
 EXTERN PetscErrorCode EPSGetIterationNumber(EPS,PetscInt*);
 EXTERN PetscErrorCode EPSGetOperationCounters(EPS,PetscInt*,PetscInt*,PetscInt*);
 
-EXTERN PetscErrorCode EPSSetInitialVector(EPS,Vec);
-EXTERN PetscErrorCode EPSGetInitialVector(EPS,Vec*);
-EXTERN PetscErrorCode EPSSetLeftInitialVector(EPS,Vec);
-EXTERN PetscErrorCode EPSGetLeftInitialVector(EPS,Vec*);
 EXTERN PetscErrorCode EPSSetWhichEigenpairs(EPS,EPSWhich);
 EXTERN PetscErrorCode EPSGetWhichEigenpairs(EPS,EPSWhich*);
+EXTERN PetscErrorCode EPSSetLeftVectorsWanted(EPS,PetscTruth);
+EXTERN PetscErrorCode EPSGetLeftVectorsWanted(EPS,PetscTruth*);
+EXTERN PetscErrorCode EPSSetMatrixNorms(EPS,PetscReal,PetscReal,PetscTruth);
+EXTERN PetscErrorCode EPSGetMatrixNorms(EPS,PetscReal*,PetscReal*,PetscTruth*);
+EXTERN PetscErrorCode EPSSetTrueResidual(EPS,PetscTruth);
+EXTERN PetscErrorCode EPSGetTrueResidual(EPS,PetscTruth*);
+EXTERN PetscErrorCode EPSSetEigenvalueComparison(EPS,PetscErrorCode (*func)(EPS,PetscScalar,PetscScalar,PetscScalar,PetscScalar,PetscInt*,void*),void*);
 EXTERN PetscErrorCode EPSIsGeneralized(EPS,PetscTruth*);
 EXTERN PetscErrorCode EPSIsHermitian(EPS,PetscTruth*);
 
-EXTERN PetscErrorCode EPSMonitorDefault(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,void*);
+EXTERN PetscErrorCode EPSMonitorFirst(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,void*);
+EXTERN PetscErrorCode EPSMonitorAll(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,void*);
+EXTERN PetscErrorCode EPSMonitorConverged(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,void*);
 EXTERN PetscErrorCode EPSMonitorLG(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,void*);
+EXTERN PetscErrorCode EPSMonitorLGAll(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,void*);
 
-EXTERN PetscErrorCode EPSAttachDeflationSpace(EPS,PetscInt,Vec*,PetscTruth);
+EXTERN PetscErrorCode EPSSetTrackAll(EPS,PetscTruth);
+EXTERN PetscErrorCode EPSGetTrackAll(EPS,PetscTruth*);
+
+EXTERN PetscErrorCode EPSSetDeflationSpace(EPS,PetscInt,Vec*);
 EXTERN PetscErrorCode EPSRemoveDeflationSpace(EPS);
+EXTERN PetscErrorCode EPSSetInitialSpace(EPS,PetscInt,Vec*);
+EXTERN PetscErrorCode EPSSetInitialSpaceLeft(EPS,PetscInt,Vec*);
 
 EXTERN PetscErrorCode EPSSetOptionsPrefix(EPS,const char*);
 EXTERN PetscErrorCode EPSAppendOptionsPrefix(EPS,const char*);
@@ -176,7 +223,7 @@ EXTERN PetscErrorCode EPSGetOptionsPrefix(EPS,const char*[]);
     EPSConvergedReason - reason an eigensolver was said to 
          have converged or diverged
 
-   Level: beginner
+    Level: beginner
 
 .seealso: EPSSolve(), EPSGetConvergedReason(), EPSSetTolerances()
 E*/
@@ -190,20 +237,21 @@ typedef enum {/* converged */
 
 EXTERN PetscErrorCode EPSGetConvergedReason(EPS,EPSConvergedReason *);
 
-EXTERN PetscErrorCode EPSSortEigenvalues(PetscInt,PetscScalar*,PetscScalar*,EPSWhich,PetscInt,PetscInt*);
-EXTERN PetscErrorCode EPSSortEigenvaluesReal(PetscInt,PetscReal*,EPSWhich,PetscInt,PetscInt*,PetscReal*);
+EXTERN PetscErrorCode EPSSortEigenvalues(EPS,PetscInt,PetscScalar*,PetscScalar*,PetscInt*);
+EXTERN PetscErrorCode EPSSortEigenvaluesReal(EPS,PetscInt,PetscReal*,PetscInt*);
+EXTERN PetscErrorCode EPSCompareEigenvalues(EPS,PetscScalar,PetscScalar,PetscScalar,PetscScalar,PetscInt*);
 EXTERN PetscErrorCode EPSDenseNHEP(PetscInt,PetscScalar*,PetscScalar*,PetscScalar*,PetscScalar*,PetscScalar*);
 EXTERN PetscErrorCode EPSDenseGNHEP(PetscInt,PetscScalar*,PetscScalar*,PetscScalar*,PetscScalar*,PetscScalar*,PetscScalar*);
 EXTERN PetscErrorCode EPSDenseHEP(PetscInt,PetscScalar*,PetscInt,PetscReal*,PetscScalar*);
 EXTERN PetscErrorCode EPSDenseGHEP(PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscScalar*);
 EXTERN PetscErrorCode EPSDenseHessenberg(PetscInt,PetscInt,PetscScalar*,PetscInt,PetscScalar*);
 EXTERN PetscErrorCode EPSDenseSchur(PetscInt,PetscInt,PetscScalar*,PetscInt,PetscScalar*,PetscScalar*,PetscScalar*);
-EXTERN PetscErrorCode EPSSortDenseSchur(PetscInt,PetscInt,PetscScalar*,PetscInt,PetscScalar*,PetscScalar*,PetscScalar*,EPSWhich);
-EXTERN PetscErrorCode EPSSortDenseSchurTarget(PetscInt,PetscInt,PetscScalar*,PetscInt,PetscScalar*,PetscScalar*,PetscScalar*,PetscScalar,EPSWhich);
+EXTERN PetscErrorCode EPSSortDenseSchur(EPS,PetscInt,PetscInt,PetscScalar*,PetscInt,PetscScalar*,PetscScalar*,PetscScalar*);
+EXTERN PetscErrorCode EPSSortDenseSchurGeneralized(EPS,PetscInt,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscInt,PetscScalar*,PetscScalar*,PetscScalar*,PetscScalar*);
 EXTERN PetscErrorCode EPSDenseTridiagonal(PetscInt,PetscReal*,PetscReal*,PetscReal*,PetscScalar*);
 
 EXTERN PetscErrorCode EPSGetStartVector(EPS,PetscInt,Vec,PetscTruth*);
-EXTERN PetscErrorCode EPSGetLeftStartVector(EPS,PetscInt,Vec);
+EXTERN PetscErrorCode EPSGetStartVectorLeft(EPS,PetscInt,Vec,PetscTruth*);
 
 EXTERN PetscErrorCode EPSRegister(const char*,const char*,const char*,PetscErrorCode(*)(EPS));
 #if defined(PETSC_USE_DYNAMIC_LIBRARIES)
@@ -222,8 +270,9 @@ EXTERN PetscErrorCode EPSRegisterDestroy(void);
 
 .seealso: EPSPowerSetShiftType(), EPSPowerGetShiftType()
 E*/
-typedef enum { EPSPOWER_SHIFT_CONSTANT, EPSPOWER_SHIFT_RAYLEIGH,
-               EPSPOWER_SHIFT_WILKINSON } EPSPowerShiftType;
+typedef enum { EPS_POWER_SHIFT_CONSTANT,
+               EPS_POWER_SHIFT_RAYLEIGH,
+               EPS_POWER_SHIFT_WILKINSON } EPSPowerShiftType;
 
 EXTERN PetscErrorCode EPSPowerSetShiftType(EPS,EPSPowerShiftType);
 EXTERN PetscErrorCode EPSPowerGetShiftType(EPS,EPSPowerShiftType*);
@@ -239,12 +288,12 @@ EXTERN PetscErrorCode EPSArnoldiGetDelayed(EPS,PetscTruth*);
 
 .seealso: EPSLanczosSetReorthog(), EPSLanczosGetReorthog()
 E*/
-typedef enum { EPSLANCZOS_REORTHOG_LOCAL, 
-               EPSLANCZOS_REORTHOG_FULL,
-               EPSLANCZOS_REORTHOG_SELECTIVE,
-               EPSLANCZOS_REORTHOG_PERIODIC,
-               EPSLANCZOS_REORTHOG_PARTIAL, 
-	       EPSLANCZOS_REORTHOG_DELAYED } EPSLanczosReorthogType;
+typedef enum { EPS_LANCZOS_REORTHOG_LOCAL, 
+               EPS_LANCZOS_REORTHOG_FULL,
+               EPS_LANCZOS_REORTHOG_SELECTIVE,
+               EPS_LANCZOS_REORTHOG_PERIODIC,
+               EPS_LANCZOS_REORTHOG_PARTIAL, 
+               EPS_LANCZOS_REORTHOG_DELAYED } EPSLanczosReorthogType;
 
 EXTERN PetscErrorCode EPSLanczosSetReorthog(EPS,EPSLanczosReorthogType);
 EXTERN PetscErrorCode EPSLanczosGetReorthog(EPS,EPSLanczosReorthogType*);
@@ -260,43 +309,46 @@ EXTERN PetscErrorCode EPSBlzpackSetNSteps(EPS,PetscInt);
 
 .seealso: EPSPRIMMESetMethod(), EPSPRIMMEGetMethod()
 E*/
-typedef enum {
-  EPSPRIMME_DYNAMIC,
-  EPSPRIMME_DEFAULT_MIN_TIME,
-  EPSPRIMME_DEFAULT_MIN_MATVECS,
-  EPSPRIMME_ARNOLDI,
-  EPSPRIMME_GD,
-  EPSPRIMME_GD_PLUSK,
-  EPSPRIMME_GD_OLSEN_PLUSK,
-  EPSPRIMME_JD_OLSEN_PLUSK,
-  EPSPRIMME_RQI,
-  EPSPRIMME_JDQR,
-  EPSPRIMME_JDQMR,
-  EPSPRIMME_JDQMR_ETOL,
-  EPSPRIMME_SUBSPACE_ITERATION,
-  EPSPRIMME_LOBPCG_ORTHOBASIS,
-  EPSPRIMME_LOBPCG_ORTHOBASISW
-} EPSPRIMMEMethod;
-
-/*E
-    EPSPRIMMEPrecond - determines the type of preconditioning
-    used in the PRIMME library
-
-    Level: advanced
-
-.seealso: EPSPRIMMESetPrecond(), EPSPRIMMEGetPrecond()
-E*/
-typedef enum {
-  EPSPRIMME_NONE,
-  EPSPRIMME_DIAGONAL
-} EPSPRIMMEPrecond;
+typedef enum { EPS_PRIMME_DYNAMIC,
+               EPS_PRIMME_DEFAULT_MIN_TIME,
+               EPS_PRIMME_DEFAULT_MIN_MATVECS,
+               EPS_PRIMME_ARNOLDI,
+               EPS_PRIMME_GD,
+               EPS_PRIMME_GD_PLUSK,
+               EPS_PRIMME_GD_OLSEN_PLUSK,
+               EPS_PRIMME_JD_OLSEN_PLUSK,
+               EPS_PRIMME_RQI,
+               EPS_PRIMME_JDQR,
+               EPS_PRIMME_JDQMR,
+               EPS_PRIMME_JDQMR_ETOL,
+               EPS_PRIMME_SUBSPACE_ITERATION,
+               EPS_PRIMME_LOBPCG_ORTHOBASIS,
+               EPS_PRIMME_LOBPCG_ORTHOBASISW } EPSPRIMMEMethod;
 
 EXTERN PetscErrorCode EPSPRIMMESetBlockSize(EPS eps,PetscInt bs);
 EXTERN PetscErrorCode EPSPRIMMESetMethod(EPS eps, EPSPRIMMEMethod method);
-EXTERN PetscErrorCode EPSPRIMMESetPrecond(EPS eps, EPSPRIMMEPrecond precond);
 EXTERN PetscErrorCode EPSPRIMMEGetBlockSize(EPS eps,PetscInt *bs);
 EXTERN PetscErrorCode EPSPRIMMEGetMethod(EPS eps, EPSPRIMMEMethod *method);
-EXTERN PetscErrorCode EPSPRIMMEGetPrecond(EPS eps, EPSPRIMMEPrecond *precond);
+
+EXTERN PetscErrorCode EPSGDSetKrylovStart(EPS eps,PetscTruth krylovstart);
+EXTERN PetscErrorCode EPSGDGetKrylovStart(EPS eps,PetscTruth *krylovstart);
+EXTERN PetscErrorCode EPSGDSetBlockSize(EPS eps,PetscInt blocksize);
+EXTERN PetscErrorCode EPSGDGetBlockSize(EPS eps,PetscInt *blocksize);
+EXTERN PetscErrorCode EPSGDSetRestart(EPS eps,PetscInt minv,PetscInt plusk);
+EXTERN PetscErrorCode EPSGDGetRestart(EPS eps,PetscInt *minv,PetscInt *plusk);
+EXTERN PetscErrorCode EPSGDSetInitialSize(EPS eps,PetscInt initialsize);
+EXTERN PetscErrorCode EPSGDGetInitialSize(EPS eps,PetscInt *initialsize);
+
+EXTERN PetscErrorCode EPSJDSetKrylovStart(EPS eps,PetscTruth krylovstart);
+EXTERN PetscErrorCode EPSJDGetKrylovStart(EPS eps,PetscTruth *krylovstart);
+EXTERN PetscErrorCode EPSJDSetBlockSize(EPS eps,PetscInt blocksize);
+EXTERN PetscErrorCode EPSJDGetBlockSize(EPS eps,PetscInt *blocksize);
+EXTERN PetscErrorCode EPSJDSetRestart(EPS eps,PetscInt minv,PetscInt plusk);
+EXTERN PetscErrorCode EPSJDGetRestart(EPS eps,PetscInt *minv,PetscInt *plusk);
+EXTERN PetscErrorCode EPSJDSetInitialSize(EPS eps,PetscInt initialsize);
+EXTERN PetscErrorCode EPSJDGetInitialSize(EPS eps,PetscInt *initialsize);
+EXTERN PetscErrorCode EPSJDSetFix(EPS eps,PetscReal fix);
+EXTERN PetscErrorCode EPSJDGetFix(EPS eps,PetscReal *fix);
 
 PETSC_EXTERN_CXX_END
 #endif
