@@ -4,7 +4,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -34,7 +34,6 @@ typedef struct {
 PetscErrorCode EPSSetUp_LAPACK(EPS eps)
 {
   PetscErrorCode ierr,ierra,ierrb;
-  PetscInt       N;
   EPS_LAPACK     *la = (EPS_LAPACK *)eps->data;
   PetscTruth     flg;
   Mat            A,B;
@@ -43,10 +42,13 @@ PetscErrorCode EPSSetUp_LAPACK(EPS eps)
   PC             pc;
   
   PetscFunctionBegin;
-  ierr = VecGetSize(eps->vec_initial,&N);CHKERRQ(ierr);
-  eps->ncv = N;
+  eps->ncv = eps->n;
   if (eps->mpd) PetscInfo(eps,"Warning: parameter mpd ignored\n");
 
+  if (!eps->which) eps->which = EPS_LARGEST_MAGNITUDE;
+  if (eps->balance!=EPS_BALANCE_NONE)
+    SETERRQ(PETSC_ERR_SUP,"Balancing not supported in Lapack solvers");
+
   if (la->OP) { ierr = MatDestroy(la->OP);CHKERRQ(ierr); }
   if (la->A) { ierr = MatDestroy(la->A);CHKERRQ(ierr); }
   if (la->B) { ierr = MatDestroy(la->B);CHKERRQ(ierr); }
@@ -99,7 +101,7 @@ PetscErrorCode EPSSetUp_LAPACK(EPS eps)
 PetscErrorCode EPSSolve_LAPACK(EPS eps)
 {
   PetscErrorCode ierr;
-  PetscInt       n,i,low,high;
+  PetscInt       n=eps->n,i,low,high;
   PetscMPIInt    size;
   PetscScalar    *array,*arrayb,*pV,*pW;
   PetscReal      *w;
@@ -108,15 +110,12 @@ PetscErrorCode EPSSolve_LAPACK(EPS eps)
   
   PetscFunctionBegin;
   ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
-  
-  ierr = VecGetSize(eps->vec_initial,&n);CHKERRQ(ierr);
-
   if (size == 1) {
     ierr = VecGetArray(eps->V[0],&pV);CHKERRQ(ierr);
   } else {
     ierr = PetscMalloc(sizeof(PetscScalar)*n*n,&pV);CHKERRQ(ierr);
   }
-  if (eps->solverclass == EPS_TWO_SIDE && (la->OP || !eps->ishermitian)) {
+  if (eps->leftvecs) {
     if (size == 1) {
       ierr = VecGetArray(eps->W[0],&pW);CHKERRQ(ierr);
     } else {
@@ -124,7 +123,6 @@ PetscErrorCode EPSSolve_LAPACK(EPS eps)
     }
   } else pW = PETSC_NULL;
   
-  
   if (la->OP) {
     ierr = MatGetArray(la->OP,&array);CHKERRQ(ierr);
     ierr = EPSDenseNHEP(n,array,eps->eigr,eps->eigi,pV,pW);CHKERRQ(ierr);  
@@ -224,7 +222,6 @@ PetscErrorCode EPSCreate_LAPACK(EPS eps)
   PetscLogObjectMemory(eps,sizeof(EPS_LAPACK));
   eps->data                      = (void *) la;
   eps->ops->solve                = EPSSolve_LAPACK;
-  eps->ops->solvets              = EPSSolve_LAPACK;
   eps->ops->setup                = EPSSetUp_LAPACK;
   eps->ops->destroy              = EPSDestroy_LAPACK;
   eps->ops->backtransform        = EPSBackTransform_Default;
