@@ -22,7 +22,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -43,6 +43,9 @@
 #include "private/epsimpl.h"                /*I "slepceps.h" I*/
 #include "slepcblaslapack.h"
 
+PetscErrorCode EPSSolve_POWER(EPS);
+PetscErrorCode EPSSolve_TS_POWER(EPS);
+
 typedef struct {
   EPSPowerShiftType shift_type;
 } EPS_POWER;
@@ -53,33 +56,42 @@ PetscErrorCode EPSSetUp_POWER(EPS eps)
 {
   PetscErrorCode ierr;
   EPS_POWER      *power = (EPS_POWER *)eps->data;
-  PetscInt       N;
   PetscTruth     flg;
   STMatMode      mode;
 
   PetscFunctionBegin;
-  ierr = VecGetSize(eps->vec_initial,&N);CHKERRQ(ierr);
   if (eps->ncv) {
     if (eps->ncv<eps->nev) SETERRQ(1,"The value of ncv must be at least nev"); 
   }
   else eps->ncv = eps->nev;
   if (eps->mpd) PetscInfo(eps,"Warning: parameter mpd ignored\n");
-  if (!eps->max_it) eps->max_it = PetscMax(2000,100*N);
+  if (!eps->max_it) eps->max_it = PetscMax(2000,100*eps->n);
+  if (!eps->which) eps->which = EPS_LARGEST_MAGNITUDE;
   if (eps->which!=EPS_LARGEST_MAGNITUDE)
     SETERRQ(1,"Wrong value of eps->which");
-  if (power->shift_type != EPSPOWER_SHIFT_CONSTANT) {
-    ierr = PetscTypeCompare((PetscObject)eps->OP,STSINV,&flg);CHKERRQ(ierr);
+  if (power->shift_type != EPS_POWER_SHIFT_CONSTANT) {
+    ierr = PetscTypeCompare((PetscObject)eps->OP,STSINVERT,&flg);CHKERRQ(ierr);
     if (!flg) 
       SETERRQ(PETSC_ERR_SUP,"Variable shifts only allowed in shift-and-invert ST");
     ierr = STGetMatMode(eps->OP,&mode);CHKERRQ(ierr); 
-    if (mode == STMATMODE_INPLACE)
+    if (mode == ST_MATMODE_INPLACE)
       SETERRQ(PETSC_ERR_SUP,"ST matrix mode inplace does not work with variable shifts");
   }
   if (eps->extraction) {
      ierr = PetscInfo(eps,"Warning: extraction type ignored\n");CHKERRQ(ierr);
   }
+  if (eps->balance!=EPS_BALANCE_NONE)
+    SETERRQ(PETSC_ERR_SUP,"Balancing not supported in this solver");
   ierr = EPSAllocateSolution(eps);CHKERRQ(ierr);
-  ierr = EPSDefaultGetWork(eps,3);CHKERRQ(ierr);
+  if (eps->leftvecs) {
+    ierr = EPSDefaultGetWork(eps,3);CHKERRQ(ierr);
+  } else {
+    ierr = EPSDefaultGetWork(eps,2);CHKERRQ(ierr);
+  }
+
+  /* dispatch solve method */
+  if (eps->leftvecs) eps->ops->solve = EPSSolve_TS_POWER;
+  else eps->ops->solve = EPSSolve_POWER;
   PetscFunctionReturn(0);
 }
 
@@ -89,25 +101,25 @@ PetscErrorCode EPSSolve_POWER(EPS eps)
 {
   PetscErrorCode ierr;
   EPS_POWER      *power = (EPS_POWER *)eps->data;
-  PetscInt       i, nsv;
-  Vec            v, y, e, *SV;
+  PetscInt       i;
+  Vec            v, y, e;
   Mat            A;
   PetscReal      relerr, norm, rt1, rt2, cs1, anorm;
   PetscScalar    theta, rho, delta, sigma, alpha2, beta1, sn1;
-  PetscTruth     breakdown;
+  PetscTruth     breakdown,*select = PETSC_NULL,hasnorm;
 
   PetscFunctionBegin;
   v = eps->V[0];
-  y = eps->work[2];
+  y = eps->work[1];
   e = eps->work[0];
 
   /* prepare for selective orthogonalization of converged vectors */
-  if (power->shift_type != EPSPOWER_SHIFT_CONSTANT) {
-    ierr = PetscMalloc(eps->nev*sizeof(Vec),&SV);CHKERRQ(ierr);
-    for (i=0;i<eps->nds;i++) SV[i]=eps->DS[i];
-    if (eps->nev>1) {
-      ierr = STGetOperators(eps->OP,&A,PETSC_NULL);CHKERRQ(ierr);
+  if (power->shift_type != EPS_POWER_SHIFT_CONSTANT && eps->nev>1) {
+    ierr = STGetOperators(eps->OP,&A,PETSC_NULL);CHKERRQ(ierr);
+    ierr = MatHasOperation(A,MATOP_NORM,&hasnorm);CHKERRQ(ierr);
+    if (hasnorm) {
       ierr = MatNorm(A,NORM_INFINITY,&anorm);CHKERRQ(ierr);
+      ierr = PetscMalloc(eps->nev*sizeof(PetscTruth),&select);CHKERRQ(ierr);
     }
   }
 
@@ -124,7 +136,7 @@ PetscErrorCode EPSSolve_POWER(EPS eps)
     /* theta = (v,y)_B */
     ierr = IPInnerProduct(eps->ip,v,y,&theta);CHKERRQ(ierr);
 
-    if (power->shift_type == EPSPOWER_SHIFT_CONSTANT) { /* direct & inverse iteration */
+    if (power->shift_type == EPS_POWER_SHIFT_CONSTANT) { /* direct & inverse iteration */
 
       /* approximate eigenvalue is the Rayleigh quotient */
       eps->eigr[eps->nconv] = theta;
@@ -154,7 +166,7 @@ PetscErrorCode EPSSolve_POWER(EPS eps)
         ierr = STSetShift(eps->OP,rho);CHKERRQ(ierr);
       } else {
         rho = rho + theta/(delta*delta);  /* Rayleigh quotient R(v) */
-        if (power->shift_type == EPSPOWER_SHIFT_WILKINSON) {
+        if (power->shift_type == EPS_POWER_SHIFT_WILKINSON) {
 #if defined(SLEPC_MISSING_LAPACK_LAEV2)
           SETERRQ(PETSC_ERR_SUP,"LAEV2 - Lapack routine is unavailable.");
 #else 
@@ -193,14 +205,14 @@ PetscErrorCode EPSSolve_POWER(EPS eps)
     EPSMonitor(eps,eps->its,eps->nconv,eps->eigr,eps->eigi,eps->errest,eps->nconv+1); 
 
     /* purge previously converged eigenvectors */
-    if (power->shift_type != EPSPOWER_SHIFT_CONSTANT) {
-      nsv = eps->nds;
+    if (select) {
       for (i=0;i<eps->nconv;i++) {
-        if(PetscAbsScalar(rho-eps->eigr[i])>eps->its*anorm/1000) SV[nsv++]=eps->V[i];
+        if(PetscAbsScalar(rho-eps->eigr[i])>eps->its*anorm/1000) select[i] = PETSC_TRUE;
+        else select[i] = PETSC_FALSE;
       }
-      ierr = IPOrthogonalize(eps->ip,nsv,PETSC_NULL,SV,y,PETSC_NULL,&norm,PETSC_NULL,eps->work[1],PETSC_NULL);CHKERRQ(ierr);
+      ierr = IPOrthogonalize(eps->ip,eps->nds,eps->DS,eps->nconv,select,eps->V,y,PETSC_NULL,&norm,PETSC_NULL);CHKERRQ(ierr);
     } else {
-      ierr = IPOrthogonalize(eps->ip,eps->nds+eps->nconv,PETSC_NULL,eps->DSV,y,PETSC_NULL,&norm,PETSC_NULL,eps->work[1],PETSC_NULL);CHKERRQ(ierr);
+      ierr = IPOrthogonalize(eps->ip,eps->nds,eps->DS,eps->nconv,PETSC_NULL,eps->V,y,PETSC_NULL,&norm,PETSC_NULL);CHKERRQ(ierr);
     }
 
     /* v = y/||y||_B */
@@ -224,9 +236,7 @@ PetscErrorCode EPSSolve_POWER(EPS eps)
     if (eps->its >= eps->max_it) eps->reason = EPS_DIVERGED_ITS;
   }
 
-  if (power->shift_type != EPSPOWER_SHIFT_CONSTANT) {
-    ierr = PetscFree(SV);CHKERRQ(ierr);
-  }
+  ierr = PetscFree(select);CHKERRQ(ierr);
 
   PetscFunctionReturn(0);
 }
@@ -250,7 +260,7 @@ PetscErrorCode EPSSolve_TS_POWER(EPS eps)
   z = eps->work[2];
 
   ierr = EPSGetStartVector(eps,0,v,PETSC_NULL);CHKERRQ(ierr);
-  ierr = EPSGetLeftStartVector(eps,0,w);CHKERRQ(ierr);
+  ierr = EPSGetStartVectorLeft(eps,0,w,PETSC_NULL);CHKERRQ(ierr);
   ierr = STGetShift(eps->OP,&sigma);CHKERRQ(ierr);    /* original shift */
   rho = sigma;
 
@@ -264,7 +274,7 @@ PetscErrorCode EPSSolve_TS_POWER(EPS eps)
     /* theta = (v,z)_B */
     ierr = IPInnerProduct(eps->ip,v,z,&theta);CHKERRQ(ierr);
 
-    if (power->shift_type == EPSPOWER_SHIFT_CONSTANT) { /* direct & inverse iteration */
+    if (power->shift_type == EPS_POWER_SHIFT_CONSTANT) { /* direct & inverse iteration */
 
       /* approximate eigenvalue is the Rayleigh quotient */
       eps->eigr[eps->nconv] = theta;
@@ -303,7 +313,7 @@ PetscErrorCode EPSSolve_TS_POWER(EPS eps)
         ierr = STSetShift(eps->OP,rho);CHKERRQ(ierr);
       } else {
         rho = rho + theta/(delta*delta);  /* Rayleigh quotient R(v,w) */
-        if (power->shift_type == EPSPOWER_SHIFT_WILKINSON) {
+        if (power->shift_type == EPS_POWER_SHIFT_WILKINSON) {
 #if defined(SLEPC_MISSING_LAPACK_LAEV2)
           SETERRQ(PETSC_ERR_SUP,"LAEV2 - Lapack routine is unavailable.");
 #else 
@@ -364,7 +374,7 @@ PetscErrorCode EPSSolve_TS_POWER(EPS eps)
       v = eps->V[eps->nconv];
       ierr = EPSGetStartVector(eps,eps->nconv,v,PETSC_NULL);CHKERRQ(ierr);
       w = eps->W[eps->nconv];
-      ierr = EPSGetLeftStartVector(eps,eps->nconv,w);CHKERRQ(ierr);
+      ierr = EPSGetStartVectorLeft(eps,eps->nconv,w,PETSC_NULL);CHKERRQ(ierr);
     }
   }
 
@@ -382,7 +392,7 @@ PetscErrorCode EPSBackTransform_POWER(EPS eps)
   EPS_POWER *power = (EPS_POWER *)eps->data;
 
   PetscFunctionBegin;
-  if (power->shift_type == EPSPOWER_SHIFT_CONSTANT) {
+  if (power->shift_type == EPS_POWER_SHIFT_CONSTANT) {
     ierr = EPSBackTransform_Default(eps);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
@@ -399,13 +409,13 @@ PetscErrorCode EPSSetFromOptions_POWER(EPS eps)
   const char     *shift_list[3] = { "constant", "rayleigh", "wilkinson" };
 
   PetscFunctionBegin;
-  ierr = PetscOptionsHead("POWER options");CHKERRQ(ierr);
+  ierr = PetscOptionsBegin(((PetscObject)eps)->comm,((PetscObject)eps)->prefix,"POWER Options","EPS");CHKERRQ(ierr);
   ierr = PetscOptionsEList("-eps_power_shift_type","Shift type","EPSPowerSetShiftType",shift_list,3,shift_list[power->shift_type],&i,&flg);CHKERRQ(ierr);
   if (flg ) power->shift_type = (EPSPowerShiftType)i;
-  if (power->shift_type != EPSPOWER_SHIFT_CONSTANT) {
-    ierr = STSetType(eps->OP,STSINV);CHKERRQ(ierr);
+  if (power->shift_type != EPS_POWER_SHIFT_CONSTANT) {
+    ierr = STSetType(eps->OP,STSINVERT);CHKERRQ(ierr);
   }
-  ierr = PetscOptionsTail();CHKERRQ(ierr);
+  ierr = PetscOptionsEnd();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -418,9 +428,9 @@ PetscErrorCode EPSPowerSetShiftType_POWER(EPS eps,EPSPowerShiftType shift)
 
   PetscFunctionBegin;
   switch (shift) {
-    case EPSPOWER_SHIFT_CONSTANT:
-    case EPSPOWER_SHIFT_RAYLEIGH:
-    case EPSPOWER_SHIFT_WILKINSON:
+    case EPS_POWER_SHIFT_CONSTANT:
+    case EPS_POWER_SHIFT_RAYLEIGH:
+    case EPS_POWER_SHIFT_WILKINSON:
       power->shift_type = shift;
       break;
     default:
@@ -448,12 +458,12 @@ EXTERN_C_END
                            'rayleigh' or 'wilkinson')
 
    Notes:
-   By default, shifts are constant (EPSPOWER_SHIFT_CONSTANT) and the iteration
+   By default, shifts are constant (EPS_POWER_SHIFT_CONSTANT) and the iteration
    is the simple power method (or inverse iteration if a shift-and-invert
    transformation is being used).
 
-   A variable shift can be specified (EPSPOWER_SHIFT_RAYLEIGH or
-   EPSPOWER_SHIFT_WILKINSON). In this case, the iteration behaves rather like
+   A variable shift can be specified (EPS_POWER_SHIFT_RAYLEIGH or
+   EPS_POWER_SHIFT_WILKINSON). In this case, the iteration behaves rather like
    a cubic converging method as RQI. See the users manual for details.
    
    Level: advanced
@@ -516,6 +526,20 @@ PetscErrorCode EPSPowerGetShiftType(EPS eps,EPSPowerShiftType *shift)
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__  
+#define __FUNCT__ "EPSDestroy_POWER"
+PetscErrorCode EPSDestroy_POWER(EPS eps)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
+  ierr = EPSDestroy_Default(eps);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSPowerSetShiftType_C","",PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSPowerGetShiftType_C","",PETSC_NULL);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 #undef __FUNCT__  
 #define __FUNCT__ "EPSView_POWER"
 PetscErrorCode EPSView_POWER(EPS eps,PetscViewer viewer)
@@ -528,7 +552,7 @@ PetscErrorCode EPSView_POWER(EPS eps,PetscViewer viewer)
   PetscFunctionBegin;
   ierr = PetscTypeCompare((PetscObject)viewer,PETSC_VIEWER_ASCII,&isascii);CHKERRQ(ierr);
   if (!isascii) {
-    SETERRQ1(1,"Viewer type %s not supported for EPSPOWER",((PetscObject)viewer)->type_name);
+    SETERRQ1(1,"Viewer type %s not supported for EPS_POWER",((PetscObject)viewer)->type_name);
   }  
   ierr = PetscViewerASCIIPrintf(viewer,"shift type: %s\n",shift_list[power->shift_type]);CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -546,15 +570,13 @@ PetscErrorCode EPSCreate_POWER(EPS eps)
   ierr = PetscNew(EPS_POWER,&power);CHKERRQ(ierr);
   PetscLogObjectMemory(eps,sizeof(EPS_POWER));
   eps->data                      = (void *) power;
-  eps->ops->solve                = EPSSolve_POWER;
-  eps->ops->solvets              = EPSSolve_TS_POWER;
   eps->ops->setup                = EPSSetUp_POWER;
   eps->ops->setfromoptions       = EPSSetFromOptions_POWER;
-  eps->ops->destroy              = EPSDestroy_Default;
+  eps->ops->destroy              = EPSDestroy_POWER;
   eps->ops->view                 = EPSView_POWER;
   eps->ops->backtransform        = EPSBackTransform_POWER;
   eps->ops->computevectors       = EPSComputeVectors_Default;
-  power->shift_type              = EPSPOWER_SHIFT_CONSTANT;
+  power->shift_type              = EPS_POWER_SHIFT_CONSTANT;
   ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSPowerSetShiftType_C","EPSPowerSetShiftType_POWER",EPSPowerSetShiftType_POWER);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSPowerGetShiftType_C","EPSPowerGetShiftType_POWER",EPSPowerGetShiftType_POWER);CHKERRQ(ierr);
   PetscFunctionReturn(0);
