@@ -18,7 +18,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -39,28 +39,35 @@
 #include "private/epsimpl.h"                /*I "slepceps.h" I*/
 #include "slepcblaslapack.h"
 
+PetscErrorCode EPSSolve_SUBSPACE(EPS);
+
+typedef struct {
+  Vec *AV;
+} EPS_SUBSPACE;
+
 #undef __FUNCT__  
 #define __FUNCT__ "EPSSetUp_SUBSPACE"
 PetscErrorCode EPSSetUp_SUBSPACE(EPS eps)
 {
   PetscErrorCode ierr;
-  PetscInt       i,N,nloc;
+  PetscInt       i;
+  EPS_SUBSPACE   *ctx = (EPS_SUBSPACE *)eps->data;
   PetscScalar    *pAV;
 
   PetscFunctionBegin;
-  ierr = VecGetSize(eps->vec_initial,&N);CHKERRQ(ierr);
   if (eps->ncv) { /* ncv set */
     if (eps->ncv<eps->nev) SETERRQ(1,"The value of ncv must be at least nev"); 
   }
   else if (eps->mpd) { /* mpd set */
-    eps->ncv = PetscMin(N,eps->nev+eps->mpd);
+    eps->ncv = PetscMin(eps->n,eps->nev+eps->mpd);
   }
   else { /* neither set: defaults depend on nev being small or large */
-    if (eps->nev<500) eps->ncv = PetscMin(N,PetscMax(2*eps->nev,eps->nev+15));
-    else { eps->mpd = 500; eps->ncv = PetscMin(N,eps->nev+eps->mpd); }
+    if (eps->nev<500) eps->ncv = PetscMin(eps->n,PetscMax(2*eps->nev,eps->nev+15));
+    else { eps->mpd = 500; eps->ncv = PetscMin(eps->n,eps->nev+eps->mpd); }
   }
   if (!eps->mpd) eps->mpd = eps->ncv;
-  if (!eps->max_it) eps->max_it = PetscMax(100,2*N/eps->ncv);
+  if (!eps->max_it) eps->max_it = PetscMax(100,2*eps->n/eps->ncv);
+  if (!eps->which) eps->which = EPS_LARGEST_MAGNITUDE;
   if (eps->which!=EPS_LARGEST_MAGNITUDE)
     SETERRQ(1,"Wrong value of eps->which");
   if (!eps->extraction) {
@@ -70,15 +77,18 @@ PetscErrorCode EPSSetUp_SUBSPACE(EPS eps)
   }
 
   ierr = EPSAllocateSolution(eps);CHKERRQ(ierr);
-  ierr = VecGetLocalSize(eps->vec_initial,&nloc);CHKERRQ(ierr);
-  ierr = PetscMalloc(eps->ncv*sizeof(Vec),&eps->AV);CHKERRQ(ierr);
-  ierr = PetscMalloc(eps->ncv*nloc*sizeof(PetscScalar),&pAV);CHKERRQ(ierr);
+  ierr = PetscMalloc(eps->ncv*sizeof(Vec),&ctx->AV);CHKERRQ(ierr);
+  ierr = PetscMalloc(eps->ncv*eps->nloc*sizeof(PetscScalar),&pAV);CHKERRQ(ierr);
   for (i=0;i<eps->ncv;i++) {
-    ierr = VecCreateMPIWithArray(((PetscObject)eps)->comm,nloc,PETSC_DECIDE,pAV+i*nloc,&eps->AV[i]);CHKERRQ(ierr);
+    ierr = VecCreateMPIWithArray(((PetscObject)eps)->comm,eps->nloc,PETSC_DECIDE,pAV+i*eps->nloc,&ctx->AV[i]);CHKERRQ(ierr);
   }
   ierr = PetscFree(eps->T);CHKERRQ(ierr);
   ierr = PetscMalloc(eps->ncv*eps->ncv*sizeof(PetscScalar),&eps->T);CHKERRQ(ierr);
-  ierr = EPSDefaultGetWork(eps,2);CHKERRQ(ierr);
+  ierr = EPSDefaultGetWork(eps,1);CHKERRQ(ierr);
+
+  /* dispatch solve method */
+  if (eps->leftvecs) SETERRQ(PETSC_ERR_SUP,"Left vectors not supported in this solver");
+  eps->ops->solve = EPSSolve_SUBSPACE;
   PetscFunctionReturn(0);
 }
 
@@ -192,14 +202,13 @@ static PetscErrorCode EPSSchurResidualNorms(EPS eps,Vec *V,Vec *AV,PetscScalar *
 
   PetscFunctionBegin;
   for (i=l;i<m;i++) {
-    ierr = VecSet(eps->work[0],0.0);CHKERRQ(ierr);
     if (i==m-1 || T[i+1+ldt*i]==0.0) k=i+1; else k=i+2;
-    ierr = VecMAXPY(eps->work[0],k,T+ldt*i,V);CHKERRQ(ierr);
-    ierr = VecWAXPY(eps->work[1],-1.0,eps->work[0],AV[i]);CHKERRQ(ierr);
+    ierr = VecCopy(AV[i],eps->work[0]);CHKERRQ(ierr);
+    ierr = SlepcVecMAXPBY(eps->work[0],1.0,-1.0,k,T+ldt*i,V);CHKERRQ(ierr);
 #if !defined(PETSC_USE_COMPLEX)
-    ierr = VecDot(eps->work[1],eps->work[1],rsd+i);CHKERRQ(ierr);
+    ierr = VecDot(eps->work[0],eps->work[0],rsd+i);CHKERRQ(ierr);
 #else
-    ierr = VecDot(eps->work[1],eps->work[1],&t);CHKERRQ(ierr);
+    ierr = VecDot(eps->work[0],eps->work[0],&t);CHKERRQ(ierr);
     rsd[i] = PetscRealPart(t);
 #endif    
   }
@@ -223,10 +232,11 @@ static PetscErrorCode EPSSchurResidualNorms(EPS eps,Vec *V,Vec *AV,PetscScalar *
 PetscErrorCode EPSSolve_SUBSPACE(EPS eps)
 {
   PetscErrorCode ierr;
-  PetscInt       i,ngrp,nogrp,*itrsd,*itrsdold,
+  EPS_SUBSPACE   *ctx = (EPS_SUBSPACE *)eps->data;
+  PetscInt       i,k,ngrp,nogrp,*itrsd,*itrsdold,
                  nxtsrr,idsrr,idort,nxtort,nv,ncv = eps->ncv,its;
   PetscScalar    *T=eps->T,*U;
-  PetscReal      arsd,oarsd,ctr,octr,ae,oae,*rsd,*rsdold,norm,tcond=1.0;
+  PetscReal      arsd,oarsd,ctr,octr,ae,oae,*rsd,norm,tcond=1.0;
   PetscTruth     breakdown;
   /* Parameters */
   PetscInt       init = 5;        /* Number of initial iterations */
@@ -242,18 +252,25 @@ PetscErrorCode EPSSolve_SUBSPACE(EPS eps)
   its = 0;
   ierr = PetscMalloc(sizeof(PetscScalar)*ncv*ncv,&U);CHKERRQ(ierr);
   ierr = PetscMalloc(sizeof(PetscReal)*ncv,&rsd);CHKERRQ(ierr);
-  ierr = PetscMalloc(sizeof(PetscReal)*ncv,&rsdold);CHKERRQ(ierr);
   ierr = PetscMalloc(sizeof(PetscInt)*ncv,&itrsd);CHKERRQ(ierr);
   ierr = PetscMalloc(sizeof(PetscInt)*ncv,&itrsdold);CHKERRQ(ierr);
 
-  /* Generate a set of random initial vectors and orthonormalize them */
   for (i=0;i<ncv;i++) {
-    ierr = SlepcVecSetRandom(eps->V[i]);CHKERRQ(ierr);
     rsd[i] = 0.0;
     itrsd[i] = -1;
   }
-  ierr = IPQRDecomposition(eps->ip,eps->V,0,ncv,PETSC_NULL,0,eps->work[0]);CHKERRQ(ierr);
   
+  /* Complete the initial basis with random vectors and orthonormalize them */
+  k = eps->nini;
+  while (k<ncv) {
+    ierr = SlepcVecSetRandom(eps->V[k],eps->rand);CHKERRQ(ierr);
+    ierr = IPOrthogonalize(eps->ip,eps->nds,eps->DS,k,PETSC_NULL,eps->V,eps->V[k],PETSC_NULL,&norm,&breakdown);CHKERRQ(ierr); 
+    if (norm>0.0 && !breakdown) {
+      ierr = VecScale(eps->V[k],1.0/norm);CHKERRQ(ierr);
+      k++;
+    }
+  }
+
   while (eps->its<eps->max_it) {
     eps->its++;
     nv = PetscMin(eps->nconv+eps->mpd,ncv);
@@ -266,12 +283,12 @@ PetscErrorCode EPSSolve_SUBSPACE(EPS eps)
 
     /* 1. AV(:,idx) = OP * V(:,idx) */
     for (i=eps->nconv;i<nv;i++) {
-      ierr = STApply(eps->OP,eps->V[i],eps->AV[i]);CHKERRQ(ierr);
+      ierr = STApply(eps->OP,eps->V[i],ctx->AV[i]);CHKERRQ(ierr);
     }
 
     /* 2. T(:,idx) = V' * AV(:,idx) */
     for (i=eps->nconv;i<nv;i++) {
-      ierr = VecMDot(eps->AV[i],nv,eps->V,T+i*ncv);CHKERRQ(ierr);
+      ierr = VecMDot(ctx->AV[i],nv,eps->V,T+i*ncv);CHKERRQ(ierr);
     }
 
     /* 3. Reduce projected matrix to Hessenberg form: [U,T] = hess(T) */
@@ -281,31 +298,26 @@ PetscErrorCode EPSSolve_SUBSPACE(EPS eps)
     ierr = EPSDenseSchur(nv,eps->nconv,T,ncv,U,eps->eigr,eps->eigi);CHKERRQ(ierr);
 
     /* 5. Sort diagonal elements in T and accumulate rotations on U */
-    ierr = EPSSortDenseSchur(nv,eps->nconv,T,ncv,U,eps->eigr,eps->eigi,eps->which);CHKERRQ(ierr);
+    ierr = EPSSortDenseSchur(eps,nv,eps->nconv,T,ncv,U,eps->eigr,eps->eigi);CHKERRQ(ierr);
     
     /* 6. AV(:,idx) = AV * U(:,idx) */
-    ierr = SlepcUpdateVectors(nv,eps->AV,eps->nconv,nv,U,nv,PETSC_FALSE);CHKERRQ(ierr);
+    ierr = SlepcUpdateVectors(nv,ctx->AV,eps->nconv,nv,U,nv,PETSC_FALSE);CHKERRQ(ierr);
     
     /* 7. V(:,idx) = V * U(:,idx) */
     ierr = SlepcUpdateVectors(nv,eps->V,eps->nconv,nv,U,nv,PETSC_FALSE);CHKERRQ(ierr);
     
-    /* Compute residuals */
-    for (i=0;i<nv;i++) { rsdold[i] = rsd[i]; }
-
-    ierr = EPSSchurResidualNorms(eps,eps->V,eps->AV,T,eps->nconv,nv,ncv,rsd);CHKERRQ(ierr);
-
-    for (i=0;i<nv;i++) { 
-      eps->errest[i] = rsd[i] / SlepcAbsEigenvalue(eps->eigr[i],eps->eigi[i]); 
-    }
-    EPSMonitor(eps,eps->its,eps->nconv,eps->eigr,eps->eigi,eps->errest,nv); 
-  
     /* Convergence check */
-    for (i=0;i<nv;i++) { itrsdold[i] = itrsd[i]; }
-    for (i=eps->nconv;i<nv;i++) { itrsd[i] = its; }
-    
+    ierr = EPSSchurResidualNorms(eps,eps->V,ctx->AV,T,eps->nconv,nv,ncv,rsd);CHKERRQ(ierr);
+
+    for (i=eps->nconv;i<nv;i++) { 
+      itrsdold[i] = itrsd[i];
+      itrsd[i] = its;
+      eps->errest[i] = rsd[i];
+    }
+  
     for (;;) {
       /* Find group in currently computed eigenvalues */
-      ierr = EPSFindGroup(eps->nconv,nv,eps->eigr,eps->eigi,rsd,grptol,&ngrp,&ctr,&ae,&arsd);CHKERRQ(ierr);
+      ierr = EPSFindGroup(eps->nconv,nv,eps->eigr,eps->eigi,eps->errest,grptol,&ngrp,&ctr,&ae,&arsd);CHKERRQ(ierr);
       if (ngrp!=nogrp) break;
       if (ngrp==0) break;
       if (PetscAbsScalar(ae-oae)>ctr*cnvtol*(itrsd[eps->nconv]-itrsdold[eps->nconv])) break;
@@ -314,6 +326,7 @@ PetscErrorCode EPSSolve_SUBSPACE(EPS eps)
       if (eps->nconv>=nv) break;
     }
     
+    EPSMonitor(eps,eps->its,eps->nconv,eps->eigr,eps->eigi,eps->errest,nv); 
     if (eps->nconv>=eps->nev) break;
     
     /* Compute nxtsrr (iteration of next projection step) */
@@ -335,7 +348,7 @@ PetscErrorCode EPSSolve_SUBSPACE(EPS eps)
 
     /* V(:,idx) = AV(:,idx) */
     for (i=eps->nconv;i<nv;i++) {
-      ierr = VecCopy(eps->AV[i],eps->V[i]);CHKERRQ(ierr);
+      ierr = VecCopy(ctx->AV[i],eps->V[i]);CHKERRQ(ierr);
     }
     its++;
 
@@ -345,12 +358,12 @@ PetscErrorCode EPSSolve_SUBSPACE(EPS eps)
       
         /* AV(:,idx) = OP * V(:,idx) */
         for (i=eps->nconv;i<nv;i++) {
-          ierr = STApply(eps->OP,eps->V[i],eps->AV[i]);CHKERRQ(ierr);
+          ierr = STApply(eps->OP,eps->V[i],ctx->AV[i]);CHKERRQ(ierr);
         }
         
         /* V(:,idx) = AV(:,idx) with normalization */
         for (i=eps->nconv;i<nv;i++) {
-          ierr = VecCopy(eps->AV[i],eps->V[i]);CHKERRQ(ierr);
+          ierr = VecCopy(ctx->AV[i],eps->V[i]);CHKERRQ(ierr);
           ierr = VecNorm(eps->V[i],NORM_INFINITY,&norm);CHKERRQ(ierr);
           ierr = VecScale(eps->V[i],1/norm);CHKERRQ(ierr);
         }
@@ -359,10 +372,10 @@ PetscErrorCode EPSSolve_SUBSPACE(EPS eps)
       }
       /* Orthonormalize vectors */
       for (i=eps->nconv;i<nv;i++) {
-        ierr = IPOrthogonalize(eps->ip,i+eps->nds,PETSC_NULL,eps->DSV,eps->V[i],PETSC_NULL,&norm,&breakdown,eps->work[0],PETSC_NULL);CHKERRQ(ierr);
+        ierr = IPOrthogonalize(eps->ip,eps->nds,eps->DS,i,PETSC_NULL,eps->V,eps->V[i],PETSC_NULL,&norm,&breakdown);CHKERRQ(ierr);
         if (breakdown) {
-          ierr = SlepcVecSetRandom(eps->V[i]);CHKERRQ(ierr);
-          ierr = IPOrthogonalize(eps->ip,i+eps->nds,PETSC_NULL,eps->DSV,eps->V[i],PETSC_NULL,&norm,&breakdown,eps->work[0],PETSC_NULL);CHKERRQ(ierr);
+          ierr = SlepcVecSetRandom(eps->V[i],eps->rand);CHKERRQ(ierr);
+          ierr = IPOrthogonalize(eps->ip,eps->nds,eps->DS,i,PETSC_NULL,eps->V,eps->V[i],PETSC_NULL,&norm,&breakdown);CHKERRQ(ierr);
         }
         ierr = VecScale(eps->V[i],1/norm);CHKERRQ(ierr);
       }
@@ -372,7 +385,6 @@ PetscErrorCode EPSSolve_SUBSPACE(EPS eps)
 
   ierr = PetscFree(U);CHKERRQ(ierr);
   ierr = PetscFree(rsd);CHKERRQ(ierr);
-  ierr = PetscFree(rsdold);CHKERRQ(ierr);
   ierr = PetscFree(itrsd);CHKERRQ(ierr);
   ierr = PetscFree(itrsdold);CHKERRQ(ierr);
 
@@ -389,14 +401,15 @@ PetscErrorCode EPSDestroy_SUBSPACE(EPS eps)
   PetscErrorCode ierr;
   PetscInt       i;
   PetscScalar    *pAV;
+  EPS_SUBSPACE   *ctx = (EPS_SUBSPACE *)eps->data;
 
   PetscFunctionBegin;
-  ierr = VecGetArray(eps->AV[0],&pAV);CHKERRQ(ierr);
+  ierr = VecGetArray(ctx->AV[0],&pAV);CHKERRQ(ierr);
   for (i=0;i<eps->ncv;i++) {
-    ierr = VecDestroy(eps->AV[i]);CHKERRQ(ierr);
+    ierr = VecDestroy(ctx->AV[i]);CHKERRQ(ierr);
   }
   ierr = PetscFree(pAV);CHKERRQ(ierr);
-  ierr = PetscFree(eps->AV);CHKERRQ(ierr);
+  ierr = PetscFree(ctx->AV);CHKERRQ(ierr);
   ierr = EPSDestroy_Default(eps);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -406,8 +419,13 @@ EXTERN_C_BEGIN
 #define __FUNCT__ "EPSCreate_SUBSPACE"
 PetscErrorCode EPSCreate_SUBSPACE(EPS eps)
 {
+  PetscErrorCode ierr;
+  EPS_SUBSPACE   *ctx;
+
   PetscFunctionBegin;
-  eps->ops->solve                = EPSSolve_SUBSPACE;
+  ierr = PetscNew(EPS_SUBSPACE,&ctx);CHKERRQ(ierr);
+  PetscLogObjectMemory(eps,sizeof(EPS_SUBSPACE));
+  eps->data                      = (void *) ctx;
   eps->ops->setup                = EPSSetUp_SUBSPACE;
   eps->ops->destroy              = EPSDestroy_SUBSPACE;
   eps->ops->backtransform        = EPSBackTransform_Default;
