@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -26,6 +26,25 @@
 PetscFList EPSList = 0;
 PetscCookie EPS_COOKIE = 0;
 PetscLogEvent EPS_SetUp = 0, EPS_Solve = 0, EPS_Dense = 0;
+static PetscTruth EPSPackageInitialized = PETSC_FALSE;
+
+#undef __FUNCT__  
+#define __FUNCT__ "EPSFinalizePackage"
+/*@C
+  EPSFinalizePackage - This function destroys everything in the Slepc interface to the EPS package. It is
+  called from SlepcFinalize().
+
+  Level: developer
+
+.seealso: SlepcFinalize()
+@*/
+PetscErrorCode EPSFinalizePackage(void) 
+{
+  PetscFunctionBegin;
+  EPSPackageInitialized = PETSC_FALSE;
+  EPSList               = 0;
+  PetscFunctionReturn(0);
+}
 
 #undef __FUNCT__  
 #define __FUNCT__ "EPSInitializePackage"
@@ -42,15 +61,14 @@ PetscLogEvent EPS_SetUp = 0, EPS_Solve = 0, EPS_Dense = 0;
 .seealso: SlepcInitialize()
 @*/
 PetscErrorCode EPSInitializePackage(char *path) {
-  static PetscTruth initialized = PETSC_FALSE;
   char              logList[256];
   char             *className;
   PetscTruth        opt;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (initialized) PetscFunctionReturn(0);
-  initialized = PETSC_TRUE;
+  if (EPSPackageInitialized) PetscFunctionReturn(0);
+  EPSPackageInitialized = PETSC_TRUE;
   /* Register Classes */
   ierr = PetscCookieRegister("Eigenproblem Solver",&EPS_COOKIE);CHKERRQ(ierr);
   /* Register Constructors */
@@ -75,6 +93,7 @@ PetscErrorCode EPSInitializePackage(char *path) {
       ierr = PetscLogEventDeactivateClass(EPS_COOKIE);CHKERRQ(ierr);
     }
   }
+  ierr = PetscRegisterFinalize(EPSFinalizePackage);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -110,8 +129,7 @@ PetscErrorCode EPSInitializePackage(char *path) {
 PetscErrorCode EPSView(EPS eps,PetscViewer viewer)
 {
   PetscErrorCode ierr;
-  const EPSType  type;
-  const char     *extr, *which;
+  const char     *type,*extr,*bal;
   PetscTruth     isascii;
 
   PetscFunctionBegin;
@@ -128,26 +146,20 @@ PetscErrorCode EPSView(EPS eps,PetscViewer viewer)
   ierr = PetscTypeCompare((PetscObject)viewer,PETSC_VIEWER_ASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
     ierr = PetscViewerASCIIPrintf(viewer,"EPS Object:\n");CHKERRQ(ierr);
-    switch (eps->problem_type) {
-      case EPS_HEP:   type = HERM " eigenvalue problem"; break;
-      case EPS_GHEP:  type = "generalized " HERM " eigenvalue problem"; break;
-      case EPS_NHEP:  type = "non-" HERM " eigenvalue problem"; break;
-      case EPS_GNHEP: type = "generalized non-" HERM " eigenvalue problem"; break;
-      case EPS_PGNHEP: type = "generalized non-" HERM " eigenvalue problem with " HERM " positive definite B"; break;
-      case 0:         type = "not yet set"; break;
-      default: SETERRQ(1,"Wrong value of eps->problem_type");
-    }
+    if (eps->problem_type) {
+      switch (eps->problem_type) {
+        case EPS_HEP:   type = HERM " eigenvalue problem"; break;
+        case EPS_GHEP:  type = "generalized " HERM " eigenvalue problem"; break;
+        case EPS_NHEP:  type = "non-" HERM " eigenvalue problem"; break;
+        case EPS_GNHEP: type = "generalized non-" HERM " eigenvalue problem"; break;
+        case EPS_PGNHEP: type = "generalized non-" HERM " eigenvalue problem with " HERM " positive definite B"; break;
+        default: SETERRQ(1,"Wrong value of eps->problem_type");
+      }
+    } else type = "not yet set";
     ierr = PetscViewerASCIIPrintf(viewer,"  problem type: %s\n",type);CHKERRQ(ierr);
     ierr = EPSGetType(eps,&type);CHKERRQ(ierr);
     if (type) {
-      ierr = PetscViewerASCIIPrintf(viewer,"  method: %s",type);CHKERRQ(ierr);
-      switch (eps->solverclass) {
-        case EPS_ONE_SIDE: 
-          ierr = PetscViewerASCIIPrintf(viewer,"\n",type);CHKERRQ(ierr); break;
-        case EPS_TWO_SIDE: 
-          ierr = PetscViewerASCIIPrintf(viewer," (two-sided)\n",type);CHKERRQ(ierr); break;
-        default: SETERRQ(1,"Wrong value of eps->solverclass");
-      }
+      ierr = PetscViewerASCIIPrintf(viewer,"  method: %s\n",type);CHKERRQ(ierr);
     } else {
       ierr = PetscViewerASCIIPrintf(viewer,"  method: not yet set\n");CHKERRQ(ierr);
     }
@@ -160,37 +172,116 @@ PetscErrorCode EPSView(EPS eps,PetscViewer viewer)
       switch (eps->extraction) {
         case EPS_RITZ:             extr = "Rayleigh-Ritz"; break;
         case EPS_HARMONIC:         extr = "harmonic Ritz"; break;
+        case EPS_HARMONIC_RELATIVE:extr = "relative harmonic Ritz"; break;
+        case EPS_HARMONIC_RIGHT:   extr = "right harmonic Ritz"; break;
+        case EPS_HARMONIC_LARGEST: extr = "largest harmonic Ritz"; break;
         case EPS_REFINED:          extr = "refined Ritz"; break;
         case EPS_REFINED_HARMONIC: extr = "refined harmonic Ritz"; break;
         default: SETERRQ(1,"Wrong value of eps->extraction");
       }
       ierr = PetscViewerASCIIPrintf(viewer,"  extraction type: %s\n",extr);CHKERRQ(ierr);
     }
-    ierr = PetscViewerASCIIPrintf(viewer,"  selected portion of the spectrum: ");CHKERRQ(ierr);
-    if (eps->target_set) {
-#if !defined(PETSC_USE_COMPLEX)
-      ierr = PetscViewerASCIIPrintf(viewer,"closest to target: %g\n",eps->target);CHKERRQ(ierr);
-#else
-      ierr = PetscViewerASCIIPrintf(viewer,"closest to target: %g+%g i\n",PetscRealPart(eps->target),PetscImaginaryPart(eps->target));CHKERRQ(ierr);
-#endif
-    } else {
-      switch (eps->which) {
-        case EPS_LARGEST_MAGNITUDE:  which = "largest eigenvalues in magnitude"; break;
-        case EPS_SMALLEST_MAGNITUDE: which = "smallest eigenvalues in magnitude"; break;
-        case EPS_LARGEST_REAL:       which = "largest real parts"; break;
-        case EPS_SMALLEST_REAL:      which = "smallest real parts"; break;
-        case EPS_LARGEST_IMAGINARY:  which = "largest imaginary parts"; break;
-        case EPS_SMALLEST_IMAGINARY: which = "smallest imaginary parts"; break;
-        default: SETERRQ(1,"Wrong value of eps->which");
+    if (eps->balance && !eps->ishermitian && eps->balance!=EPS_BALANCE_NONE) {
+      switch (eps->balance) {
+        case EPS_BALANCE_ONESIDE:   bal = "one-sided Krylov"; break;
+        case EPS_BALANCE_TWOSIDE:   bal = "two-sided Krylov"; break;
+        case EPS_BALANCE_USER:      bal = "user-defined matrix"; break;
+        default: SETERRQ(1,"Wrong value of eps->balance");
       }
-      ierr = PetscViewerASCIIPrintf(viewer,"%s\n",which);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  balancing enabled: %s",bal);CHKERRQ(ierr);
+      if (eps->balance==EPS_BALANCE_ONESIDE || eps->balance==EPS_BALANCE_TWOSIDE) {
+        ierr = PetscViewerASCIIPrintf(viewer,", with its=%d",eps->balance_its);CHKERRQ(ierr);
+      }
+      if (eps->balance==EPS_BALANCE_TWOSIDE && eps->balance_cutoff!=0.0) {
+        ierr = PetscViewerASCIIPrintf(viewer," and cutoff=%g",eps->balance_cutoff);CHKERRQ(ierr);
+      }
+      ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
+    }
+    ierr = PetscViewerASCIIPrintf(viewer,"  selected portion of the spectrum: ");CHKERRQ(ierr);
+    if (!eps->which) {
+      ierr = PetscViewerASCIIPrintf(viewer,"not yet set\n");CHKERRQ(ierr);
+    } else switch (eps->which) {
+      case EPS_WHICH_USER:
+        ierr = PetscViewerASCIIPrintf(viewer,"user defined\n");CHKERRQ(ierr);
+        break;
+      case EPS_TARGET_MAGNITUDE:
+#if !defined(PETSC_USE_COMPLEX)
+        ierr = PetscViewerASCIIPrintf(viewer,"closest to target: %g (in magnitude)\n",eps->target);CHKERRQ(ierr);
+#else
+        ierr = PetscViewerASCIIPrintf(viewer,"closest to target: %g+%g i (in magnitude)\n",PetscRealPart(eps->target),PetscImaginaryPart(eps->target));CHKERRQ(ierr);
+#endif
+        break;
+      case EPS_TARGET_REAL:
+#if !defined(PETSC_USE_COMPLEX)
+        ierr = PetscViewerASCIIPrintf(viewer,"closest to target: %g (along the real axis)\n",eps->target);CHKERRQ(ierr);
+#else
+        ierr = PetscViewerASCIIPrintf(viewer,"closest to target: %g+%g i (along the real axis)\n",PetscRealPart(eps->target),PetscImaginaryPart(eps->target));CHKERRQ(ierr);
+#endif
+        break;
+#if defined(PETSC_USE_COMPLEX)
+      case EPS_TARGET_IMAGINARY:
+        ierr = PetscViewerASCIIPrintf(viewer,"closest to target: %g+%g i (along the imaginary axis)\n",PetscRealPart(eps->target),PetscImaginaryPart(eps->target));CHKERRQ(ierr);
+        break;
+#endif
+      case EPS_LARGEST_MAGNITUDE:
+        ierr = PetscViewerASCIIPrintf(viewer,"largest eigenvalues in magnitude\n");CHKERRQ(ierr);
+        break;
+      case EPS_SMALLEST_MAGNITUDE:
+        ierr = PetscViewerASCIIPrintf(viewer,"smallest eigenvalues in magnitude\n");CHKERRQ(ierr);
+        break;
+      case EPS_LARGEST_REAL:
+        ierr = PetscViewerASCIIPrintf(viewer,"largest real parts\n");CHKERRQ(ierr);
+        break;
+      case EPS_SMALLEST_REAL:
+        ierr = PetscViewerASCIIPrintf(viewer,"smallest real parts\n");CHKERRQ(ierr);
+        break;
+      case EPS_LARGEST_IMAGINARY:
+        ierr = PetscViewerASCIIPrintf(viewer,"largest imaginary parts\n");CHKERRQ(ierr);
+        break;
+      case EPS_SMALLEST_IMAGINARY:
+        ierr = PetscViewerASCIIPrintf(viewer,"smallest imaginary parts\n");CHKERRQ(ierr);
+        break;
+      default: SETERRQ(1,"Wrong value of eps->which");
+    }    
+    if (eps->leftvecs) {
+      ierr = PetscViewerASCIIPrintf(viewer,"  computing left eigenvectors also\n");CHKERRQ(ierr);
+    }
+    if (eps->trueres) {
+      ierr = PetscViewerASCIIPrintf(viewer,"  computing true residuals explicitly\n");CHKERRQ(ierr);
+    }
+    if (eps->trackall) {
+      ierr = PetscViewerASCIIPrintf(viewer,"  computing all residuals (for tracking convergence)\n");CHKERRQ(ierr);
     }
     ierr = PetscViewerASCIIPrintf(viewer,"  number of eigenvalues (nev): %d\n",eps->nev);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  number of column vectors (ncv): %d\n",eps->ncv);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  maximum dimension of projected problem (mpd): %d\n",eps->mpd);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  maximum number of iterations: %d\n", eps->max_it);
     ierr = PetscViewerASCIIPrintf(viewer,"  tolerance: %g\n",eps->tol);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  dimension of user-provided deflation space: %d\n",eps->nds);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  convergence test: ");CHKERRQ(ierr);
+    switch(eps->conv) {
+    case EPS_CONV_ABS: 
+      ierr = PetscViewerASCIIPrintf(viewer,"absolute\n");CHKERRQ(ierr);break;
+    case EPS_CONV_EIG: 
+      ierr = PetscViewerASCIIPrintf(viewer,"relative to the eigenvalue\n");CHKERRQ(ierr);break;
+    case EPS_CONV_NORM: 
+      ierr = PetscViewerASCIIPrintf(viewer,"relative to the eigenvalue and matrix norms\n");CHKERRQ(ierr);break;
+    default:
+      ierr = PetscViewerASCIIPrintf(viewer,"user-defined\n");CHKERRQ(ierr);break;
+    }
+    if (eps->nini!=0) {
+      ierr = PetscViewerASCIIPrintf(viewer,"  dimension of user-provided initial space: %d\n",PetscAbs(eps->nini));CHKERRQ(ierr);
+    }
+    if (eps->ninil!=0) {
+      ierr = PetscViewerASCIIPrintf(viewer,"  dimension of user-provided initial left space: %d\n",PetscAbs(eps->ninil));CHKERRQ(ierr);
+    }
+    if (eps->nds>0) {
+      ierr = PetscViewerASCIIPrintf(viewer,"  dimension of user-provided deflation space: %d\n",eps->nds);CHKERRQ(ierr);
+    }
+    ierr = PetscViewerASCIIPrintf(viewer,"  estimates of matrix norms (%s): norm(A)=%g",eps->adaptive?"adaptive":"constant",eps->nrma);CHKERRQ(ierr);
+    if (eps->isgeneralized) {
+      ierr = PetscViewerASCIIPrintf(viewer,", norm(B)=%g",eps->nrmb);CHKERRQ(ierr);
+    }
+    ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
     ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
     ierr = IPView(eps->ip,viewer); CHKERRQ(ierr);
     ierr = STView(eps->OP,viewer); CHKERRQ(ierr);
@@ -243,24 +334,37 @@ PetscErrorCode EPSCreate(MPI_Comm comm,EPS *outeps)
   eps->ncv             = 0;
   eps->mpd             = 0;
   eps->allocated_ncv   = 0;
+  eps->nini            = 0;
+  eps->ninil           = 0;
   eps->nds             = 0;
   eps->tol             = 1e-7;
-  eps->which           = EPS_LARGEST_MAGNITUDE;
+  eps->conv            = EPS_CONV_EIG;
+  eps->conv_func       = EPSEigRelativeConverged;
+  eps->conv_ctx        = PETSC_NULL;
+  eps->which           = (EPSWhich)0;
+  eps->which_func      = PETSC_NULL;
+  eps->which_ctx       = PETSC_NULL;
+  eps->leftvecs        = PETSC_FALSE;
+  eps->trueres         = PETSC_FALSE;
+  eps->trackall        = PETSC_FALSE;
   eps->target          = 0.0;
-  eps->target_set      = PETSC_FALSE;
   eps->evecsavailable  = PETSC_FALSE;
   eps->problem_type    = (EPSProblemType)0;
   eps->extraction      = (EPSExtraction)0;
-  eps->solverclass     = (EPSClass)0;
+  eps->balance         = (EPSBalance)0;
+  eps->balance_its     = 5;
+  eps->balance_cutoff  = 1e-8;
+  eps->nrma            = 1.0;
+  eps->nrmb            = 1.0;
+  eps->adaptive        = PETSC_FALSE;
 
-  eps->vec_initial     = 0;
-  eps->vec_initial_left= 0;
   eps->V               = 0;
-  eps->AV              = 0;
   eps->W               = 0;
   eps->T               = 0;
   eps->DS              = 0;
-  eps->ds_ortho        = PETSC_TRUE;
+  eps->IS              = 0;
+  eps->ISL             = 0;
+  eps->ds_ortho        = PETSC_FALSE;
   eps->eigr            = 0;
   eps->eigi            = 0;
   eps->errest          = 0;
@@ -495,21 +599,15 @@ PetscErrorCode EPSDestroy(EPS eps)
   ierr = PetscFree(eps->T);CHKERRQ(ierr);
   ierr = PetscFree(eps->Tl);CHKERRQ(ierr);
   ierr = PetscFree(eps->perm);CHKERRQ(ierr);
-
-  if (eps->vec_initial) {
-    ierr = VecDestroy(eps->vec_initial);CHKERRQ(ierr);
+  if (eps->rand) {
+    ierr = PetscRandomDestroy(eps->rand);CHKERRQ(ierr);
   }
 
-  if (eps->vec_initial_left) {
-    ierr = VecDestroy(eps->vec_initial_left);CHKERRQ(ierr);
+  if (eps->D) {
+    ierr = VecDestroy(eps->D);CHKERRQ(ierr);
   }
 
-  if (eps->nds > 0) {
-    ierr = VecDestroyVecs(eps->DS, eps->nds);CHKERRQ(ierr);
-  }
-  
-  ierr = PetscFree(eps->DSV);CHKERRQ(ierr);
-
+  ierr = EPSRemoveDeflationSpace(eps);CHKERRQ(ierr);
   ierr = EPSMonitorCancel(eps);CHKERRQ(ierr);
 
   ierr = PetscHeaderDestroy(eps);CHKERRQ(ierr);
@@ -521,7 +619,7 @@ PetscErrorCode EPSDestroy(EPS eps)
 /*@
    EPSSetTarget - Sets the value of the target.
 
-   Not collective
+   Collective on EPS
 
    Input Parameters:
 +  eps    - eigensolver context
@@ -529,24 +627,20 @@ PetscErrorCode EPSDestroy(EPS eps)
 
    Notes:
    The target is a scalar value used to determine the portion of the spectrum
-   of interest.
-
-   If the target is not specified, then eigenvalues are computed according to
-   the which parameter (see EPSSetWhichEigenpairs()).
+   of interest. It is used in combination with EPSSetWhichEigenpairs().
    
-   If the target is specified, then the sought-after eigenvalues are those
-   closest to the target.
-
    Level: beginner
 
 .seealso: EPSGetTarget(), EPSSetWhichEigenpairs()
 @*/
 PetscErrorCode EPSSetTarget(EPS eps,PetscScalar target)
 {
+  PetscErrorCode ierr;
+
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
   eps->target = target;
-  eps->target_set = PETSC_TRUE;
+  ierr = STSetDefaultShift(eps->OP,target);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -575,8 +669,7 @@ PetscErrorCode EPSGetTarget(EPS eps,PetscScalar* target)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
   if (target) {
-    if (eps->target_set) *target = eps->target;
-    else *target = 0.0;
+    *target = eps->target;
   }
   PetscFunctionReturn(0);
 }
@@ -587,7 +680,7 @@ PetscErrorCode EPSGetTarget(EPS eps,PetscScalar* target)
    EPSSetST - Associates a spectral transformation object to the
    eigensolver. 
 
-   Collective on EPS
+   Collective on EPS and ST
 
    Input Parameters:
 +  eps - eigensolver context obtained from EPSCreate()
@@ -645,10 +738,9 @@ PetscErrorCode EPSGetST(EPS eps, ST *st)
 #undef __FUNCT__  
 #define __FUNCT__ "EPSSetIP"
 /*@
-   EPSSetIP - Associates an inner product object to the
-   eigensolver. 
+   EPSSetIP - Associates an inner product object to the eigensolver. 
 
-   Collective on EPS
+   Collective on EPS and IP
 
    Input Parameters:
 +  eps - eigensolver context obtained from EPSCreate()
