@@ -6,7 +6,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -521,24 +521,24 @@ PetscErrorCode EPSDenseSchur(PetscInt n_,PetscInt k,PetscScalar *H,PetscInt ldh_
    Not Collective
 
    Input Parameters:
-+  n     - dimension of the matrix 
++  eps - the eigensolver context
+.  n     - dimension of the matrix 
 .  k     - first active column
-.  ldt   - leading dimension of T
--  which - eigenvalue sort order
+-  ldt   - leading dimension of T
 
    Input/Output Parameters:
 +  T  - the upper (quasi-)triangular matrix
-.  Z  - the orthogonal matrix of Schur vectors
+.  Q  - the orthogonal matrix of Schur vectors
 .  wr - pointer to the array to store the computed eigenvalues
 -  wi - imaginary part of the eigenvalues (only when using real numbers)
 
    Notes:
    This function reorders the eigenvalues in wr,wi located in positions k
-   to n according to the sort order specified in which. The Schur 
-   decomposition Z*T*Z^T, is also reordered by means of rotations so that 
-   eigenvalues in the diagonal blocks of T follow the same order.
+   to n according to the sort order specified in EPSetWhicheigenpairs. 
+   The Schur decomposition Z*T*Z^T, is also reordered by means of rotations 
+   so that eigenvalues in the diagonal blocks of T follow the same order.
 
-   Both T and Z are overwritten.
+   Both T and Q are overwritten.
    
    This routine uses LAPACK routines xTREXC.
 
@@ -546,16 +546,16 @@ PetscErrorCode EPSDenseSchur(PetscInt n_,PetscInt k,PetscScalar *H,PetscInt ldh_
 
 .seealso: EPSDenseHessenberg(), EPSDenseSchur(), EPSDenseTridiagonal()
 @*/
-PetscErrorCode EPSSortDenseSchur(PetscInt n_,PetscInt k,PetscScalar *T,PetscInt ldt_,PetscScalar *Z,PetscScalar *wr,PetscScalar *wi,EPSWhich which)
+PetscErrorCode EPSSortDenseSchur(EPS eps,PetscInt n_,PetscInt k,PetscScalar *T,PetscInt ldt_,PetscScalar *Q,PetscScalar *wr,PetscScalar *wi)
 {
 #if defined(SLEPC_MISSING_LAPACK_TREXC)
   PetscFunctionBegin;
   SETERRQ(PETSC_ERR_SUP,"TREXC - Lapack routine is unavailable.");
 #else
   PetscErrorCode ierr;
-  PetscReal      value,v;
-  PetscInt       i,j;
-  PetscBLASInt   ifst,ilst,info,pos,n,ldt;
+  PetscScalar    re,im;
+  PetscInt       i,j,pos,result;
+  PetscBLASInt   ifst,ilst,info,n,ldt;
 #if !defined(PETSC_USE_COMPLEX)
   PetscScalar    *work;
 #endif
@@ -568,105 +568,58 @@ PetscErrorCode EPSSortDenseSchur(PetscInt n_,PetscInt k,PetscScalar *T,PetscInt
   ierr = PetscMalloc(n*sizeof(PetscScalar),&work);CHKERRQ(ierr);
 #endif
   
+  /* selection sort */
   for (i=k;i<n-1;i++) {
-    switch(which) {
-      case EPS_LARGEST_MAGNITUDE:
-      case EPS_SMALLEST_MAGNITUDE:
-	value = SlepcAbsEigenvalue(wr[i],wi[i]);
-	break;
-      case EPS_LARGEST_REAL:
-      case EPS_SMALLEST_REAL:
-	value = PetscRealPart(wr[i]);
-	break;
-      case EPS_LARGEST_IMAGINARY:
-      case EPS_SMALLEST_IMAGINARY:
-#if !defined(PETSC_USE_COMPLEX)
-	value = PetscAbsReal(wi[i]);
-#else
-        value = PetscImaginaryPart(wr[i]);
-#endif
-	break;
-      default: SETERRQ(1,"Wrong value of which");
-    }
+    re = wr[i];
+    im = wi[i];
     pos = 0;
-    for (j=i+1;j<n;j++) {
-      switch(which) {
-	case EPS_LARGEST_MAGNITUDE:
-	case EPS_SMALLEST_MAGNITUDE:
-	  v = SlepcAbsEigenvalue(wr[j],wi[j]);
-	  break;
-	case EPS_LARGEST_REAL:
-	case EPS_SMALLEST_REAL:
-	  v = PetscRealPart(wr[j]);
-	  break;
-	case EPS_LARGEST_IMAGINARY:
-	case EPS_SMALLEST_IMAGINARY:
+    j=i+1; /* j points to the next eigenvalue */
 #if !defined(PETSC_USE_COMPLEX)
-	  v = PetscAbsReal(wi[j]);
-#else
-          v = PetscImaginaryPart(wr[j]);
+    if (im != 0) j=i+2;
 #endif
-	  break;
-	default: SETERRQ(1,"Wrong value of which");
-      }
-      switch(which) {
-	case EPS_LARGEST_MAGNITUDE:
-	case EPS_LARGEST_REAL:
-	case EPS_LARGEST_IMAGINARY:
-	  if (v > value) {
-            value = v;
-            pos = j;
-	  }
-	  break;
-	case EPS_SMALLEST_MAGNITUDE:
-	case EPS_SMALLEST_REAL:
-	case EPS_SMALLEST_IMAGINARY:
-	  if (v < value) {
-            value = v;
-            pos = j;
-	  }
-	  break;
-	default: SETERRQ(1,"Wrong value of which");
+    /* find minimum eigenvalue */
+    for (;j<n;j++) { 
+      ierr = EPSCompareEigenvalues(eps,re,im,wr[j],wi[j],&result);CHKERRQ(ierr);
+      if (result > 0) {
+        re = wr[j];
+        im = wi[j];
+        pos = j;
       }
 #if !defined(PETSC_USE_COMPLEX)
       if (wi[j] != 0) j++;
 #endif
     }
     if (pos) {
+      /* interchange blocks */
       ifst = PetscBLASIntCast(pos + 1);
       ilst = PetscBLASIntCast(i + 1);
 #if !defined(PETSC_USE_COMPLEX)
-      LAPACKtrexc_("V",&n,T,&ldt,Z,&n,&ifst,&ilst,work,&info);
+      LAPACKtrexc_("V",&n,T,&ldt,Q,&n,&ifst,&ilst,work,&info);
 #else
-      LAPACKtrexc_("V",&n,T,&ldt,Z,&n,&ifst,&ilst,&info);
+      LAPACKtrexc_("V",&n,T,&ldt,Q,&n,&ifst,&ilst,&info);
 #endif
       if (info) SETERRQ1(PETSC_ERR_LIB,"Error in Lapack xTREXC %d",info);
-      
-      for (j=k;j<n;j++) {
+      /* recover original eigenvalues from T matrix */
+      for (j=i;j<n;j++) {
+        wr[j] = T[j*ldt+j];
 #if !defined(PETSC_USE_COMPLEX)
-        if (j==n-1 || T[j*ldt+j+1] == 0.0) { 
-          /* real eigenvalue */
-          wr[j] = T[j*ldt+j];
-          wi[j] = 0.0;
-        } else {
-          /* complex eigenvalue */
-          wr[j] = T[j*ldt+j];
-          wr[j+1] = T[j*ldt+j];
+        if (j<n-1 && T[j*ldt+j+1] != 0.0) {
+          /* complex conjugate eigenvalue */
           wi[j] = sqrt(PetscAbsReal(T[j*ldt+j+1])) *
                   sqrt(PetscAbsReal(T[(j+1)*ldt+j]));
+          wr[j+1] = wr[j];
           wi[j+1] = -wi[j];
           j++;
-        }
-#else
-        wr[j] = T[j*(ldt+1)];
+        } else
 #endif
+        wi[j] = 0.0;
       }
     }
 #if !defined(PETSC_USE_COMPLEX)
     if (wi[i] != 0) i++;
 #endif
   }
-  
+
 #if !defined(PETSC_USE_COMPLEX)
   ierr = PetscFree(work);CHKERRQ(ierr);
 #endif
@@ -677,54 +630,54 @@ PetscErrorCode EPSSortDenseSchur(PetscInt n_,PetscInt k,PetscScalar *T,PetscInt
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "EPSSortDenseSchurTarget"
+#define __FUNCT__ "EPSSortDenseSchurGeneralized"
 /*@
-   EPSSortDenseSchurTarget - Reorders the Schur decomposition computed by
-   EPSDenseSchur().
+   EPSSortDenseSchurGeneralized - Reorders a generalized Schur decomposition.
 
    Not Collective
 
    Input Parameters:
-+  n     - dimension of the matrix 
-.  k     - first active column
-.  ldt   - leading dimension of T
-.  target - the target value
--  which - eigenvalue sort order
++  eps   - the eigensolver context
+.  n     - dimension of the matrix 
+.  k0    - first active column
+.  k1    - last column to be ordered
+-  ldt   - leading dimension of T
 
    Input/Output Parameters:
-+  T  - the upper (quasi-)triangular matrix
-.  Z  - the orthogonal matrix of Schur vectors
++  T,S  - the upper (quasi-)triangular matrices
+.  Q,Z  - the orthogonal matrix of Schur vectors
 .  wr - pointer to the array to store the computed eigenvalues
 -  wi - imaginary part of the eigenvalues (only when using real numbers)
 
    Notes:
-   This function reorders the eigenvalues in wr,wi located in positions k
-   to n according to increasing distance to the target. The parameter which
-   is used to determine if distance is relative to magnitude, real axis,
-   or imaginary axis. The Schur decomposition Z*T*Z^T, is also reordered 
-   by means of rotations so that eigenvalues in the diagonal blocks of T 
-   follow the same order.
+   This function reorders the eigenvalues in wr,wi located in positions k0
+   to n according to the sort order specified in EPSetWhicheigenpairs. 
+   The selection sort is the method used to sort the eigenvalues, and it
+   stops when the column k1-1 is ordered. The Schur decomposition Z*T*Z^T,
+   is also reordered by means of rotations so that eigenvalues in the
+   diagonal blocks of T follow the same order.
 
-   Both T and Z are overwritten.
+   T,S,Q and Z are overwritten.
    
-   This routine uses LAPACK routines xTREXC.
+   This routine uses LAPACK routines xTGEXC.
 
    Level: developer
 
-.seealso: EPSDenseHessenberg(), EPSDenseSchur(), EPSDenseTridiagonal()
+.seealso:  EPSSortDenseSchur(), EPSDenseHessenberg(), EPSDenseSchur(), EPSDenseTridiagonal()
 @*/
-PetscErrorCode EPSSortDenseSchurTarget(PetscInt n_,PetscInt k,PetscScalar *T,PetscInt ldt_,PetscScalar *Z,PetscScalar *wr,PetscScalar *wi,PetscScalar target,EPSWhich which)
+PetscErrorCode EPSSortDenseSchurGeneralized(EPS eps,PetscInt n_,PetscInt k0,PetscInt k1,PetscScalar *T,PetscScalar *S,PetscInt ldt_,PetscScalar *Q,PetscScalar *Z,PetscScalar *wr,PetscScalar *wi)
 {
-#if defined(SLEPC_MISSING_LAPACK_TREXC)
+#if defined(SLEPC_MISSING_LAPACK_TGEXC) || !defined(PETSC_USE_COMPLEX) && (defined(SLEPC_MISSING_LAPACK_LAMCH) || defined(SLEPC_MISSING_LAPACK_LAG2))
   PetscFunctionBegin;
-  SETERRQ(PETSC_ERR_SUP,"TREXC - Lapack routine is unavailable.");
+  SETERRQ(PETSC_ERR_SUP,"TGEXC/LAMCH/LAG2 - Lapack routines are unavailable.");
 #else
   PetscErrorCode ierr;
-  PetscReal      value,v;
-  PetscInt       i,j;
-  PetscBLASInt   ifst,ilst,info,pos,n,ldt;
+  PetscScalar    re,im;
+  PetscInt       i,j,result,pos;
+  PetscBLASInt   ione = 1,ifst,ilst,info,n,ldt;
 #if !defined(PETSC_USE_COMPLEX)
-  PetscScalar    *work;
+  PetscBLASInt   lwork;
+  PetscScalar    *work,safmin,scale1,scale2,tmp;
 #endif
   
   PetscFunctionBegin;
@@ -732,50 +685,29 @@ PetscErrorCode EPSSortDenseSchurTarget(PetscInt n_,PetscInt k,PetscScalar *T,Pet
   n = PetscBLASIntCast(n_);
   ldt = PetscBLASIntCast(ldt_);
 #if !defined(PETSC_USE_COMPLEX)
-  ierr = PetscMalloc(n*sizeof(PetscScalar),&work);CHKERRQ(ierr);
+  lwork = -1;
+  LAPACKtgexc_(&ione,&ione,&n,T,&ldt,S,&ldt,Q,&n,Z,&n,&ione,&ione,&tmp,&lwork,&info);
+  if (info) SETERRQ1(PETSC_ERR_LIB,"Error in Lapack xTGEXC %d",info);
+  lwork = (PetscBLASInt)tmp;
+  safmin = LAPACKlamch_("S");
+  ierr = PetscMalloc(lwork*sizeof(PetscScalar),&work);CHKERRQ(ierr);
 #endif
   
-  for (i=k;i<n-1;i++) {
-    switch(which) {
-      case EPS_LARGEST_MAGNITUDE:
-        /* complex target only allowed if scalartype=complex */
-        value = SlepcAbsEigenvalue(wr[i]-target,wi[i]);
-        break;
-      case EPS_LARGEST_REAL:
-        value = PetscAbsReal(PetscRealPart(wr[i]-target));
-        break;
-      case EPS_LARGEST_IMAGINARY:
-#if !defined(PETSC_USE_COMPLEX)
-        /* complex target only allowed if scalartype=complex */
-        value = PetscAbsReal(wi[i]);
-#else
-        value = PetscAbsReal(PetscImaginaryPart(wr[i]-target));
-#endif
-        break;
-      default: SETERRQ(1,"Wrong value of which");
-    }
+  /* selection sort */
+  for (i=k0;i<PetscMin(n-1,k1);i++) {
+    re = wr[i];
+    im = wi[i];
     pos = 0;
-    for (j=i+1;j<n;j++) {
-      switch(which) {
-        case EPS_LARGEST_MAGNITUDE:
-          /* complex target only allowed if scalartype=complex */
-          v = SlepcAbsEigenvalue(wr[j]-target,wi[j]);
-          break;
-        case EPS_LARGEST_REAL:
-          v = PetscAbsReal(PetscRealPart(wr[j]-target));
-          break;
-        case EPS_LARGEST_IMAGINARY:
+    j=i+1; /* j points to the next eigenvalue */
 #if !defined(PETSC_USE_COMPLEX)
-          /* complex target only allowed if scalartype=complex */
-          v = PetscAbsReal(wi[j]);
-#else
-          v = PetscAbsReal(PetscImaginaryPart(wr[j]-target));
+    if (im != 0) j=i+2;
 #endif
-          break;
-        default: SETERRQ(1,"Wrong value of which");
-      }
-      if (v < value) {
-        value = v;
+    /* find minimum eigenvalue */
+    for (;j<n;j++) { 
+      ierr = EPSCompareEigenvalues(eps,re,im,wr[j],wi[j],&result);CHKERRQ(ierr);
+      if (result > 0) {
+        re = wr[j];
+        im = wi[j];
         pos = j;
       }
 #if !defined(PETSC_USE_COMPLEX)
@@ -783,40 +715,42 @@ PetscErrorCode EPSSortDenseSchurTarget(PetscInt n_,PetscInt k,PetscScalar *T,Pet
 #endif
     }
     if (pos) {
+      /* interchange blocks */
       ifst = PetscBLASIntCast(pos + 1);
       ilst = PetscBLASIntCast(i + 1);
 #if !defined(PETSC_USE_COMPLEX)
-      LAPACKtrexc_("V",&n,T,&ldt,Z,&n,&ifst,&ilst,work,&info);
+      LAPACKtgexc_(&ione,&ione,&n,T,&ldt,S,&ldt,Q,&n,Z,&n,&ifst,&ilst,work,&lwork,&info);
 #else
-      LAPACKtrexc_("V",&n,T,&ldt,Z,&n,&ifst,&ilst,&info);
+      LAPACKtgexc_(&ione,&ione,&n,T,&ldt,S,&ldt,Q,&n,Z,&n,&ifst,&ilst,&info);
 #endif
-      if (info) SETERRQ1(PETSC_ERR_LIB,"Error in Lapack xTREXC %d",info);
-      
-      for (j=k;j<n;j++) {
+      if (info) SETERRQ1(PETSC_ERR_LIB,"Error in Lapack xTGEXC %d",info);
+      /* recover original eigenvalues from T and S matrices */
+      for (j=k0;j<n;j++) {
 #if !defined(PETSC_USE_COMPLEX)
-        if (j==n-1 || T[j*ldt+j+1] == 0.0) { 
-          /* real eigenvalue */
-          wr[j] = T[j*ldt+j];
-          wi[j] = 0.0;
-        } else {
-          /* complex eigenvalue */
-          wr[j] = T[j*ldt+j];
-          wr[j+1] = T[j*ldt+j];
-          wi[j] = sqrt(PetscAbsReal(T[j*ldt+j+1])) *
-                  sqrt(PetscAbsReal(T[(j+1)*ldt+j]));
+        if (j<n-1 && T[j*ldt+j+1] != 0.0) {
+          /* complex conjugate eigenvalue */
+          LAPACKlag2_(T+j*ldt+j,&ldt,S+j*ldt+j,&ldt,&safmin,&scale1,&scale2,&re,&tmp,&im);
+          wr[j] = re / scale1;
+          wi[j] = im / scale1;
+          wr[j+1] = tmp / scale2;
           wi[j+1] = -wi[j];
           j++;
-        }
-#else
-        wr[j] = T[j*(ldt+1)];
+        } else
 #endif
+        {
+          if (S[j*ldt+j] == 0.0) {
+            if (PetscRealPart(T[j*ldt+j]) < 0.0) wr[j] = PETSC_MIN;
+            else wr[j] = PETSC_MAX;
+          } else wr[j] = T[j*ldt+j] / S[j*ldt+j];
+          wi[j] = 0.0;
+        }
       }
     }
 #if !defined(PETSC_USE_COMPLEX)
     if (wi[i] != 0) i++;
 #endif
   }
-  
+
 #if !defined(PETSC_USE_COMPLEX)
   ierr = PetscFree(work);CHKERRQ(ierr);
 #endif
@@ -835,8 +769,8 @@ PetscErrorCode EPSSortDenseSchurTarget(PetscInt n_,PetscInt k,PetscScalar *T,Pet
 
    Input Parameters:
 +  n   - dimension of the eigenproblem
-.  A   - pointer to the array containing the matrix values
--  lda - leading dimension of A
+.  D   - pointer to the array containing the diagonal elements
+-  E   - pointer to the array containing the off-diagonal elements
 
    Output Parameters:
 +  w  - pointer to the array to store the computed eigenvalues
@@ -845,7 +779,7 @@ PetscErrorCode EPSSortDenseSchurTarget(PetscInt n_,PetscInt k,PetscScalar *T,Pet
    Notes:
    If V is PETSC_NULL then the eigenvectors are not computed.
 
-   This routine use LAPACK routines DSTEVR.
+   This routine use LAPACK routines xSTEVR.
 
    Level: developer
 
@@ -901,3 +835,69 @@ PetscErrorCode EPSDenseTridiagonal(PetscInt n_,PetscReal *D,PetscReal *E,PetscRe
   PetscFunctionReturn(0);
 #endif
 }
+
+#undef __FUNCT__  
+#define __FUNCT__ "DenseSelectedEvec"
+/*
+   DenseSelectedEvec - Computes a selected eigenvector of matrix in Schur form.
+
+   Input Parameters:
+     S - (quasi-)triangular matrix (dimension nv, leading dimension lds)
+     U - orthogonal transformation matrix (dimension nv, leading dimension nv)
+     i - which eigenvector to process
+     iscomplex - true if a complex conjugate pair (in real scalars)
+
+   Output parameters:
+     Y - computed eigenvector, 2 columns if iscomplex=true (leading dimension nv)
+
+   Workspace:
+     work is workspace to store 3*nv scalars, nv booleans and nv reals
+*/
+PetscErrorCode DenseSelectedEvec(PetscScalar *S,PetscInt lds_,PetscScalar *U,PetscScalar *Y,PetscInt i,PetscTruth iscomplex,PetscInt nv_,PetscScalar *work)
+{
+#if defined(SLEPC_MISSING_LAPACK_TREVC)
+  PetscFunctionBegin;
+  SETERRQ(PETSC_ERR_SUP,"TREVC - Lapack routine is unavailable.");
+#else
+  PetscErrorCode ierr;
+  PetscInt       k;
+  PetscBLASInt   mm,mout,info,lds,nv,inc = 1;
+  PetscScalar    tmp,done=1.0,zero=0.0;
+  PetscReal      norm;
+  PetscTruth     *select=(PetscTruth*)(work+4*nv_);
+#if defined(PETSC_USE_COMPLEX)
+  PetscReal      *rwork=(PetscReal*)(work+3*nv_);
+#endif
+
+  PetscFunctionBegin;
+  lds = PetscBLASIntCast(lds_);
+  nv = PetscBLASIntCast(nv_);
+  for (k=0;k<nv;k++) select[k] = PETSC_FALSE;
+
+  /* Compute eigenvectors Y of S */
+  mm = iscomplex? 2: 1;
+  select[i] = PETSC_TRUE;
+#if !defined(PETSC_USE_COMPLEX)
+  if (iscomplex) select[i+1] = PETSC_TRUE;
+  LAPACKtrevc_("R","S",select,&nv,S,&lds,PETSC_NULL,&nv,Y,&nv,&mm,&mout,work,&info);
+#else
+  LAPACKtrevc_("R","S",select,&nv,S,&lds,PETSC_NULL,&nv,Y,&nv,&mm,&mout,work,rwork,&info);
+#endif
+  if (info) SETERRQ1(PETSC_ERR_LIB,"Error in Lapack xTREVC %i",info);
+  if (mout != mm) SETERRQ(PETSC_ERR_ARG_WRONG,"Inconsistent arguments");
+  ierr = PetscMemcpy(work,Y,mout*nv*sizeof(PetscScalar));CHKERRQ(ierr);
+
+  /* accumulate and normalize eigenvectors */
+  BLASgemv_("N",&nv,&nv,&done,U,&nv,work,&inc,&zero,Y,&inc);
+#if !defined(PETSC_USE_COMPLEX)
+  if (iscomplex) BLASgemv_("N",&nv,&nv,&done,U,&nv,work+nv,&inc,&zero,Y+nv,&inc);
+#endif
+  mm = mm*nv;
+  norm = BLASnrm2_(&mm,Y,&inc);
+  tmp = 1.0 / norm;
+  BLASscal_(&mm,&tmp,Y,&inc);
+
+  PetscFunctionReturn(0);
+#endif
+}
+
