@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -24,8 +24,8 @@
 #include "private/stimpl.h"          /*I "slepcst.h" I*/
 
 typedef struct {
-  PetscScalar tau;
-  PetscTruth  tau_set;
+  PetscScalar nu;
+  PetscTruth  nu_set;
   Vec         w2;
 } ST_CAYLEY;
 
@@ -35,22 +35,22 @@ PetscErrorCode STApply_Cayley(ST st,Vec x,Vec y)
 {
   PetscErrorCode ierr;
   ST_CAYLEY      *ctx = (ST_CAYLEY *) st->data;
-  PetscScalar    tau = ctx->tau;
+  PetscScalar    nu = ctx->nu;
   
   PetscFunctionBegin;
-  if (st->shift_matrix == STMATMODE_INPLACE) { tau = tau + st->sigma; };
+  if (st->shift_matrix == ST_MATMODE_INPLACE) { nu = nu + st->sigma; };
 
   if (st->B) {
     /* generalized eigenproblem: y = (A - sB)^-1 (A + tB)x */
     ierr = MatMult(st->A,x,st->w);CHKERRQ(ierr);
     ierr = MatMult(st->B,x,ctx->w2);CHKERRQ(ierr);
-    ierr = VecAXPY(st->w,tau,ctx->w2);CHKERRQ(ierr);    
+    ierr = VecAXPY(st->w,nu,ctx->w2);CHKERRQ(ierr);    
     ierr = STAssociatedKSPSolve(st,st->w,y);CHKERRQ(ierr);
   }
   else {
     /* standard eigenproblem: y = (A - sI)^-1 (A + tI)x */
     ierr = MatMult(st->A,x,st->w);CHKERRQ(ierr);
-    ierr = VecAXPY(st->w,tau,x);CHKERRQ(ierr);
+    ierr = VecAXPY(st->w,nu,x);CHKERRQ(ierr);
     ierr = STAssociatedKSPSolve(st,st->w,y);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
@@ -62,23 +62,23 @@ PetscErrorCode STApplyTranspose_Cayley(ST st,Vec x,Vec y)
 {
   PetscErrorCode ierr;
   ST_CAYLEY      *ctx = (ST_CAYLEY *) st->data;
-  PetscScalar    tau = ctx->tau;
+  PetscScalar    nu = ctx->nu;
   
   PetscFunctionBegin;
-  if (st->shift_matrix == STMATMODE_INPLACE) { tau = tau + st->sigma; };
+  if (st->shift_matrix == ST_MATMODE_INPLACE) { nu = nu + st->sigma; };
 
   if (st->B) {
     /* generalized eigenproblem: y = (A + tB)^T (A - sB)^-T x */
-    ierr = STAssociatedKSPSolve(st,x,st->w);CHKERRQ(ierr);
-    ierr = MatMult(st->A,st->w,y);CHKERRQ(ierr);
-    ierr = MatMult(st->B,st->w,ctx->w2);CHKERRQ(ierr);
-    ierr = VecAXPY(y,tau,ctx->w2);CHKERRQ(ierr);    
+    ierr = STAssociatedKSPSolveTranspose(st,x,st->w);CHKERRQ(ierr);
+    ierr = MatMultTranspose(st->A,st->w,y);CHKERRQ(ierr);
+    ierr = MatMultTranspose(st->B,st->w,ctx->w2);CHKERRQ(ierr);
+    ierr = VecAXPY(y,nu,ctx->w2);CHKERRQ(ierr);    
   }
   else {
     /* standard eigenproblem: y =  (A + tI)^T (A - sI)^-T x */
-    ierr = STAssociatedKSPSolve(st,x,st->w);CHKERRQ(ierr);
-    ierr = MatMult(st->A,st->w,y);CHKERRQ(ierr);
-    ierr = VecAXPY(y,tau,st->w);CHKERRQ(ierr);
+    ierr = STAssociatedKSPSolveTranspose(st,x,st->w);CHKERRQ(ierr);
+    ierr = MatMultTranspose(st->A,st->w,y);CHKERRQ(ierr);
+    ierr = VecAXPY(y,nu,st->w);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -90,25 +90,25 @@ PetscErrorCode STBilinearMatMult_Cayley(Mat B,Vec x,Vec y)
   PetscErrorCode ierr;
   ST             st;
   ST_CAYLEY      *ctx;
-  PetscScalar    tau;
+  PetscScalar    nu;
   
   PetscFunctionBegin;
   ierr = MatShellGetContext(B,(void**)&st);CHKERRQ(ierr);
   ctx = (ST_CAYLEY *) st->data;
-  tau = ctx->tau;
+  nu = ctx->nu;
   
-  if (st->shift_matrix == STMATMODE_INPLACE) { tau = tau + st->sigma; };
+  if (st->shift_matrix == ST_MATMODE_INPLACE) { nu = nu + st->sigma; };
 
   if (st->B) {
     /* generalized eigenproblem: y = (A + tB)x */
     ierr = MatMult(st->A,x,y);CHKERRQ(ierr);
     ierr = MatMult(st->B,x,ctx->w2);CHKERRQ(ierr);
-    ierr = VecAXPY(y,tau,ctx->w2);CHKERRQ(ierr);    
+    ierr = VecAXPY(y,nu,ctx->w2);CHKERRQ(ierr);    
   }
   else {
     /* standard eigenproblem: y = (A + tI)x */
     ierr = MatMult(st->A,x,y);CHKERRQ(ierr);
-    ierr = VecAXPY(y,tau,x);CHKERRQ(ierr);
+    ierr = VecAXPY(y,nu,x);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -129,28 +129,33 @@ PetscErrorCode STGetBilinearForm_Cayley(ST st,Mat *B)
 
 #undef __FUNCT__  
 #define __FUNCT__ "STBackTransform_Cayley"
-PetscErrorCode STBackTransform_Cayley(ST st,PetscScalar *eigr,PetscScalar *eigi)
+PetscErrorCode STBackTransform_Cayley(ST st,PetscInt n,PetscScalar *eigr,PetscScalar *eigi)
 {
   ST_CAYLEY   *ctx = (ST_CAYLEY *) st->data;
+  PetscInt    j;
 #ifndef PETSC_USE_COMPLEX
   PetscScalar t,i,r;
   PetscFunctionBegin;
-  PetscValidPointer(eigr,2);
-  PetscValidPointer(eigi,3);
-  if (*eigi == 0.0) *eigr = (ctx->tau + *eigr * st->sigma) / (*eigr - 1.0);
-  else {
-    r = *eigr;
-    i = *eigi;
-    r = st->sigma * (r * r + i * i - r) + ctx->tau * (r - 1);
-    i = - st->sigma * i - ctx->tau * i;
-    t = i * i + r * (r - 2.0) + 1.0;    
-    *eigr = r / t;
-    *eigi = i / t;    
+  PetscValidPointer(eigr,3);
+  PetscValidPointer(eigi,4);
+  for (j=0;j<n;j++) {
+    if (eigi[j] == 0.0) eigr[j] = (ctx->nu + eigr[j] * st->sigma) / (eigr[j] - 1.0);
+    else {
+      r = eigr[j];
+      i = eigi[j];
+      r = st->sigma * (r * r + i * i - r) + ctx->nu * (r - 1);
+      i = - st->sigma * i - ctx->nu * i;
+      t = i * i + r * (r - 2.0) + 1.0;    
+      eigr[j] = r / t;
+      eigi[j] = i / t;    
+    }
   }
 #else
   PetscFunctionBegin;
   PetscValidPointer(eigr,2);
-  *eigr = (ctx->tau + *eigr * st->sigma) / (*eigr - 1.0);
+  for (j=0;j<n;j++) {
+    eigr[j] = (ctx->nu + eigr[j] * st->sigma) / (eigr[j] - 1.0);
+  }
 #endif
   PetscFunctionReturn(0);
 }
@@ -162,7 +167,7 @@ PetscErrorCode STPostSolve_Cayley(ST st)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (st->shift_matrix == STMATMODE_INPLACE) {
+  if (st->shift_matrix == ST_MATMODE_INPLACE) {
     if (st->B) {
       ierr = MatAXPY(st->A,st->sigma,st->B,st->str);CHKERRQ(ierr);
     } else { 
@@ -183,13 +188,17 @@ PetscErrorCode STSetUp_Cayley(ST st)
   PetscFunctionBegin;
 
   if (st->mat) { ierr = MatDestroy(st->mat);CHKERRQ(ierr); }
-  if (!ctx->tau_set) { ctx->tau = st->sigma; }
-  if (ctx->tau == 0.0 &&  st->sigma == 0.0) {
+
+  /* if the user did not set the shift, use the target value */
+  if (!st->sigma_set) st->sigma = st->defsigma;
+
+  if (!ctx->nu_set) { ctx->nu = st->sigma; }
+  if (ctx->nu == 0.0 &&  st->sigma == 0.0) {
     SETERRQ(1,"Values of shift and antishift cannot be zero simultaneously");
   }
 
   switch (st->shift_matrix) {
-  case STMATMODE_INPLACE:
+  case ST_MATMODE_INPLACE:
     st->mat = PETSC_NULL;
     if (st->sigma != 0.0) {
       if (st->B) { 
@@ -200,7 +209,7 @@ PetscErrorCode STSetUp_Cayley(ST st)
     }
     ierr = KSPSetOperators(st->ksp,st->A,st->A,DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr);
     break;
-  case STMATMODE_SHELL:
+  case ST_MATMODE_SHELL:
     ierr = STMatShellCreate(st,&st->mat);CHKERRQ(ierr);
     ierr = KSPSetOperators(st->ksp,st->mat,st->mat,DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr);
     break;
@@ -232,8 +241,8 @@ PetscErrorCode STSetShift_Cayley(ST st,PetscScalar newshift)
   MatStructure   flg;
 
   PetscFunctionBegin;
-  if (!ctx->tau_set) { ctx->tau = newshift; }
-  if (ctx->tau == 0.0 &&  newshift == 0.0) {
+  if (!ctx->nu_set) { ctx->nu = newshift; }
+  if (ctx->nu == 0.0 &&  newshift == 0.0) {
     SETERRQ(1,"Values of shift and antishift cannot be zero simultaneously");
   }
 
@@ -248,7 +257,7 @@ PetscErrorCode STSetShift_Cayley(ST st,PetscScalar newshift)
   }
 
   switch (st->shift_matrix) {
-  case STMATMODE_INPLACE:
+  case ST_MATMODE_INPLACE:
     /* Undo previous operations */
     if (st->sigma != 0.0) {
       if (st->B) { 
@@ -267,7 +276,7 @@ PetscErrorCode STSetShift_Cayley(ST st,PetscScalar newshift)
     }
     ierr = KSPSetOperators(st->ksp,st->A,st->A,flg);CHKERRQ(ierr);
     break;
-  case STMATMODE_SHELL:
+  case ST_MATMODE_SHELL:
     ierr = KSPSetOperators(st->ksp,st->mat,st->mat,DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr);    
     break;
   default:
@@ -288,17 +297,36 @@ PetscErrorCode STSetShift_Cayley(ST st,PetscScalar newshift)
 PetscErrorCode STSetFromOptions_Cayley(ST st) 
 {
   PetscErrorCode ierr;
-  PetscScalar    tau;
+  PetscScalar    nu;
   PetscTruth     flg;
   ST_CAYLEY      *ctx = (ST_CAYLEY *) st->data;
+  PC             pc;
+  const PCType   pctype;
+  const KSPType  ksptype;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsHead("ST Cayley Options");CHKERRQ(ierr);
-  ierr = PetscOptionsScalar("-st_antishift","Value of the antishift","STSetAntishift",ctx->tau,&tau,&flg); CHKERRQ(ierr);
-  if (flg) {
-    ierr = STCayleySetAntishift(st,tau);CHKERRQ(ierr);
+
+  ierr = KSPGetPC(st->ksp,&pc);CHKERRQ(ierr);
+  ierr = KSPGetType(st->ksp,&ksptype);CHKERRQ(ierr);
+  ierr = PCGetType(pc,&pctype);CHKERRQ(ierr);
+  if (!pctype && !ksptype) {
+    if (st->shift_matrix == ST_MATMODE_SHELL) {
+      /* in shell mode use GMRES with Jacobi as the default */
+      ierr = KSPSetType(st->ksp,KSPGMRES);CHKERRQ(ierr);
+      ierr = PCSetType(pc,PCJACOBI);CHKERRQ(ierr);
+    } else {
+      /* use direct solver as default */
+      ierr = KSPSetType(st->ksp,KSPPREONLY);CHKERRQ(ierr);
+      ierr = PCSetType(pc,PCREDUNDANT);CHKERRQ(ierr);
+    }
   }
-  ierr = PetscOptionsTail();CHKERRQ(ierr);
+
+  ierr = PetscOptionsBegin(((PetscObject)st)->comm,((PetscObject)st)->prefix,"ST Cayley Options","ST");CHKERRQ(ierr);
+  ierr = PetscOptionsScalar("-st_cayley_antishift","Value of the antishift","STCayleySetAntishift",ctx->nu,&nu,&flg); CHKERRQ(ierr);
+  if (flg) {
+    ierr = STCayleySetAntishift(st,nu);CHKERRQ(ierr);
+  }
+  ierr = PetscOptionsEnd();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -310,8 +338,8 @@ PetscErrorCode STCayleySetAntishift_Cayley(ST st,PetscScalar newshift)
   ST_CAYLEY *ctx = (ST_CAYLEY *) st->data;
 
   PetscFunctionBegin;
-  ctx->tau = newshift;
-  ctx->tau_set = PETSC_TRUE;
+  ctx->nu = newshift;
+  ctx->nu_set = PETSC_TRUE;
   PetscFunctionReturn(0);
 }
 EXTERN_C_END
@@ -326,21 +354,21 @@ EXTERN_C_END
 
    Input Parameters:
 +  st  - the spectral transformation context
--  tau - the anti-shift
+-  nu  - the anti-shift
 
    Options Database Key:
-.  -st_antishift - Sets the value of the anti-shift
+.  -st_cayley_antishift - Sets the value of the anti-shift
 
    Level: intermediate
 
    Note:
    In the generalized Cayley transform, the operator can be expressed as
-   OP = inv(A - sigma B)*(A + tau B). This function sets the value of tau.
+   OP = inv(A - sigma B)*(A + nu B). This function sets the value of nu.
    Use STSetShift() for setting sigma.
 
 .seealso: STSetShift()
 @*/
-PetscErrorCode STCayleySetAntishift(ST st,PetscScalar tau)
+PetscErrorCode STCayleySetAntishift(ST st,PetscScalar nu)
 {
   PetscErrorCode ierr, (*f)(ST,PetscScalar);
 
@@ -348,7 +376,7 @@ PetscErrorCode STCayleySetAntishift(ST st,PetscScalar tau)
   PetscValidHeaderSpecific(st,ST_COOKIE,1);
   ierr = PetscObjectQueryFunction((PetscObject)st,"STCayleySetAntishift_C",(void (**)(void))&f);CHKERRQ(ierr);
   if (f) {
-    ierr = (*f)(st,tau);CHKERRQ(ierr);
+    ierr = (*f)(st,nu);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -362,9 +390,9 @@ PetscErrorCode STView_Cayley(ST st,PetscViewer viewer)
 
   PetscFunctionBegin;
 #if !defined(PETSC_USE_COMPLEX)
-  ierr = PetscViewerASCIIPrintf(viewer,"  antishift: %g\n",ctx->tau);CHKERRQ(ierr);
+  ierr = PetscViewerASCIIPrintf(viewer,"  antishift: %g\n",ctx->nu);CHKERRQ(ierr);
 #else
-  ierr = PetscViewerASCIIPrintf(viewer,"  antishift: %g+%g i\n",PetscRealPart(ctx->tau),PetscImaginaryPart(ctx->tau));CHKERRQ(ierr);
+  ierr = PetscViewerASCIIPrintf(viewer,"  antishift: %g+%g i\n",PetscRealPart(ctx->nu),PetscImaginaryPart(ctx->nu));CHKERRQ(ierr);
 #endif
   ierr = STView_Default(st,viewer);CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -380,6 +408,7 @@ PetscErrorCode STDestroy_Cayley(ST st)
   PetscFunctionBegin;
   if (ctx->w2) { ierr = VecDestroy(ctx->w2);CHKERRQ(ierr); }
   ierr = PetscFree(ctx);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STCayleySetAntishift_C","",PETSC_NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -407,13 +436,12 @@ PetscErrorCode STCreate_Cayley(ST st)
   st->ops->destroy         = STDestroy_Cayley;
   st->ops->view            = STView_Cayley;
   
-  st->checknullspace      = STCheckNullSpace_Default;
+  st->checknullspace       = STCheckNullSpace_Default;
 
-  ctx->tau                = 0.0;
-  ctx->tau_set            = PETSC_FALSE;
+  ctx->nu                  = 0.0;
+  ctx->nu_set              = PETSC_FALSE;
 
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STCayleySetAntishift_C","STCayleySetAntishift_Cayley",
-                    STCayleySetAntishift_Cayley);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STCayleySetAntishift_C","STCayleySetAntishift_Cayley",STCayleySetAntishift_Cayley);CHKERRQ(ierr);
 
   PetscFunctionReturn(0);
 }
