@@ -4,7 +4,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -25,7 +25,6 @@
 #include "private/stimpl.h"          /*I "slepcst.h" I*/
 
 typedef struct {
-  PetscTruth  left;
   Vec         w2;
 } ST_FOLD;
 
@@ -99,33 +98,29 @@ PetscErrorCode STApplyTranspose_Fold(ST st,Vec x,Vec y)
 
 #undef __FUNCT__  
 #define __FUNCT__ "STBackTransform_Fold"
-PetscErrorCode STBackTransform_Fold(ST st,PetscScalar *eigr,PetscScalar *eigi)
+PetscErrorCode STBackTransform_Fold(ST st,PetscInt n,PetscScalar *eigr,PetscScalar *eigi)
 {
-  ST_FOLD *ctx = (ST_FOLD *) st->data;
+  PetscInt j;
   PetscFunctionBegin;
-  PetscValidScalarPointer(eigr,2);
-  PetscValidScalarPointer(eigi,3);
+  PetscValidScalarPointer(eigr,3);
+  PetscValidScalarPointer(eigi,4);
+  for (j=0;j<n;j++) {
 #if !defined(PETSC_USE_COMPLEX)
-  if (*eigi == 0) {
+    if (eigi[j] == 0) {
 #endif
-    if (ctx->left) *eigr = st->sigma - PetscSqrtScalar(*eigr);
-    else *eigr = st->sigma + PetscSqrtScalar(*eigr);
+      eigr[j] = st->sigma + PetscSqrtScalar(eigr[j]);
 #if !defined(PETSC_USE_COMPLEX)
-  } else {
-    PetscScalar r,x,y;
-    r = PetscSqrtScalar(*eigr * *eigr + *eigi * *eigi);
-    x = PetscSqrtScalar((r + *eigr) / 2);
-    y = PetscSqrtScalar((r - *eigr) / 2);
-    if (*eigi < 0) y = - y;
-    if (ctx->left) {
-      *eigr = st->sigma - x;
-      *eigi = - y;
     } else {
-      *eigr = st->sigma + x;
-      *eigi = y;
+      PetscScalar r,x,y;
+      r = PetscSqrtScalar(eigr[j] * eigr[j] + eigi[j] * eigi[j]);
+      x = PetscSqrtScalar((r + eigr[j]) / 2);
+      y = PetscSqrtScalar((r - eigr[j]) / 2);
+      if (eigi[j] < 0) y = - y;
+      eigr[j] = st->sigma + x;
+      eigi[j] = y;
     }
-  }
 #endif
+  }
   PetscFunctionReturn(0);
 }
 
@@ -137,6 +132,9 @@ PetscErrorCode STSetUp_Fold(ST st)
   ST_FOLD        *ctx = (ST_FOLD *) st->data;
 
   PetscFunctionBegin;
+  /* if the user did not set the shift, use the target value */
+  if (!st->sigma_set) st->sigma = st->defsigma;
+
   if (st->B) {
     ierr = KSPSetOperators(st->ksp,st->B,st->B,DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr);
     ierr = KSPSetUp(st->ksp);CHKERRQ(ierr);
@@ -146,61 +144,13 @@ PetscErrorCode STSetUp_Fold(ST st)
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
-#undef __FUNCT__  
-#define __FUNCT__ "STFoldSetLeftSide_Fold"
-PetscErrorCode STFoldSetLeftSide_Fold(ST st,PetscTruth left)
-{
-  ST_FOLD *ctx = (ST_FOLD *) st->data;
-
-  PetscFunctionBegin;
-  ctx->left = left;
-  PetscFunctionReturn(0);
-}
-EXTERN_C_END
-
-#undef __FUNCT__  
-#define __FUNCT__ "STFoldSetLeftSide"
-/*@
-   STFoldSetLeftSide - Sets a flag to compute eigenvalues on the left side of shift.
-
-   Collective on ST
-
-   Input Parameters:
-+  st  - the spectral transformation context
--  left - if true compute eigenvalues on the left side 
-
-   Options Database Key:
-.  -st_fold_leftside - Sets the value of the flag
-
-   Level: intermediate
-
-.seealso: STSetShift()
-@*/
-PetscErrorCode STFoldSetLeftSide(ST st,PetscTruth left)
-{
-  PetscErrorCode ierr, (*f)(ST,PetscTruth);
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(st,ST_COOKIE,1);
-  ierr = PetscObjectQueryFunction((PetscObject)st,"STFoldSetLeftSide_C",(void (**)(void))&f);CHKERRQ(ierr);
-  if (f) {
-    ierr = (*f)(st,left);CHKERRQ(ierr);
-  }
-  PetscFunctionReturn(0);
-}
-
 #undef __FUNCT__  
 #define __FUNCT__ "STView_Fold"
 PetscErrorCode STView_Fold(ST st,PetscViewer viewer)
 {
   PetscErrorCode ierr;
-  ST_FOLD        *ctx = (ST_FOLD *) st->data;
 
   PetscFunctionBegin;
-  if (ctx->left) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  computing eigenvalues on left side of shift\n");CHKERRQ(ierr);
-  }  
   if (st->B) {
     ierr = STView_Default(st,viewer);CHKERRQ(ierr);
   } 
@@ -212,12 +162,27 @@ PetscErrorCode STView_Fold(ST st,PetscViewer viewer)
 PetscErrorCode STSetFromOptions_Fold(ST st) 
 {
   PetscErrorCode ierr;
-  ST_FOLD      *ctx = (ST_FOLD *) st->data;
+  PC             pc;
+  const PCType   pctype;
+  const KSPType  ksptype;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsHead("ST Fold Options");CHKERRQ(ierr);
-  ierr = PetscOptionsTruth("-st_fold_leftside","Compute eigenvalues on left side of shift","STFoldSetLeftSide",ctx->left,&ctx->left,PETSC_NULL); CHKERRQ(ierr);
-  ierr = PetscOptionsTail();CHKERRQ(ierr);
+
+  ierr = KSPGetPC(st->ksp,&pc);CHKERRQ(ierr);
+  ierr = KSPGetType(st->ksp,&ksptype);CHKERRQ(ierr);
+  ierr = PCGetType(pc,&pctype);CHKERRQ(ierr);
+  if (!pctype && !ksptype) {
+    if (st->shift_matrix == ST_MATMODE_SHELL) {
+      /* in shell mode use GMRES with Jacobi as the default */
+      ierr = KSPSetType(st->ksp,KSPGMRES);CHKERRQ(ierr);
+      ierr = PCSetType(pc,PCJACOBI);CHKERRQ(ierr);
+    } else {
+      /* use direct solver as default */
+      ierr = KSPSetType(st->ksp,KSPPREONLY);CHKERRQ(ierr);
+      ierr = PCSetType(pc,PCREDUNDANT);CHKERRQ(ierr);
+    }
+  }
+
   PetscFunctionReturn(0);
 }
 
@@ -258,11 +223,6 @@ PetscErrorCode STCreate_Fold(ST st)
   st->ops->destroy	   = STDestroy_Fold;
   st->checknullspace	   = 0;
   
-  ctx->left            = PETSC_FALSE;
-  
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STFoldSetLeftSide_C","STFoldSetLeftSide_Fold",
-                    STFoldSetLeftSide_Fold);CHKERRQ(ierr);
-
   PetscFunctionReturn(0);
 }
 EXTERN_C_END
