@@ -5,7 +5,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -29,9 +29,9 @@
 EXTERN_C_BEGIN 
 typedef struct {
   void           *ctx;                       /* user provided context */
-  PetscErrorCode (*apply)(void *,Vec,Vec);
-  PetscErrorCode (*applytrans)(void *,Vec,Vec);
-  PetscErrorCode (*backtr)(void *,PetscScalar*,PetscScalar*);
+  PetscErrorCode (*apply)(ST,Vec,Vec);
+  PetscErrorCode (*applytrans)(ST,Vec,Vec);
+  PetscErrorCode (*backtr)(ST,PetscInt n,PetscScalar*,PetscScalar*);
   char           *name;
 } ST_Shell;
 EXTERN_C_END
@@ -72,7 +72,7 @@ PetscErrorCode STShellGetContext(ST st,void **ctx)
 
 #undef __FUNCT__  
 #define __FUNCT__ "STShellSetContext"
-/*@C
+/*@
     STShellSetContext - sets the context for a shell ST
 
    Collective on ST
@@ -112,9 +112,9 @@ PetscErrorCode STApply_Shell(ST st,Vec x,Vec y)
 
   PetscFunctionBegin;
   if (!shell->apply) SETERRQ(PETSC_ERR_USER,"No apply() routine provided to Shell ST");
-  PetscStackPush("PCSHELL user function");
+  PetscStackPush("STSHELL apply() user function");
   CHKMEMQ;
-  ierr  = (*shell->apply)(shell->ctx,x,y);CHKERRQ(ierr);
+  ierr  = (*shell->apply)(st,x,y);CHKERRQ(ierr);
   CHKMEMQ;
   PetscStackPop;
   PetscFunctionReturn(0);
@@ -129,20 +129,28 @@ PetscErrorCode STApplyTranspose_Shell(ST st,Vec x,Vec y)
 
   PetscFunctionBegin;
   if (!shell->applytrans) SETERRQ(PETSC_ERR_USER,"No applytranspose() routine provided to Shell ST");
-  ierr  = (*shell->applytrans)(shell->ctx,x,y);CHKERRQ(ierr);
+  PetscStackPush("STSHELL applytranspose() user function");
+  CHKMEMQ;
+  ierr  = (*shell->applytrans)(st,x,y);CHKERRQ(ierr);
+  CHKMEMQ;
+  PetscStackPop;
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
 #define __FUNCT__ "STBackTransform_Shell"
-PetscErrorCode STBackTransform_Shell(ST st,PetscScalar *eigr,PetscScalar *eigi)
+PetscErrorCode STBackTransform_Shell(ST st,PetscInt n,PetscScalar *eigr,PetscScalar *eigi)
 {
   PetscErrorCode ierr;
   ST_Shell       *shell = (ST_Shell*)st->data;
 
   PetscFunctionBegin;
   if (shell->backtr) {
-    ierr  = (*shell->backtr)(shell->ctx,eigr,eigi);CHKERRQ(ierr);
+    PetscStackPush("STSHELL backtransform() user function");
+    CHKMEMQ;
+    ierr  = (*shell->backtr)(st,n,eigr,eigi);CHKERRQ(ierr);
+    CHKMEMQ;
+    PetscStackPop;
   }
   PetscFunctionReturn(0);
 }
@@ -157,6 +165,11 @@ PetscErrorCode STDestroy_Shell(ST st)
   PetscFunctionBegin;
   ierr = PetscFree(shell->name);CHKERRQ(ierr);
   ierr = PetscFree(shell);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STShellSetApply_C","",PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STShellSetApplyTranspose_C","",PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STShellSetBackTransform_C","",PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STShellSetName_C","",PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STShellGetName_C","",PETSC_NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -182,7 +195,7 @@ PetscErrorCode STView_Shell(ST st,PetscViewer viewer)
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "STShellSetApply_Shell"
-PetscErrorCode STShellSetApply_Shell(ST st,PetscErrorCode (*apply)(void*,Vec,Vec))
+PetscErrorCode STShellSetApply_Shell(ST st,PetscErrorCode (*apply)(ST,Vec,Vec))
 {
   ST_Shell *shell = (ST_Shell*)st->data;
 
@@ -195,7 +208,7 @@ EXTERN_C_END
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "STShellSetApplyTranspose_Shell"
-PetscErrorCode STShellSetApplyTranspose_Shell(ST st,PetscErrorCode (*applytrans)(void*,Vec,Vec))
+PetscErrorCode STShellSetApplyTranspose_Shell(ST st,PetscErrorCode (*applytrans)(ST,Vec,Vec))
 {
   ST_Shell *shell = (ST_Shell*)st->data;
 
@@ -208,7 +221,7 @@ EXTERN_C_END
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "STShellSetBackTransform_Shell"
-PetscErrorCode STShellSetBackTransform_Shell(ST st,PetscErrorCode (*backtr)(void*,PetscScalar*,PetscScalar*))
+PetscErrorCode STShellSetBackTransform_Shell(ST st,PetscErrorCode (*backtr)(ST,PetscInt,PetscScalar*,PetscScalar*))
 {
   ST_Shell *shell = (ST_Shell *) st->data;
 
@@ -227,7 +240,7 @@ PetscErrorCode STShellSetName_Shell(ST st,const char name[])
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = PetscStrfree(shell->name);CHKERRQ(ierr);    
+  ierr = PetscFree(shell->name);CHKERRQ(ierr);    
   ierr = PetscStrallocpy(name,&shell->name);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -260,10 +273,10 @@ EXTERN_C_END
 
    Calling sequence of apply:
 .vb
-   PetscErrorCode apply (void *ptr,Vec xin,Vec xout)
+   PetscErrorCode apply (ST st,Vec xin,Vec xout)
 .ve
 
-+  ptr  - the application context
++  st   - the spectral transformation context
 .  xin  - input vector
 -  xout - output vector
 
@@ -271,9 +284,9 @@ EXTERN_C_END
 
 .seealso: STShellSetBackTransform(), STShellSetApplyTranspose()
 @*/
-PetscErrorCode STShellSetApply(ST st,PetscErrorCode (*apply)(void*,Vec,Vec))
+PetscErrorCode STShellSetApply(ST st,PetscErrorCode (*apply)(ST,Vec,Vec))
 {
-  PetscErrorCode ierr, (*f)(ST,PetscErrorCode (*)(void*,Vec,Vec));
+  PetscErrorCode ierr, (*f)(ST,PetscErrorCode (*)(ST,Vec,Vec));
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(st,ST_COOKIE,1);
@@ -298,10 +311,10 @@ PetscErrorCode STShellSetApply(ST st,PetscErrorCode (*apply)(void*,Vec,Vec))
 
    Calling sequence of apply:
 .vb
-   PetscErrorCode applytrans (void *ptr,Vec xin,Vec xout)
+   PetscErrorCode applytrans (ST st,Vec xin,Vec xout)
 .ve
 
-+  ptr  - the application context
++  st   - the spectral transformation context
 .  xin  - input vector
 -  xout - output vector
 
@@ -309,9 +322,9 @@ PetscErrorCode STShellSetApply(ST st,PetscErrorCode (*apply)(void*,Vec,Vec))
 
 .seealso: STShellSetApply(), STShellSetBackTransform()
 @*/
-PetscErrorCode STShellSetApplyTranspose(ST st,PetscErrorCode (*applytrans)(void*,Vec,Vec))
+PetscErrorCode STShellSetApplyTranspose(ST st,PetscErrorCode (*applytrans)(ST,Vec,Vec))
 {
-  PetscErrorCode ierr, (*f)(ST,PetscErrorCode (*)(void*,Vec,Vec));
+  PetscErrorCode ierr, (*f)(ST,PetscErrorCode (*)(ST,Vec,Vec));
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(st,ST_COOKIE,1);
@@ -337,10 +350,10 @@ PetscErrorCode STShellSetApplyTranspose(ST st,PetscErrorCode (*applytrans)(void*
 
    Calling sequence of backtr:
 .vb
-   PetscErrorCode backtr (void *ptr,PetscScalar *eigr,PetscScalar *eigi)
+   PetscErrorCode backtr (ST st,PetscScalar *eigr,PetscScalar *eigi)
 .ve
 
-+  ptr  - the application context
++  st   - the spectral transformation context
 .  eigr - pointer ot the real part of the eigenvalue to transform back
 -  eigi - pointer ot the imaginary part 
 
@@ -348,15 +361,15 @@ PetscErrorCode STShellSetApplyTranspose(ST st,PetscErrorCode (*applytrans)(void*
 
 .seealso: STShellSetApply(), STShellSetApplyTranspose()
 @*/
-PetscErrorCode STShellSetBackTransform(ST st,PetscErrorCode (*backtr)(void*,PetscScalar*,PetscScalar*))
+PetscErrorCode STShellSetBackTransform(ST st,PetscErrorCode (*backtr)(ST,PetscInt,PetscScalar*,PetscScalar*))
 {
-  PetscErrorCode ierr, (*f)(ST,PetscErrorCode (*)(void*,PetscScalar*,PetscScalar*));
+  PetscErrorCode ierr, (*f)(ST,PetscErrorCode (*)(ST,PetscInt,PetscScalar*,PetscScalar*));
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(st,ST_COOKIE,1);
   ierr = PetscObjectQueryFunction((PetscObject)st,"STShellSetBackTransform_C",(void (**)(void))&f);CHKERRQ(ierr);
   if (f) {
-    ierr = (*f)(st,(PetscErrorCode (*)(void*,PetscScalar*,PetscScalar*))backtr);CHKERRQ(ierr);
+    ierr = (*f)(st,backtr);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -424,6 +437,35 @@ PetscErrorCode STShellGetName(ST st,char *name[])
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__  
+#define __FUNCT__ "STSetFromOptions_Shell"
+PetscErrorCode STSetFromOptions_Shell(ST st) 
+{
+  PetscErrorCode ierr;
+  PC             pc;
+  const PCType   pctype;
+  const KSPType  ksptype;
+
+  PetscFunctionBegin;
+
+  ierr = KSPGetPC(st->ksp,&pc);CHKERRQ(ierr);
+  ierr = KSPGetType(st->ksp,&ksptype);CHKERRQ(ierr);
+  ierr = PCGetType(pc,&pctype);CHKERRQ(ierr);
+  if (!pctype && !ksptype) {
+    if (st->shift_matrix == ST_MATMODE_SHELL) {
+      /* in shell mode use GMRES with Jacobi as the default */
+      ierr = KSPSetType(st->ksp,KSPGMRES);CHKERRQ(ierr);
+      ierr = PCSetType(pc,PCJACOBI);CHKERRQ(ierr);
+    } else {
+      /* use direct solver as default */
+      ierr = KSPSetType(st->ksp,KSPPREONLY);CHKERRQ(ierr);
+      ierr = PCSetType(pc,PCREDUNDANT);CHKERRQ(ierr);
+    }
+  }
+
+  PetscFunctionReturn(0);
+}
+
 /*MC
    STSHELL - Creates a new spectral transformation class.
           This is intended to provide a simple class to use with EPS.
@@ -459,10 +501,11 @@ PetscErrorCode STCreate_Shell(ST st)
   st->data           = (void *) shell;
   ((PetscObject)st)->name           = 0;
 
-  st->ops->apply     = STApply_Shell;
-  st->ops->applytrans= STApplyTranspose_Shell;
-  st->ops->backtr    = STBackTransform_Shell;
-  st->ops->view      = STView_Shell;
+  st->ops->apply          = STApply_Shell;
+  st->ops->applytrans     = STApplyTranspose_Shell;
+  st->ops->backtr         = STBackTransform_Shell;
+  st->ops->view           = STView_Shell;
+  st->ops->setfromoptions = STSetFromOptions_Shell;
 
   shell->apply       = 0;
   shell->applytrans  = 0;
@@ -470,16 +513,11 @@ PetscErrorCode STCreate_Shell(ST st)
   shell->name        = 0;
   shell->ctx         = 0;
 
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STShellSetApply_C","STShellSetApply_Shell",
-                    STShellSetApply_Shell);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STShellSetApplyTranspose_C","STShellSetApplyTranspose_Shell",
-                    STShellSetApplyTranspose_Shell);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STShellSetBackTransform_C","STShellSetBackTransform_Shell",
-                    STShellSetBackTransform_Shell);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STShellSetName_C","STShellSetName_Shell",
-                    STShellSetName_Shell);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STShellGetName_C","STShellGetName_Shell",
-                    STShellGetName_Shell);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STShellSetApply_C","STShellSetApply_Shell",STShellSetApply_Shell);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STShellSetApplyTranspose_C","STShellSetApplyTranspose_Shell",STShellSetApplyTranspose_Shell);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STShellSetBackTransform_C","STShellSetBackTransform_Shell",STShellSetBackTransform_Shell);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STShellSetName_C","STShellSetName_Shell",STShellSetName_Shell);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)st,"STShellGetName_C","STShellGetName_Shell",STShellGetName_Shell);CHKERRQ(ierr);
 
   PetscFunctionReturn(0);
 }
