@@ -4,7 +4,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -70,10 +70,14 @@ PetscErrorCode STApplyTranspose_Shift(ST st,Vec x,Vec y)
 
 #undef __FUNCT__  
 #define __FUNCT__ "STBackTransform_Shift"
-PetscErrorCode STBackTransform_Shift(ST st,PetscScalar *eigr,PetscScalar *eigi)
+PetscErrorCode STBackTransform_Shift(ST st,PetscInt n,PetscScalar *eigr,PetscScalar *eigi)
 {
+  PetscInt j;
   PetscFunctionBegin;
-  if (eigr) *eigr -= st->sigma;
+  PetscValidPointer(eigr,3);
+  for (j=0;j<n;j++) {
+    eigr[j] -= st->sigma;
+  }
   PetscFunctionReturn(0);
 }
 
@@ -104,6 +108,35 @@ PetscErrorCode STView_Shift(ST st,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__  
+#define __FUNCT__ "STSetFromOptions_Shift"
+PetscErrorCode STSetFromOptions_Shift(ST st) 
+{
+  PetscErrorCode ierr;
+  PC             pc;
+  const PCType   pctype;
+  const KSPType  ksptype;
+
+  PetscFunctionBegin;
+
+  ierr = KSPGetPC(st->ksp,&pc);CHKERRQ(ierr);
+  ierr = KSPGetType(st->ksp,&ksptype);CHKERRQ(ierr);
+  ierr = PCGetType(pc,&pctype);CHKERRQ(ierr);
+  if (!pctype && !ksptype) {
+    if (st->shift_matrix == ST_MATMODE_SHELL) {
+      /* in shell mode use GMRES with Jacobi as the default */
+      ierr = KSPSetType(st->ksp,KSPGMRES);CHKERRQ(ierr);
+      ierr = PCSetType(pc,PCJACOBI);CHKERRQ(ierr);
+    } else {
+      /* use direct solver as default */
+      ierr = KSPSetType(st->ksp,KSPPREONLY);CHKERRQ(ierr);
+      ierr = PCSetType(pc,PCREDUNDANT);CHKERRQ(ierr);
+    }
+  }
+
+  PetscFunctionReturn(0);
+}
+
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "STCreate_Shift"
@@ -114,6 +147,7 @@ PetscErrorCode STCreate_Shift(ST st)
   st->ops->getbilinearform = STGetBilinearForm_Default;
   st->ops->applytrans      = STApplyTranspose_Shift;
   st->ops->backtr          = STBackTransform_Shift;
+  st->ops->setfromoptions  = STSetFromOptions_Shift;
   st->ops->setup           = STSetUp_Shift;
   st->ops->view            = STView_Shift;
   st->checknullspace       = 0;
