@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -25,6 +25,25 @@
 
 PetscCookie ST_COOKIE = 0;
 PetscLogEvent ST_SetUp = 0, ST_Apply = 0, ST_ApplyTranspose = 0;
+static PetscTruth STPackageInitialized = PETSC_FALSE;
+
+#undef __FUNCT__  
+#define __FUNCT__ "STFinalizePackage"
+/*@C
+  STFinalizePackage - This function destroys everything in the Slepc interface to the ST package. It is
+  called from SlepcFinalize().
+
+  Level: developer
+
+.seealso: SlepcFinalize()
+@*/
+PetscErrorCode STFinalizePackage(void) 
+{
+  PetscFunctionBegin;
+  STPackageInitialized = PETSC_FALSE;
+  STList               = 0;
+  PetscFunctionReturn(0);
+}
 
 #undef __FUNCT__  
 #define __FUNCT__ "STInitializePackage"
@@ -41,15 +60,14 @@ PetscLogEvent ST_SetUp = 0, ST_Apply = 0, ST_ApplyTranspose = 0;
 .seealso: SlepcInitialize()
 @*/
 PetscErrorCode STInitializePackage(char *path) {
-  static PetscTruth initialized = PETSC_FALSE;
   char              logList[256];
   char             *className;
   PetscTruth        opt;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (initialized) PetscFunctionReturn(0);
-  initialized = PETSC_TRUE;
+  if (STPackageInitialized) PetscFunctionReturn(0);
+  STPackageInitialized = PETSC_TRUE;
   /* Register Classes */
   ierr = PetscCookieRegister("Spectral Transform",&ST_COOKIE);CHKERRQ(ierr);
   /* Register Constructors */
@@ -74,6 +92,7 @@ PetscErrorCode STInitializePackage(char *path) {
       ierr = PetscLogEventDeactivateClass(ST_COOKIE);CHKERRQ(ierr);
     }
   }
+  ierr = PetscRegisterFinalize(STFinalizePackage);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -107,7 +126,9 @@ PetscErrorCode STDestroy(ST st)
   if (st->B) { ierr = MatDestroy(st->B);CHKERRQ(ierr); }
   if (st->ksp) { ierr = KSPDestroy(st->ksp);CHKERRQ(ierr); } 
   if (st->w) { ierr = VecDestroy(st->w);CHKERRQ(ierr); } 
-  if (st->shift_matrix != STMATMODE_INPLACE && st->mat) { 
+  if (st->D) { ierr = VecDestroy(st->D);CHKERRQ(ierr); } 
+  if (st->wb){ ierr = VecDestroy(st->wb);CHKERRQ(ierr); } 
+  if (st->shift_matrix != ST_MATMODE_INPLACE && st->mat) { 
     ierr = MatDestroy(st->mat);CHKERRQ(ierr); 
   }
 
@@ -148,10 +169,14 @@ PetscErrorCode STCreate(MPI_Comm comm,ST *newst)
   st->A                   = 0;
   st->B                   = 0;
   st->sigma               = 0.0;
+  st->sigma_set           = PETSC_FALSE;
+  st->defsigma            = 0.0;
   st->data                = 0;
   st->setupcalled         = 0;
   st->w                   = 0;
-  st->shift_matrix        = STMATMODE_COPY;
+  st->D                   = 0;
+  st->wb                  = 0;
+  st->shift_matrix        = ST_MATMODE_COPY;
   st->str                 = DIFFERENT_NONZERO_PATTERN;
   
   ierr = KSPCreate(((PetscObject)st)->comm,&st->ksp);CHKERRQ(ierr);
@@ -234,9 +259,9 @@ PetscErrorCode STGetOperators(ST st,Mat *A,Mat *B)
 #undef __FUNCT__  
 #define __FUNCT__ "STSetShift"
 /*@
-   STSetShift - Sets the shift associated with the spectral transformation
+   STSetShift - Sets the shift associated with the spectral transformation.
 
-   Not collective
+   Collective on ST
 
    Input Parameters:
 +  st - the spectral transformation context
@@ -261,6 +286,7 @@ PetscErrorCode STSetShift(ST st,PetscScalar shift)
     }
   }
   st->sigma = shift;
+  st->sigma_set = PETSC_TRUE;
   PetscFunctionReturn(0);
 }
 
@@ -288,6 +314,98 @@ PetscErrorCode STGetShift(ST st,PetscScalar* shift)
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__  
+#define __FUNCT__ "STSetDefaultShift"
+/*@
+   STSetDefaultShift - Sets the value of the shift that should be employed if
+   the user did not specify one.
+
+   Collective on ST
+
+   Input Parameters:
++  st - the spectral transformation context
+-  defaultshift - the default value of the shift
+
+   Level: developer
+
+@*/
+PetscErrorCode STSetDefaultShift(ST st,PetscScalar defaultshift)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(st,ST_COOKIE,1);
+  st->defsigma = defaultshift;
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "STSetBalanceMatrix"
+/*@
+   STSetBalanceMatrix - Sets the diagonal matrix to be used for balancing.
+
+   Collective on ST and Vec
+
+   Input Parameters:
++  st - the spectral transformation context
+-  D  - the diagonal matrix (represented as a vector)
+
+   Notes:
+   If this matrix is set, STApply will effectively apply D*OP*D^{-1}.
+
+   Balancing is usually set via EPSSetBalance, but the advanced user may use
+   this function to bypass the usual balancing methods.
+   
+   Level: developer
+
+.seealso: EPSSetBalance(), STApply(), STGetBalanceMatrix()
+@*/
+PetscErrorCode STSetBalanceMatrix(ST st,Vec D)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(st,ST_COOKIE,1);
+  PetscValidHeaderSpecific(D,VEC_COOKIE,2);
+  PetscCheckSameComm(st,1,D,2);
+  ierr = PetscObjectReference((PetscObject)D);CHKERRQ(ierr);
+  if (st->D) {
+    ierr = VecDestroy(st->D); CHKERRQ(ierr);
+  }
+  st->D = D;
+  if (!st->wb) {
+    ierr = VecDuplicate(st->D,&st->wb); CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "STGetBalanceMatrix"
+/*@
+   STGetBalanceMatrix - Gets the balance matrix used by the spectral transformation.
+
+   Not collective, but vector is shared by all processors that share the ST
+
+   Input Parameter:
+.  st - the spectral transformation context
+
+   Output Parameter:
+.  D  - the diagonal matrix (represented as a vector)
+
+   Note:
+   If the matrix was not set, a null pointer will be returned.
+
+   Level: developer
+
+.seealso: STSetBalanceMatrix()
+@*/
+PetscErrorCode STGetBalanceMatrix(ST st,Vec *D)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(st,ST_COOKIE,1);
+  PetscValidPointer(D,2);
+  *D = st->D;
+  PetscFunctionReturn(0);
+}
+
 #undef __FUNCT__  
 #define __FUNCT__ "STSetOptionsPrefix"
 /*@C
@@ -442,16 +560,16 @@ PetscErrorCode STView(ST st,PetscViewer viewer)
     ierr = PetscViewerASCIIPrintf(viewer,"  shift: %g+%g i\n",PetscRealPart(st->sigma),PetscImaginaryPart(st->sigma));CHKERRQ(ierr);
 #endif
     switch (st->shift_matrix) {
-    case STMATMODE_COPY:
+    case ST_MATMODE_COPY:
       break;
-    case STMATMODE_INPLACE:
+    case ST_MATMODE_INPLACE:
       ierr = PetscViewerASCIIPrintf(viewer,"Shifting the matrix and unshifting at exit\n");CHKERRQ(ierr);
       break;
-    case STMATMODE_SHELL:
+    case ST_MATMODE_SHELL:
       ierr = PetscViewerASCIIPrintf(viewer,"Using a shell matrix\n");CHKERRQ(ierr);
       break;
     }
-    if (st->B && st->shift_matrix != STMATMODE_SHELL) { 
+    if (st->B && st->shift_matrix != ST_MATMODE_SHELL) { 
       switch (st->str) {
         case SAME_NONZERO_PATTERN:      str = "same nonzero pattern";break;
         case DIFFERENT_NONZERO_PATTERN: str = "different nonzero pattern";break;
