@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -137,12 +137,11 @@ PetscErrorCode STSetFromOptions(ST st)
 {
   PetscErrorCode ierr;
   PetscInt       i;
+  PetscScalar    s;
   char           type[256];
   PetscTruth     flg;
   const char     *mode_list[3] = { "copy", "inplace", "shell" };
   const char     *structure_list[3] = { "same", "different", "subset" };
-  PC             pc;
-  const char     *pctype;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(st,ST_COOKIE,1);
@@ -159,7 +158,8 @@ PetscErrorCode STSetFromOptions(ST st)
       ierr = STSetType(st,STSHIFT);CHKERRQ(ierr);
     }
 
-    ierr = PetscOptionsScalar("-st_shift","Value of the shift","STSetShift",st->sigma,&st->sigma,PETSC_NULL); CHKERRQ(ierr);
+    ierr = PetscOptionsScalar("-st_shift","Value of the shift","STSetShift",st->sigma,&s,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = STSetShift(st,s);CHKERRQ(ierr); }
 
     ierr = PetscOptionsEList("-st_matmode", "Shift matrix mode","STSetMatMode",mode_list,3,mode_list[st->shift_matrix],&i,&flg);CHKERRQ(ierr);
     if (flg) { st->shift_matrix = (STMatMode)i; }
@@ -173,22 +173,7 @@ PetscErrorCode STSetFromOptions(ST st)
 
   ierr = PetscOptionsEnd();CHKERRQ(ierr);
 
-  if (st->ksp) {   
-    ierr = KSPGetPC(st->ksp,&pc);CHKERRQ(ierr);
-    ierr = PCGetType(pc,&pctype);CHKERRQ(ierr);
-    if (!pctype) {
-      if (st->shift_matrix == STMATMODE_SHELL) {
-        /* in shell mode use GMRES with Jacobi as the default */
-        ierr = KSPSetType(st->ksp,KSPGMRES);CHKERRQ(ierr);
-        ierr = PCSetType(pc,PCJACOBI);CHKERRQ(ierr);
-      } else {
-        /* use direct solver as default */
-        ierr = KSPSetType(st->ksp,KSPPREONLY);CHKERRQ(ierr);
-        ierr = PCSetType(pc,PCREDUNDANT);CHKERRQ(ierr);
-      }
-    }
-    ierr = KSPSetFromOptions(st->ksp);CHKERRQ(ierr); 
-  }
+  ierr = KSPSetFromOptions(st->ksp);CHKERRQ(ierr); 
 
   PetscFunctionReturn(0);
 }
@@ -239,6 +224,36 @@ PetscErrorCode STSetMatStructure(ST st,MatStructure str)
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__  
+#define __FUNCT__ "STGetMatStructure"
+/*@
+   STGetMatStructure - Gets the internal MatStructure attribute to 
+   indicate which is the relation of the sparsity pattern of the two matrices
+   A and B constituting the generalized eigenvalue problem. This function
+   has no effect in the case of standard eigenproblems.
+
+   Collective on ST
+
+   Input Parameters:
+.  st  - the spectral transformation context
+
+   Output Parameters:
+.  str - either SAME_NONZERO_PATTERN, DIFFERENT_NONZERO_PATTERN or
+         SUBSET_NONZERO_PATTERN
+
+   Level: advanced
+
+.seealso: STSetMatStructure(), STSetOperators(), MatAXPY()
+@*/
+PetscErrorCode STGetMatStructure(ST st,MatStructure *str)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(st,ST_COOKIE,1);
+  PetscValidPointer(str,2);
+  *str = st->str;
+  PetscFunctionReturn(0);
+}
+
 #undef __FUNCT__  
 #define __FUNCT__ "STSetMatMode"
 /*@
@@ -249,24 +264,24 @@ PetscErrorCode STSetMatStructure(ST st,MatStructure str)
 
    Input Parameters:
 +  st - the spectral transformation context
--  mode - the mode flag, one of STMATMODE_COPY, 
-          STMATMODE_INPLACE or STMATMODE_SHELL
+-  mode - the mode flag, one of ST_MATMODE_COPY, 
+          ST_MATMODE_INPLACE or ST_MATMODE_SHELL
 
    Options Database Key:
 .  -st_matmode <mode> - Indicates the mode flag, where <mode> is one of
           'copy', 'inplace' or 'shell' (see explanation below).
 
    Notes:
-   By default (STMATMODE_COPY), a copy of matrix A is made and then 
+   By default (ST_MATMODE_COPY), a copy of matrix A is made and then 
    this copy is shifted explicitly, e.g. A <- (A - s B). 
 
-   With STMATMODE_INPLACE, the original matrix A is shifted at 
+   With ST_MATMODE_INPLACE, the original matrix A is shifted at 
    STSetUp() and unshifted at the end of the computations. With respect to
    the previous one, this mode avoids a copy of matrix A. However, a
    backdraw is that the recovered matrix might be slightly different 
    from the original one (due to roundoff).
 
-   With STMATMODE_SHELL, the solver works with an implicit shell 
+   With ST_MATMODE_SHELL, the solver works with an implicit shell 
    matrix that represents the shifted matrix. This mode is the most efficient 
    in creating the shifted matrix but it places serious limitations to the 
    linear solves performed in each iteration of the eigensolver (typically,
