@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -57,7 +57,14 @@ PetscErrorCode STApply(ST st,Vec x,Vec y)
 
   ierr = PetscLogEventBegin(ST_Apply,st,x,y,0);CHKERRQ(ierr);
   st->applys++;
-  ierr = (*st->ops->apply)(st,x,y);CHKERRQ(ierr);
+  if (st->D) { /* with balancing */
+    ierr = VecPointwiseDivide(st->wb,x,st->D);CHKERRQ(ierr);
+    ierr = (*st->ops->apply)(st,st->wb,y);CHKERRQ(ierr);
+    ierr = VecPointwiseMult(y,y,st->D);CHKERRQ(ierr);
+  }
+  else {
+    ierr = (*st->ops->apply)(st,x,y);CHKERRQ(ierr);
+  }
   ierr = PetscLogEventEnd(ST_Apply,st,x,y,0);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -65,7 +72,8 @@ PetscErrorCode STApply(ST st,Vec x,Vec y)
 #undef __FUNCT__  
 #define __FUNCT__ "STGetBilinearForm"
 /*@
-   STGetBilinearForm - Returns the matrix used in the bilinear form with a semi-definite generalised problem.
+   STGetBilinearForm - Returns the matrix used in the bilinear form with a 
+   generalized problem with semi-definite B.
 
    Collective on ST and Mat
 
@@ -138,7 +146,15 @@ PetscErrorCode STApplyTranspose(ST st,Vec x,Vec y)
   if (!st->setupcalled) { ierr = STSetUp(st); CHKERRQ(ierr); }
 
   ierr = PetscLogEventBegin(ST_ApplyTranspose,st,x,y,0);CHKERRQ(ierr);
-  ierr = (*st->ops->applytrans)(st,x,y);CHKERRQ(ierr);
+  st->applys++;
+  if (st->D) { /* with balancing */
+    ierr = VecPointwiseMult(st->wb,x,st->D);CHKERRQ(ierr);
+    ierr = (*st->ops->applytrans)(st,st->wb,y);CHKERRQ(ierr);
+    ierr = VecPointwiseDivide(y,y,st->D);CHKERRQ(ierr);
+  }
+  else {
+    ierr = (*st->ops->applytrans)(st,x,y);CHKERRQ(ierr);
+  }
   ierr = PetscLogEventEnd(ST_ApplyTranspose,st,x,y,0);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -293,14 +309,14 @@ PetscErrorCode STPostSolve(ST st)
 
 .seealso: EPSBackTransform()
 @*/
-PetscErrorCode STBackTransform(ST st,PetscScalar* eigr,PetscScalar* eigi)
+PetscErrorCode STBackTransform(ST st,PetscInt n,PetscScalar* eigr,PetscScalar* eigi)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(st,ST_COOKIE,1);
   if (st->ops->backtr) {
-    ierr = (*st->ops->backtr)(st,eigr,eigi);CHKERRQ(ierr);
+    ierr = (*st->ops->backtr)(st,n,eigr,eigi);CHKERRQ(ierr);
   }
 
   PetscFunctionReturn(0);
