@@ -8,7 +8,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -27,10 +27,12 @@
 */
 
 #include "private/svdimpl.h"                /*I "slepcsvd.h" I*/
+#include "private/epsimpl.h"                /*I "slepceps.h" I*/
 #include "slepceps.h"
 
 typedef struct {
   EPS eps;
+  PetscTruth setfromoptionscalled;
   Mat mat;
   Vec w,diag;
 } SVD_CROSS;
@@ -110,7 +112,8 @@ PetscErrorCode SVDSetUp_CROSS(SVD svd)
 {
   PetscErrorCode    ierr;
   SVD_CROSS         *cross = (SVD_CROSS *)svd->data;
-  PetscInt          n;
+  PetscInt          n,i;
+  PetscTruth        trackall;
 
   PetscFunctionBegin;
   if (cross->mat) { 
@@ -132,9 +135,25 @@ PetscErrorCode SVDSetUp_CROSS(SVD svd)
   ierr = EPSSetWhichEigenpairs(cross->eps,svd->which == SVD_LARGEST ? EPS_LARGEST_REAL : EPS_SMALLEST_REAL);CHKERRQ(ierr);
   ierr = EPSSetDimensions(cross->eps,svd->nsv,svd->ncv,svd->mpd);CHKERRQ(ierr);
   ierr = EPSSetTolerances(cross->eps,svd->tol,svd->max_it);CHKERRQ(ierr);
+  /* Transfer the trackall option from svd to eps */
+  ierr = SVDGetTrackAll(svd,&trackall);CHKERRQ(ierr);
+  ierr = EPSSetTrackAll(cross->eps,trackall);CHKERRQ(ierr);
+  if (cross->setfromoptionscalled) {
+    ierr = EPSSetFromOptions(cross->eps);CHKERRQ(ierr);
+    cross->setfromoptionscalled = PETSC_FALSE;
+  }
   ierr = EPSSetUp(cross->eps);CHKERRQ(ierr);
   ierr = EPSGetDimensions(cross->eps,PETSC_NULL,&svd->ncv,&svd->mpd);CHKERRQ(ierr);
   ierr = EPSGetTolerances(cross->eps,&svd->tol,&svd->max_it);CHKERRQ(ierr);
+  /* Transfer the initial space from svd to eps */
+  if (svd->nini < 0) {
+    ierr = EPSSetInitialSpace(cross->eps,-svd->nini,svd->IS);CHKERRQ(ierr);
+    for(i=0; i<-svd->nini; i++) {
+      ierr = VecDestroy(svd->IS[i]);CHKERRQ(ierr);
+    }
+    ierr = PetscFree(svd->IS);CHKERRQ(ierr);
+    svd->nini = 0;
+  }
   PetscFunctionReturn(0);
 }
 
@@ -148,7 +167,6 @@ PetscErrorCode SVDSolve_CROSS(SVD svd)
   PetscScalar    sigma;
   
   PetscFunctionBegin;
-  ierr = EPSSetInitialVector(cross->eps,svd->vec_initial);CHKERRQ(ierr);
   ierr = EPSSolve(cross->eps);CHKERRQ(ierr);
   ierr = EPSGetConverged(cross->eps,&svd->nconv);CHKERRQ(ierr);
   ierr = EPSGetIterationNumber(cross->eps,&svd->its);CHKERRQ(ierr);
@@ -164,12 +182,16 @@ PetscErrorCode SVDSolve_CROSS(SVD svd)
 #define __FUNCT__ "SVDMonitor_CROSS"
 PetscErrorCode SVDMonitor_CROSS(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,void *ctx)
 {
-  PetscInt  i;
-  SVD       svd = (SVD)ctx;
+  PetscInt       i;
+  SVD            svd = (SVD)ctx;
+  PetscScalar    er,ei;
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
   for (i=0;i<nest;i++) {
-    svd->sigma[i] = sqrt(PetscRealPart(eigr[i]));
+    er = eigr[i]; ei = eigi[i];
+    ierr = STBackTransform(eps->OP, 1, &er, &ei); CHKERRQ(ierr);
+    svd->sigma[i] = sqrt(PetscRealPart(er));
     svd->errest[i] = errest[i];
   }
   SVDMonitor(svd,its,nconv,svd->sigma,svd->errest,nest);
@@ -180,11 +202,10 @@ PetscErrorCode SVDMonitor_CROSS(EPS eps,PetscInt its,PetscInt nconv,PetscScalar
 #define __FUNCT__ "SVDSetFromOptions_CROSS"
 PetscErrorCode SVDSetFromOptions_CROSS(SVD svd)
 {
-  PetscErrorCode ierr;
   SVD_CROSS      *cross = (SVD_CROSS *)svd->data;
 
   PetscFunctionBegin;
-  ierr = EPSSetFromOptions(cross->eps);CHKERRQ(ierr);
+  cross->setfromoptionscalled = PETSC_TRUE;
   PetscFunctionReturn(0);
 }
 
@@ -252,7 +273,7 @@ EXTERN_C_END
 
 #undef __FUNCT__  
 #define __FUNCT__ "SVDCrossGetEPS"
-/*@C
+/*@
    SVDCrossGetEPS - Retrieve the eigensolver object (EPS) associated
    to the singular value solver.
 
@@ -310,6 +331,8 @@ PetscErrorCode SVDDestroy_CROSS(SVD svd)
     ierr = VecDestroy(cross->diag);CHKERRQ(ierr);
   }
   ierr = PetscFree(svd->data);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)svd,"SVDCrossSetEPS_C","",PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)svd,"SVDCrossGetEPS_C","",PETSC_NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -343,10 +366,11 @@ PetscErrorCode SVDCreate_CROSS(SVD svd)
   ierr = EPSSetWhichEigenpairs(cross->eps,EPS_LARGEST_REAL);CHKERRQ(ierr);
   ierr = EPSMonitorSet(cross->eps,SVDMonitor_CROSS,svd,PETSC_NULL);CHKERRQ(ierr);
   ierr = EPSGetST(cross->eps,&st);CHKERRQ(ierr);
-  ierr = STSetMatMode(st,STMATMODE_SHELL);CHKERRQ(ierr);
+  ierr = STSetMatMode(st,ST_MATMODE_SHELL);CHKERRQ(ierr);
   cross->mat = PETSC_NULL;
   cross->w = PETSC_NULL;
   cross->diag = PETSC_NULL;
+  cross->setfromoptionscalled = PETSC_FALSE;
   PetscFunctionReturn(0);
 }
 EXTERN_C_END
