@@ -8,7 +8,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -27,11 +27,13 @@
 */
 
 #include "private/svdimpl.h"                /*I "slepcsvd.h" I*/
+#include "private/epsimpl.h"                /*I "slepceps.h" I*/
 #include "slepceps.h"
 
 typedef struct {
   PetscTruth explicitmatrix;
   EPS        eps;
+  PetscTruth setfromoptionscalled;
   Mat        mat;
   Vec        x1,x2,y1,y2;
 } SVD_CYCLIC;
@@ -85,10 +87,13 @@ PetscErrorCode SVDSetUp_CYCLIC(SVD svd)
 {
   PetscErrorCode    ierr;
   SVD_CYCLIC        *cyclic = (SVD_CYCLIC *)svd->data;
-  PetscInt          M,N,m,n,i,j,start,end,ncols,*pos,nloc;
+  PetscInt          M,N,m,n,i,j,start,end,ncols,*pos,nloc,isl;
   const PetscInt    *cols;
   const PetscScalar *vals;
   PetscScalar       *pU;
+  PetscTruth        trackall;
+  Vec               v;
+  PetscScalar       *isa,*va;
 
   PetscFunctionBegin;
   
@@ -148,6 +153,42 @@ PetscErrorCode SVDSetUp_CYCLIC(SVD svd)
   ierr = EPSSetWhichEigenpairs(cyclic->eps,svd->which == SVD_LARGEST ? EPS_LARGEST_REAL : EPS_SMALLEST_MAGNITUDE);CHKERRQ(ierr);
   ierr = EPSSetDimensions(cyclic->eps,svd->nsv,svd->ncv,svd->mpd);CHKERRQ(ierr);
   ierr = EPSSetTolerances(cyclic->eps,svd->tol,svd->max_it);CHKERRQ(ierr);
+  /* Transfer the trackall option from svd to eps */
+  ierr = SVDGetTrackAll(svd,&trackall);CHKERRQ(ierr);
+  ierr = EPSSetTrackAll(cyclic->eps,trackall);CHKERRQ(ierr);
+  /* Transfer the initial subspace from svd to eps */
+  if (svd->nini < 0) {
+    for (i=0; i<-svd->nini; i++) {
+      ierr = MatGetVecs(cyclic->mat,&v,PETSC_NULL);CHKERRQ(ierr);
+      ierr = VecGetArray(v,&va);CHKERRQ(ierr);
+      ierr = VecGetArray(svd->IS[i],&isa);CHKERRQ(ierr);
+      ierr = VecGetSize(svd->IS[i],&isl);CHKERRQ(ierr);
+      if (isl == m) {
+        ierr = PetscMemcpy(va,isa,sizeof(PetscScalar)*m);CHKERRQ(ierr);
+        ierr = PetscMemzero(&va[m],sizeof(PetscScalar)*n);CHKERRQ(ierr);
+      } else if (isl == n) {
+        ierr = PetscMemzero(va,sizeof(PetscScalar)*m);CHKERRQ(ierr);
+        ierr = PetscMemcpy(&va[m],isa,sizeof(PetscScalar)*n);CHKERRQ(ierr);
+      } else {
+        SETERRQ(PETSC_ERR_SUP,"Size of the initial subspace vectors should match to some dimension of A");
+      }
+      ierr = VecRestoreArray(v,&va);CHKERRQ(ierr);
+      ierr = VecRestoreArray(svd->IS[i],&isa);CHKERRQ(ierr);
+      ierr = VecDestroy(svd->IS[i]);CHKERRQ(ierr);
+      svd->IS[i] = v;
+    }
+    ierr = EPSSetInitialSpace(cyclic->eps,-svd->nini,svd->IS);CHKERRQ(ierr);
+    for (i=0; i<-svd->nini; i++) {
+      ierr = VecDestroy(svd->IS[i]);CHKERRQ(ierr);
+    }
+    ierr = PetscFree(svd->IS);CHKERRQ(ierr);
+    svd->IS = PETSC_NULL;
+    svd->nini = 0;
+  }
+  if (cyclic->setfromoptionscalled) {
+    ierr = EPSSetFromOptions(cyclic->eps);CHKERRQ(ierr);
+    cyclic->setfromoptionscalled = PETSC_FALSE;
+  }
   ierr = EPSSetUp(cyclic->eps);CHKERRQ(ierr);
   ierr = EPSGetDimensions(cyclic->eps,PETSC_NULL,&svd->ncv,&svd->mpd);CHKERRQ(ierr);
   ierr = EPSGetTolerances(cyclic->eps,&svd->tol,&svd->max_it);CHKERRQ(ierr);
@@ -252,14 +293,18 @@ PetscErrorCode SVDSolve_CYCLIC(SVD svd)
 #define __FUNCT__ "SVDMonitor_CYCLIC"
 PetscErrorCode SVDMonitor_CYCLIC(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,void *ctx)
 {
-  PetscInt   i,j;
-  SVD        svd = (SVD)ctx;
+  PetscInt       i,j;
+  SVD            svd = (SVD)ctx;
+  PetscScalar    er,ei;
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
   nconv = 0;
   for (i=0,j=0;i<nest;i++) {
-    if (PetscRealPart(eigr[i]) > 0.0) {
-      svd->sigma[j] = PetscRealPart(eigr[i]);
+    er = eigr[i]; ei = eigi[i];
+    ierr = STBackTransform(eps->OP, 1, &er, &ei); CHKERRQ(ierr);
+    if (PetscRealPart(er) > 0.0) {
+      svd->sigma[j] = PetscRealPart(er);
       svd->errest[j] = errest[i];
       if (errest[i] < svd->tol) nconv++;
       j++;
@@ -275,12 +320,16 @@ PetscErrorCode SVDMonitor_CYCLIC(EPS eps,PetscInt its,PetscInt nconv,PetscScalar
 PetscErrorCode SVDSetFromOptions_CYCLIC(SVD svd)
 {
   PetscErrorCode ierr;
+  PetscTruth     set,val;
   SVD_CYCLIC     *cyclic = (SVD_CYCLIC *)svd->data;
   ST             st;
 
   PetscFunctionBegin;
   ierr = PetscOptionsBegin(((PetscObject)svd)->comm,((PetscObject)svd)->prefix,"CYCLIC Singular Value Solver Options","SVD");CHKERRQ(ierr);
-  ierr = PetscOptionsTruth("-svd_cyclic_explicitmatrix","Use cyclic explicit matrix","SVDCyclicSetExplicitMatrix",PETSC_FALSE,&cyclic->explicitmatrix,PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsTruth("-svd_cyclic_explicitmatrix","Use cyclic explicit matrix","SVDCyclicSetExplicitMatrix",cyclic->explicitmatrix,&val,&set);CHKERRQ(ierr);
+  if (set) {
+    ierr = SVDCyclicSetExplicitMatrix(svd,val);CHKERRQ(ierr);
+  }
   if (cyclic->explicitmatrix) {
     /* don't build the transpose */
     if (svd->transmode == PETSC_DECIDE)
@@ -288,11 +337,10 @@ PetscErrorCode SVDSetFromOptions_CYCLIC(SVD svd)
   } else {
     /* use as default an ST with shell matrix and Jacobi */ 
     ierr = EPSGetST(cyclic->eps,&st);CHKERRQ(ierr);
-    ierr = STSetMatMode(st,STMATMODE_SHELL);CHKERRQ(ierr);
+    ierr = STSetMatMode(st,ST_MATMODE_SHELL);CHKERRQ(ierr);
   }
   ierr = PetscOptionsEnd();CHKERRQ(ierr);
-  ierr = EPSSetFromOptions(cyclic->eps);CHKERRQ(ierr);
-  ierr = PetscOptionsTail();CHKERRQ(ierr);
+  cyclic->setfromoptionscalled = PETSC_TRUE;
   PetscFunctionReturn(0);
 }
 
@@ -357,7 +405,7 @@ EXTERN_C_END
 
 #undef __FUNCT__
 #define __FUNCT__ "SVDCyclicGetExplicitMatrix"
-/*@C
+/*@
    SVDCyclicGetExplicitMatrix - Returns the flag indicating if H(A) is built explicitly
 
    Not collective
@@ -449,7 +497,7 @@ EXTERN_C_END
 
 #undef __FUNCT__  
 #define __FUNCT__ "SVDCyclicGetEPS"
-/*@C
+/*@
    SVDCyclicGetEPS - Retrieve the eigensolver object (EPS) associated
    to the singular value solver.
 
@@ -512,6 +560,10 @@ PetscErrorCode SVDDestroy_CYCLIC(SVD svd)
     ierr = VecDestroy(cyclic->y2);CHKERRQ(ierr);
   }
   ierr = PetscFree(svd->data);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)svd,"SVDCyclicSetEPS_C","",PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)svd,"SVDCyclicGetEPS_C","",PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)svd,"SVDCyclicSetExplicitMatrix_C","",PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)svd,"SVDCyclicGetExplicitMatrix_C","",PETSC_NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -551,6 +603,7 @@ PetscErrorCode SVDCreate_CYCLIC(SVD svd)
   cyclic->x2 = PETSC_NULL;
   cyclic->y1 = PETSC_NULL;
   cyclic->y2 = PETSC_NULL;
+  cyclic->setfromoptionscalled = PETSC_FALSE;
   PetscFunctionReturn(0);
 }
 EXTERN_C_END
