@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -26,6 +26,25 @@
 PetscFList SVDList = 0;
 PetscCookie SVD_COOKIE = 0;
 PetscLogEvent SVD_SetUp = 0, SVD_Solve = 0, SVD_Dense = 0;
+static PetscTruth SVDPackageInitialized = PETSC_FALSE;
+
+#undef __FUNCT__  
+#define __FUNCT__ "SVDFinalizePackage"
+/*@C
+  SVDFinalizePackage - This function destroys everything in the Slepc interface to the SVD package. It is
+  called from SlepcFinalize().
+
+  Level: developer
+
+.seealso: SlepcFinalize()
+@*/
+PetscErrorCode SVDFinalizePackage(void) 
+{
+  PetscFunctionBegin;
+  SVDPackageInitialized = PETSC_FALSE;
+  SVDList               = 0;
+  PetscFunctionReturn(0);
+}
 
 #undef __FUNCT__  
 #define __FUNCT__ "SVDInitializePackage"
@@ -43,15 +62,14 @@ PetscLogEvent SVD_SetUp = 0, SVD_Solve = 0, SVD_Dense = 0;
 @*/
 PetscErrorCode SVDInitializePackage(char *path)
 {
-  static PetscTruth initialized = PETSC_FALSE;
   char              logList[256];
   char              *className;
   PetscTruth        opt;
   PetscErrorCode    ierr;
 
   PetscFunctionBegin;
-  if (initialized) PetscFunctionReturn(0);
-  initialized = PETSC_TRUE;
+  if (SVDPackageInitialized) PetscFunctionReturn(0);
+  SVDPackageInitialized = PETSC_TRUE;
   /* Register Classes */
   ierr = PetscCookieRegister("Singular Value Solver",&SVD_COOKIE);CHKERRQ(ierr);
   /* Register Constructors */
@@ -76,6 +94,7 @@ PetscErrorCode SVDInitializePackage(char *path)
       ierr = PetscLogEventDeactivateClass(SVD_COOKIE);CHKERRQ(ierr);
     }
   }
+  ierr = PetscRegisterFinalize(SVDFinalizePackage);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -149,6 +168,9 @@ PetscErrorCode SVDView(SVD svd,PetscViewer viewer)
     ierr = PetscViewerASCIIPrintf(viewer,"  maximum dimension of projected problem (mpd): %d\n",svd->mpd);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  maximum number of iterations: %d\n",svd->max_it);
     ierr = PetscViewerASCIIPrintf(viewer,"  tolerance: %g\n",svd->tol);CHKERRQ(ierr);
+    if (svd->nini!=0) {
+      ierr = PetscViewerASCIIPrintf(viewer,"  dimension of user-provided initial space: %d\n",PetscAbs(svd->nini));CHKERRQ(ierr);
+    }
     if (svd->ops->view) {
       ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
       ierr = (*svd->ops->view)(svd,viewer);CHKERRQ(ierr);
@@ -206,13 +228,14 @@ PetscErrorCode SVDCreate(MPI_Comm comm,SVD *outsvd)
   svd->perm        = PETSC_NULL;
   svd->U           = PETSC_NULL;
   svd->V           = PETSC_NULL;
-  svd->vec_initial = PETSC_NULL;
+  svd->IS          = PETSC_NULL;
   svd->which       = SVD_LARGEST;
   svd->n           = 0;
   svd->nconv       = 0;
   svd->nsv         = 1;    
   svd->ncv         = 0;    
   svd->mpd         = 0;    
+  svd->nini        = 0;
   svd->its         = 0;
   svd->max_it      = 0;  
   svd->tol         = 1e-7;    
@@ -222,6 +245,7 @@ PetscErrorCode SVDCreate(MPI_Comm comm,SVD *outsvd)
   svd->reason      = SVD_CONVERGED_ITERATING;
   svd->numbermonitors = 0;
   svd->matvecs = 0;
+  svd->trackall    = PETSC_FALSE;
 
   ierr = IPCreate(comm,&svd->ip);CHKERRQ(ierr);
   ierr = IPSetOptionsPrefix(svd->ip,((PetscObject)svd)->prefix);
@@ -285,10 +309,12 @@ PetscErrorCode SVDDestroy(SVD svd)
     ierr = PetscFree(p);CHKERRQ(ierr);
     ierr = PetscFree(svd->V);CHKERRQ(ierr);
   }
-  if (svd->vec_initial) { ierr = VecDestroy(svd->vec_initial);CHKERRQ(ierr); }
   ierr = SVDMonitorCancel(svd);CHKERRQ(ierr);
   
   ierr = IPDestroy(svd->ip);CHKERRQ(ierr);
+  if (svd->rand) {
+    ierr = PetscRandomDestroy(svd->rand);CHKERRQ(ierr);
+  }
   
   ierr = PetscHeaderDestroy(svd);CHKERRQ(ierr);
   PetscFunctionReturn(0);
