@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -29,7 +29,7 @@
    SVDSetTransposeMode - Sets how to handle the transpose of the matrix 
    associated with the singular value problem.
 
-   Collective on SVD and Mat
+   Collective on SVD
 
    Input Parameters:
 +  svd  - the singular value solver context
@@ -61,12 +61,10 @@ PetscErrorCode SVDSetTransposeMode(SVD svd,SVDTransposeMode mode)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
-  switch (mode) {
-    case PETSC_DEFAULT:
-      mode = (SVDTransposeMode)PETSC_DECIDE;
+  if (mode == PETSC_DEFAULT || mode == PETSC_DECIDE) mode = (SVDTransposeMode)PETSC_DECIDE;
+  else switch (mode) {
     case SVD_TRANSPOSE_EXPLICIT:
     case SVD_TRANSPOSE_IMPLICIT:
-    case PETSC_DECIDE:
       svd->transmode = mode;
       svd->setupcalled = 0;
       break;
@@ -79,7 +77,7 @@ PetscErrorCode SVDSetTransposeMode(SVD svd,SVDTransposeMode mode)
 #undef __FUNCT__  
 #define __FUNCT__ "SVDGetTransposeMode"
 /*@C
-   SVDGetTransposeMode - Gets the mode use to compute the  transpose 
+   SVDGetTransposeMode - Gets the mode used to compute the transpose 
    of the matrix associated with the singular value problem.
 
    Not collective
@@ -389,6 +387,7 @@ PetscErrorCode SVDSetFromOptions(SVD svd)
   PetscInt       i,j,k;
   PetscReal      r;
   PetscViewerASCIIMonitor monviewer;
+  SVDMONITOR_CONV *ctx;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
@@ -425,20 +424,40 @@ PetscErrorCode SVDSetFromOptions(SVD svd)
   ierr = PetscOptionsTruthGroupEnd("-svd_smallest","compute smallest singular values","SVDSetWhichSingularTriplets",&flg);CHKERRQ(ierr);
   if (flg) { ierr = SVDSetWhichSingularTriplets(svd,SVD_SMALLEST);CHKERRQ(ierr); }
 
-  ierr = PetscOptionsName("-svd_monitor_cancel","Remove any hardwired monitor routines","SVDMonitorCancel",&flg);CHKERRQ(ierr);
+  flg = PETSC_FALSE;
+  ierr = PetscOptionsTruth("-svd_monitor_cancel","Remove any hardwired monitor routines","SVDMonitorCancel",flg,&flg,PETSC_NULL);CHKERRQ(ierr);
   if (flg) {
     ierr = SVDMonitorCancel(svd);CHKERRQ(ierr); 
   }
 
-  ierr = PetscOptionsString("-svd_monitor","Monitor approximate singular values and error estimates","SVDMonitorSet","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr); 
+  ierr = PetscOptionsString("-svd_monitor_all","Monitor approximate singular values and error estimates","SVDMonitorSet","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr); 
   if (flg) {
     ierr = PetscViewerASCIIMonitorCreate(((PetscObject)svd)->comm,monfilename,((PetscObject)svd)->tablevel,&monviewer);CHKERRQ(ierr);
-    ierr = SVDMonitorSet(svd,SVDMonitorDefault,monviewer,(PetscErrorCode (*)(void*))PetscViewerASCIIMonitorDestroy);CHKERRQ(ierr);
+    ierr = SVDMonitorSet(svd,SVDMonitorAll,monviewer,(PetscErrorCode (*)(void*))PetscViewerASCIIMonitorDestroy);CHKERRQ(ierr);
+    ierr = SVDSetTrackAll(svd,PETSC_TRUE);CHKERRQ(ierr);
   }
-  ierr = PetscOptionsName("-svd_monitor_draw","Monitor error estimates graphically","SVDMonitorSet",&flg);CHKERRQ(ierr); 
+  ierr = PetscOptionsString("-svd_monitor_conv","Monitor approximate singular values and error estimates as they converge","SVDMonitorSet","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr); 
+  if (flg) {
+      ierr = PetscNew(SVDMONITOR_CONV,&ctx);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIMonitorCreate(((PetscObject)svd)->comm,monfilename,((PetscObject)svd)->tablevel,&ctx->viewer);CHKERRQ(ierr);
+      ierr = SVDMonitorSet(svd,SVDMonitorConverged,ctx,(PetscErrorCode (*)(void*))SVDMonitorDestroy_Converged);CHKERRQ(ierr);
+  }
+  ierr = PetscOptionsString("-svd_monitor","Monitor first unconverged approximate singular value and error estimate","SVDMonitorSet","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr); 
+  if (flg) {
+    ierr = PetscViewerASCIIMonitorCreate(((PetscObject)svd)->comm,monfilename,((PetscObject)svd)->tablevel,&monviewer);CHKERRQ(ierr);
+    ierr = SVDMonitorSet(svd,SVDMonitorFirst,monviewer,(PetscErrorCode (*)(void*))PetscViewerASCIIMonitorDestroy);CHKERRQ(ierr);
+  }
+  flg = PETSC_FALSE;
+  ierr = PetscOptionsTruth("-svd_monitor_draw","Monitor first unconverged approximate singular value and error estimate graphically","SVDMonitorSet",flg,&flg,PETSC_NULL);CHKERRQ(ierr); 
   if (flg) {
     ierr = SVDMonitorSet(svd,SVDMonitorLG,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
   }
+  flg = PETSC_FALSE;
+  ierr = PetscOptionsTruth("-svd_monitor_draw_all","Monitor error estimates graphically","SVDMonitorSet",flg,&flg,PETSC_NULL);CHKERRQ(ierr); 
+  if (flg) {
+    ierr = SVDMonitorSet(svd,SVDMonitorLGAll,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
+    ierr = SVDSetTrackAll(svd,PETSC_TRUE);CHKERRQ(ierr);
+  }
 
   ierr = PetscOptionsEnd();CHKERRQ(ierr);
   ierr = IPSetFromOptions(svd->ip);CHKERRQ(ierr);
@@ -448,6 +467,67 @@ PetscErrorCode SVDSetFromOptions(SVD svd)
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__  
+#define __FUNCT__ "SVDSetTrackAll"
+/*@
+    SVDSetTrackAll - Specifies if the solver must compute the residual norm of all
+    approximate singular value or not.
+
+    Collective on SVD
+
+    Input Parameters:
++   svd      - the singular value solver context
+-   trackall - whether to compute all residuals or not
+
+    Notes:
+    If the user sets trackall=PETSC_TRUE then the solver computes (or estimates)
+    the residual norm for each singular value approximation. Computing the residual is
+    usually an expensive operation and solvers commonly compute only the residual 
+    associated to the first unconverged singular value.
+
+    The options '-svd_monitor_all' and '-svd_monitor_draw_all' automatically
+    activate this option.
+
+    Level: intermediate
+
+.seealso: SVDGetTrackAll()
+@*/
+PetscErrorCode SVDSetTrackAll(SVD svd,PetscTruth trackall)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  svd->trackall = trackall;
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "SVDGetTrackAll"
+/*@
+    SVDGetTrackAll - Returns the flag indicating whether all residual norms must
+    be computed or not.
+
+    Not Collective
+
+    Input Parameter:
+.   svd - the singular value solver context
+
+    Output Parameter:
+.   trackall - the returned flag
+
+    Level: intermediate
+
+.seealso: SVDSetTrackAll()
+@*/
+PetscErrorCode SVDGetTrackAll(SVD svd,PetscTruth *trackall) 
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  PetscValidPointer(trackall,2);
+  *trackall = svd->trackall;
+  PetscFunctionReturn(0);
+}
+
+
 #undef __FUNCT__  
 #define __FUNCT__ "SVDSetOptionsPrefix"
 /*@C
