@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -51,10 +51,6 @@ PetscErrorCode SVDSetOperator(SVD svd,Mat mat)
     ierr = MatDestroy(svd->OP);CHKERRQ(ierr);
   }
   svd->OP = mat;
-  if (svd->vec_initial) {
-    ierr = VecDestroy(svd->vec_initial);CHKERRQ(ierr);
-    svd->vec_initial = PETSC_NULL;
-  }
   svd->setupcalled = 0;
   PetscFunctionReturn(0);
 }
@@ -85,68 +81,6 @@ PetscErrorCode SVDGetOperator(SVD svd,Mat *A)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "SVDSetInitialVector"
-/*@
-   SVDSetInitialVector - Sets the initial vector from which the 
-   singular value solver starts to iterate.
-
-   Collective on SVD and Vec
-
-   Input Parameters:
-+  svd - the singular value solver context
--  vec - the vector
-
-   Level: intermediate
-
-.seealso: SVDGetInitialVector()
-
-@*/
-PetscErrorCode SVDSetInitialVector(SVD svd,Vec vec)
-{
-  PetscErrorCode ierr;
-  
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
-  PetscValidHeaderSpecific(vec,VEC_COOKIE,2);
-  PetscCheckSameComm(svd,1,vec,2);
-  ierr = PetscObjectReference((PetscObject)vec);CHKERRQ(ierr);
-  if (svd->vec_initial) {
-    ierr = VecDestroy(svd->vec_initial); CHKERRQ(ierr);
-  }
-  svd->vec_initial = vec;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__  
-#define __FUNCT__ "SVDGetInitialVector"
-/*@
-   SVDGetInitialVector - Gets the initial vector associated with the 
-   singular value solver; if the vector was not set it will return a 0 
-   pointer or a vector randomly generated by SVDSetUp().
-
-   Not collective, but vector is shared by all processors that share the SVD
-
-   Input Parameter:
-.  svd - the singular value solver context
-
-   Output Parameter:
-.  vec - the vector
-
-   Level: intermediate
-
-.seealso: SVDSetInitialVector()
-
-@*/
-PetscErrorCode SVDGetInitialVector(SVD svd,Vec *vec)
-{
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
-  PetscValidPointer(vec,2);
-  *vec = svd->vec_initial;
-  PetscFunctionReturn(0);
-}
-
 #undef __FUNCT__  
 #define __FUNCT__ "SVDSetUp"
 /*@
@@ -170,9 +104,10 @@ PetscErrorCode SVDGetInitialVector(SVD svd,Vec *vec)
 PetscErrorCode SVDSetUp(SVD svd)
 {
   PetscErrorCode ierr;
-  PetscTruth     flg;
-  PetscInt       i,M,N,nloc;
+  PetscTruth     flg,lindep;
+  PetscInt       i,k,M,N,nloc;
   PetscScalar    *pV;
+  PetscReal      norm;
   
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
@@ -227,11 +162,9 @@ PetscErrorCode SVDSetUp(SVD svd)
       SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Invalid transpose mode"); 
   }
 
-  /* set initial vector */
-  if (!svd->vec_initial) {
-    ierr = SVDMatGetVecs(svd,&svd->vec_initial,PETSC_NULL);CHKERRQ(ierr);
-    ierr = SlepcVecSetRandom(svd->vec_initial);CHKERRQ(ierr);
-  }
+  /* initialize the random number generator */
+  ierr = PetscRandomCreate(((PetscObject)svd)->comm,&svd->rand);CHKERRQ(ierr);
+  ierr = PetscRandomSetFromOptions(svd->rand);CHKERRQ(ierr);
 
   /* call specific solver setup */
   ierr = (*svd->ops->setup)(svd);CHKERRQ(ierr);
@@ -256,10 +189,14 @@ PetscErrorCode SVDSetUp(SVD svd)
     }
     /* allocate memory for next solution */
     ierr = PetscMalloc(svd->ncv*sizeof(PetscReal),&svd->sigma);CHKERRQ(ierr);
-    ierr = PetscMalloc(svd->ncv*sizeof(PetscReal),&svd->perm);CHKERRQ(ierr);
+    ierr = PetscMalloc(svd->ncv*sizeof(PetscInt),&svd->perm);CHKERRQ(ierr);
     ierr = PetscMalloc(svd->ncv*sizeof(PetscReal),&svd->errest);CHKERRQ(ierr);
     ierr = PetscMalloc(svd->ncv*sizeof(Vec),&svd->V);CHKERRQ(ierr);
-    ierr = VecGetLocalSize(svd->vec_initial,&nloc);CHKERRQ(ierr);
+    if (svd->A) {
+      ierr = MatGetLocalSize(svd->A,PETSC_NULL,&nloc);CHKERRQ(ierr);
+    } else {
+      ierr = MatGetLocalSize(svd->AT,&nloc,PETSC_NULL);CHKERRQ(ierr);
+    }
     ierr = PetscMalloc(svd->ncv*nloc*sizeof(PetscScalar),&pV);CHKERRQ(ierr);
     for (i=0;i<svd->ncv;i++) {
       ierr = VecCreateMPIWithArray(((PetscObject)svd)->comm,nloc,PETSC_DECIDE,pV+i*nloc,&svd->V[i]);CHKERRQ(ierr);
@@ -267,7 +204,84 @@ PetscErrorCode SVDSetUp(SVD svd)
     svd->n = svd->ncv;
   }
 
+  /* process initial vectors */
+  if (svd->nini<0) {
+    svd->nini = -svd->nini;
+    if (svd->nini>svd->ncv) SETERRQ(1,"The number of initial vectors is larger than ncv")
+    k = 0;
+    for (i=0;i<svd->nini;i++) {
+      ierr = VecCopy(svd->IS[i],svd->V[k]);CHKERRQ(ierr);
+      ierr = VecDestroy(svd->IS[i]);CHKERRQ(ierr);
+      ierr = IPOrthogonalize(svd->ip,0,PETSC_NULL,k,PETSC_NULL,svd->V,svd->V[k],PETSC_NULL,&norm,&lindep);CHKERRQ(ierr); 
+      if (norm==0.0 || lindep) PetscInfo(svd,"Linearly dependent initial vector found, removing...\n");
+      else {
+        ierr = VecScale(svd->V[k],1.0/norm);CHKERRQ(ierr);
+        k++;
+      }
+    }
+    svd->nini = k;
+    ierr = PetscFree(svd->IS);CHKERRQ(ierr);
+  }
+
   ierr = PetscLogEventEnd(SVD_SetUp,svd,0,0,0);CHKERRQ(ierr);
   svd->setupcalled = 1;
   PetscFunctionReturn(0);
 }
+
+#undef __FUNCT__  
+#define __FUNCT__ "SVDSetInitialSpace"
+/*@
+   SVDSetInitialSpace - Specify a basis of vectors that constitute the initial
+   space, that is, the subspace from which the solver starts to iterate.
+
+   Collective on SVD and Vec
+
+   Input Parameter:
++  svd   - the singular value solver context
+.  n     - number of vectors
+-  is    - set of basis vectors of the initial space
+
+   Notes:
+   Some solvers start to iterate on a single vector (initial vector). In that case,
+   the other vectors are ignored.
+
+   These vectors do not persist from one SVDSolve() call to the other, so the
+   initial space should be set every time.
+
+   The vectors do not need to be mutually orthonormal, since they are explicitly
+   orthonormalized internally.
+
+   Common usage of this function is when the user can provide a rough approximation
+   of the wanted singular space. Then, convergence may be faster.
+
+   Level: intermediate
+@*/
+PetscErrorCode SVDSetInitialSpace(SVD svd,PetscInt n,Vec *is)
+{
+  PetscErrorCode ierr;
+  PetscInt       i;
+  
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  if (n<0) SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Argument n cannot be negative"); 
+
+  /* free previous non-processed vectors */
+  if (svd->nini<0) {
+    for (i=0;i<-svd->nini;i++) {
+      ierr = VecDestroy(svd->IS[i]);CHKERRQ(ierr);
+    }
+    ierr = PetscFree(svd->IS);CHKERRQ(ierr);
+  }
+
+  /* get references of passed vectors */
+  ierr = PetscMalloc(n*sizeof(Vec),&svd->IS);CHKERRQ(ierr);
+  for (i=0;i<n;i++) {
+    ierr = PetscObjectReference((PetscObject)is[i]);CHKERRQ(ierr);
+    svd->IS[i] = is[i];
+  }
+
+  svd->nini = -n;
+  svd->setupcalled = 0;
+  PetscFunctionReturn(0);
+}
+
