@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -45,6 +45,8 @@ PetscErrorCode SVDSolve(SVD svd)
   PetscErrorCode ierr;
   PetscTruth     flg;
   PetscInt       i,*workperm;
+  char           filename[PETSC_MAX_PATH_LEN];
+  PetscViewer    viewer;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
@@ -74,8 +76,15 @@ PetscErrorCode SVDSolve(SVD svd)
     ierr = PetscFree(workperm);CHKERRQ(ierr);
   }
 
-  ierr = PetscOptionsHasName(((PetscObject)svd)->prefix,"-svd_view",&flg);CHKERRQ(ierr); 
-  if (flg && !PetscPreLoadingOn) { ierr = SVDView(svd,PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr); }
+  ierr = PetscOptionsGetString(((PetscObject)svd)->prefix,"-svd_view",filename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
+  if (flg && !PetscPreLoadingOn) {
+    ierr = PetscViewerASCIIOpen(((PetscObject)svd)->comm,filename,&viewer);CHKERRQ(ierr);
+    ierr = SVDView(svd,viewer);CHKERRQ(ierr); 
+    ierr = PetscViewerDestroy(viewer);CHKERRQ(ierr);
+  }
+
+  /* Remove the initial subspace */
+  svd->nini = 0;
 
   PetscFunctionReturn(0);
 }
@@ -210,8 +219,9 @@ PetscErrorCode SVDGetSingularTriplet(SVD svd, PetscInt i, PetscReal *sigma, Vec
 {
   PetscErrorCode ierr;
   PetscReal      norm;
-  PetscInt       j,nloc;
+  PetscInt       j,nloc,M,N;
   PetscScalar    *pU;
+  Vec            w;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
@@ -223,18 +233,20 @@ PetscErrorCode SVDGetSingularTriplet(SVD svd, PetscInt i, PetscReal *sigma, Vec
     SETERRQ(PETSC_ERR_ARG_OUTOFRANGE, "Argument 2 out of range"); 
   }
   *sigma = svd->sigma[svd->perm[i]];
+  ierr = MatGetSize(svd->OP,&M,&N);CHKERRQ(ierr);
+  if (M<N) { w = u; u = v; v = w; }
   if (u) {
     PetscValidHeaderSpecific(u,VEC_COOKIE,4);
     if (!svd->U) {
       ierr = PetscMalloc(sizeof(Vec)*svd->ncv,&svd->U);CHKERRQ(ierr);
       ierr = SVDMatGetLocalSize(svd,&nloc,PETSC_NULL);CHKERRQ(ierr);
       ierr = PetscMalloc(svd->ncv*nloc*sizeof(PetscScalar),&pU);CHKERRQ(ierr);
-      for (i=0;i<svd->ncv;i++) {
-        ierr = VecCreateMPIWithArray(((PetscObject)svd)->comm,nloc,PETSC_DECIDE,pU+i*nloc,&svd->U[i]);CHKERRQ(ierr);
+      for (j=0;j<svd->ncv;j++) {
+        ierr = VecCreateMPIWithArray(((PetscObject)svd)->comm,nloc,PETSC_DECIDE,pU+j*nloc,&svd->U[j]);CHKERRQ(ierr);
       }
       for (j=0;j<svd->nconv;j++) {
         ierr = SVDMatMult(svd,PETSC_FALSE,svd->V[j],svd->U[j]);CHKERRQ(ierr);
-        ierr = IPOrthogonalize(svd->ip,j,PETSC_NULL,svd->U,svd->U[j],PETSC_NULL,&norm,PETSC_NULL,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
+        ierr = IPOrthogonalize(svd->ip,0,PETSC_NULL,j,PETSC_NULL,svd->U,svd->U[j],PETSC_NULL,&norm,PETSC_NULL);CHKERRQ(ierr);
         ierr = VecScale(svd->U[j],1.0/norm);CHKERRQ(ierr);
       }
     }
@@ -277,6 +289,7 @@ PetscErrorCode SVDComputeResidualNorms(SVD svd, PetscInt i, PetscReal *norm1, Pe
   PetscErrorCode ierr;
   Vec            u,v,x = PETSC_NULL,y = PETSC_NULL;
   PetscReal      sigma;
+  PetscInt       M,N;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
@@ -287,17 +300,26 @@ PetscErrorCode SVDComputeResidualNorms(SVD svd, PetscInt i, PetscReal *norm1, Pe
     SETERRQ(PETSC_ERR_ARG_OUTOFRANGE, "Argument 2 out of range"); 
   }
   
-  ierr = SVDMatGetVecs(svd,&v,&u);CHKERRQ(ierr);
+  ierr = MatGetVecs(svd->OP,&v,&u);CHKERRQ(ierr);
   ierr = SVDGetSingularTriplet(svd,i,&sigma,u,v);CHKERRQ(ierr);
   if (norm1) {
     ierr = VecDuplicate(u,&x);CHKERRQ(ierr);
-    ierr = SVDMatMult(svd,PETSC_FALSE,v,x);CHKERRQ(ierr);
+    ierr = MatMult(svd->OP,v,x);CHKERRQ(ierr);
     ierr = VecAXPY(x,-sigma,u);CHKERRQ(ierr);
     ierr = VecNorm(x,NORM_2,norm1);CHKERRQ(ierr);
   }
   if (norm2) {
     ierr = VecDuplicate(v,&y);CHKERRQ(ierr);
-    ierr = SVDMatMult(svd,PETSC_TRUE,u,y);CHKERRQ(ierr);
+    if (svd->A && svd->AT) {
+      ierr = MatGetSize(svd->OP,&M,&N);CHKERRQ(ierr);
+      if (M<N) {
+        ierr = MatMult(svd->A,u,y);CHKERRQ(ierr);
+      } else {
+        ierr = MatMult(svd->AT,u,y);CHKERRQ(ierr);
+      }
+    } else {
+      ierr = MatMultTranspose(svd->OP,u,y);CHKERRQ(ierr);
+    }
     ierr = VecAXPY(y,-sigma,v);CHKERRQ(ierr);
     ierr = VecNorm(y,NORM_2,norm2);CHKERRQ(ierr);
   }
