@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -22,15 +22,16 @@
 */
 
 #include "private/fortranimpl.h" 
-#include "slepc.h"
-#include "slepcst.h"
-#include "slepceps.h"
-#include "slepcsvd.h"
-#include "slepcip.h"
+#include "slepcsys.h"
+#include "private/epsimpl.h"
+#include "private/stimpl.h"
+#include "private/svdimpl.h"
+#include "private/ipimpl.h"
+#include <stdlib.h>
 
 extern PetscTruth SlepcBeganPetsc;
-
 extern PetscTruth SlepcInitializeCalled;
+extern PetscLogEvent SLEPC_UpdateVectors;
 
 #ifdef PETSC_HAVE_FORTRAN_CAPS
 #define petscinitialize_              PETSCINITIALIZE
@@ -92,6 +93,13 @@ void PETSC_STDCALL slepcinitialize_(CHAR filename PETSC_MIXED_LEN(len),PetscErro
   *ierr = IPInitializePackage(PETSC_NULL); if (*ierr) { (*PetscErrorPrintf)("SlepcInitialize:Initializing IP package");return;}
 #endif
 
+  *ierr = PetscLogEventRegister("UpdateVectors",0,&SLEPC_UpdateVectors); if (*ierr) { (*PetscErrorPrintf)("SlepcInitialize:Registering log event");return;}
+
+#if defined(PETSC_HAVE_DRAND48)
+  /* work-around for Cygwin drand48() initialization bug */
+  srand48(0);
+#endif
+
   SlepcInitializeCalled = PETSC_TRUE;
   *ierr = PetscInfo(0,"SLEPc successfully started from Fortran\n");
   if (*ierr) { (*PetscErrorPrintf)("SlepcInitialize:Calling PetscInfo()");return;}
