@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -19,11 +19,12 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "slepc.h" /*I "slepc.h" I*/
-#include "slepceps.h"
-#include "slepcst.h"
-#include "slepcsvd.h"
-#include "slepcip.h"
+#include "slepcsys.h"           /*I "slepcsys.h" I*/
+#include "private/epsimpl.h"
+#include "private/stimpl.h"
+#include "private/svdimpl.h"
+#include "private/qepimpl.h"
+#include "private/ipimpl.h"
 #include <stdlib.h>
 
 #undef __FUNCT__  
@@ -88,7 +89,8 @@ PetscErrorCode SlepcPrintHelpIntro(MPI_Comm comm)
 */
 PetscTruth  SlepcBeganPetsc = PETSC_FALSE; 
 PetscTruth  SlepcInitializeCalled = PETSC_FALSE;
-extern PetscLogEvent SLEPC_UpdateVectors;
+extern PetscLogEvent SLEPC_UpdateVectors, SLEPC_VecMAXPBY, SLEPC_SlepcDenseMatProd, SLEPC_SlepcDenseOrth,
+                     SLEPC_SlepcDenseNorm, SLEPC_SlepcDenseCopy, SLEPC_VecsMult;
 
 #if defined(PETSC_USE_DYNAMIC_LIBRARIES)
 extern PetscDLLibrary DLLibrariesLoaded;
@@ -158,10 +160,17 @@ PetscErrorCode SlepcInitialize(int *argc,char ***args,char file[],const char hel
   ierr = STInitializePackage(PETSC_NULL); CHKERRQ(ierr);
   ierr = EPSInitializePackage(PETSC_NULL); CHKERRQ(ierr);
   ierr = SVDInitializePackage(PETSC_NULL); CHKERRQ(ierr);
+  ierr = QEPInitializePackage(PETSC_NULL); CHKERRQ(ierr);
   ierr = IPInitializePackage(PETSC_NULL); CHKERRQ(ierr);
 #endif
 
   ierr = PetscLogEventRegister("UpdateVectors",0,&SLEPC_UpdateVectors);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecMAXPBY",0,&SLEPC_VecMAXPBY);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("DenseMatProd", EPS_COOKIE, &SLEPC_SlepcDenseMatProd); CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("DenseOrth", EPS_COOKIE, &SLEPC_SlepcDenseOrth); CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("DenseMatNorm", EPS_COOKIE, &SLEPC_SlepcDenseNorm); CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("DenseCopy", EPS_COOKIE, &SLEPC_SlepcDenseCopy); CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecsMult", EPS_COOKIE, &SLEPC_VecsMult); CHKERRQ(ierr);
 
 #if defined(PETSC_HAVE_DRAND48)
   /* work-around for Cygwin drand48() initialization bug */
@@ -228,6 +237,7 @@ PetscErrorCode PetscDLLibraryRegister_slepc(char *path)
   ierr = STInitializePackage(path); CHKERRQ(ierr);
   ierr = EPSInitializePackage(path); CHKERRQ(ierr);
   ierr = SVDInitializePackage(path); CHKERRQ(ierr);
+  ierr = QEPInitializePackage(path); CHKERRQ(ierr);
   ierr = IPInitializePackage(path); CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
